@@ -456,6 +456,14 @@ $cached_env_img{$key} = q|<IMG
  SRC="|."$dir".q|img112.png"
  ALT="$\displaystyle r+Z_R+k_R(V-R)$">|; 
 
+$key = q/{displaymath}widetilde{Z}=widetilde{Z}+sum_k(y_k-widetilde{Z})W'_k{displaymath};MSF=1.6;LFS=11;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="181" HEIGHT="48" BORDER="0"
+ SRC="|."$dir".q|img185.png"
+ ALT="\begin{displaymath}
+\widetilde{Z}=\widetilde{Z} + \sum_k(y_k-\widetilde{Z})W'_k
+\end{displaymath}">|; 
+
 $key = q/S;MSF=1.6;LFS=11;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="16" HEIGHT="15" ALIGN="BOTTOM" BORDER="0"
@@ -872,6 +880,12 @@ $cached_env_img{$key} = q|<IMG
  SRC="|."$dir".q|img9.png"
  ALT="$I(x,y)$">|; 
 
+$key = q/D_M^F(x,y);MSF=1.6;LFS=11;AAT/;
+$cached_env_img{$key} = q|<IMG
+ WIDTH="74" HEIGHT="39" ALIGN="MIDDLE" BORDER="0"
+ SRC="|."$dir".q|img41.png"
+ ALT="$D_M^F(x,y)$">|; 
+
 $key = q/{displaymath}epsilon_i=2.51188logleft(1+frac{sqrt{sigma_n^2+sigma_{{rm{sc}^2}}{F}right).{displaymath};MSF=1.6;LFS=11;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="264" HEIGHT="54" BORDER="0"
@@ -880,12 +894,6 @@ $cached_env_img{$key} = q|<IMG
 \epsilon_i = 2.51188 \log\left(1 + \frac{\sqrt{\sigma_n^2 + \sigma_{\rm sc}^2}}{F}\right).
 \end{displaymath}">|; 
 
-$key = q/D_M^F(x,y);MSF=1.6;LFS=11;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="74" HEIGHT="39" ALIGN="MIDDLE" BORDER="0"
- SRC="|."$dir".q|img41.png"
- ALT="$D_M^F(x,y)$">|; 
-
 $key = q/A_1;MSF=1.6;LFS=11;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="25" HEIGHT="32" ALIGN="MIDDLE" BORDER="0"
@@ -1233,14 +1241,6 @@ $cached_env_img{$key} = q|<IMG
  SRC="|."$dir".q|img154.png"
  ALT="$\sigma_f$">|; 
 
-$key = q/{displaymath}widetilde{Z}=sum_k(y_k-widetilde{Z})W'_k{displaymath};MSF=1.6;LFS=11;AAT/;
-$cached_env_img{$key} = q|<IMG
- WIDTH="147" HEIGHT="48" BORDER="0"
- SRC="|."$dir".q|img185.png"
- ALT="\begin{displaymath}
-\widetilde{Z}=\sum_k(y_k-\widetilde{Z})W'_k
-\end{displaymath}">|; 
-
 $key = q/sigma_r';MSF=1.6;LFS=11;AAT/;
 $cached_env_img{$key} = q|<IMG
  WIDTH="22" HEIGHT="35" ALIGN="MIDDLE" BORDER="0"
