@@ -137,7 +137,8 @@ static int gsc_search(struct cat_star *cst[], struct catalog *cat,
 		cats->mag = tc[i].mag;
 		strcpy(cats->name, tc[i].name);
 		cats->flags = CAT_STAR_TYPE_SREF | CAT_ASTROMET;
-		asprintf(&cats->comments, "p=G ");
+		if (-1 ==  asprintf(&cats->comments, "p=G "))
+			cats->comments = NULL;
 		cst[i] = cats;
 	}  
 	free(regs);
@@ -256,19 +257,23 @@ static int tycho2_cat_search(struct cat_star *cst[], struct catalog *cat,
 				verr = 0.02;
 			if (bterr < 0.02)
 				berr = 0.02;
-			asprintf(&cats->smags, 
-				 "v=%.3f/%.3f b=%.3f/%.3f vt=%.3f/%.3f bt=%.3f/%.3f",
-				 vt - 0.090 * (bt - vt), verr,
-				 0.850 * (bt - vt) + vt - 0.090 * (bt - vt), 
-				 berr, vt, vterr, bt, bterr);
-			asprintf(&cats->comments, "p=T ");
+			if (-1 == asprintf(&cats->smags, 
+			                   "v=%.3f/%.3f b=%.3f/%.3f vt=%.3f/%.3f bt=%.3f/%.3f",
+			                   vt - 0.090 * (bt - vt), verr,
+			                   0.850 * (bt - vt) + vt - 0.090 * (bt - vt), 
+			                   berr, vt, vterr, bt, bterr))
+				cats->smags = NULL;
+			if (-1 == asprintf(&cats->comments, "p=T "))
+				cats->comments = NULL;
 			cats->flags = CAT_ASTROMET | CAT_STAR_TYPE_SREF;
 		} else if (vt < 30.0) {
 			cats->mag = vt;
-			asprintf(&cats->smags, "vt=%.2f/%.3f", vt, vterr);
+			if (-1 == asprintf(&cats->smags, "vt=%.2f/%.3f", vt, vterr))
+				cats->smags = NULL;
 		} else {
 			cats->mag = bt;
-			asprintf(&cats->smags, "bt=%.2f/%.3f", bt, bterr);
+			if (-1 == asprintf(&cats->smags, "bt=%.2f/%.3f", bt, bterr))
+				cats->smags = NULL;
 		}
 		st[i] = cats;
 		p+= TYCRECSZ + 1;
@@ -417,27 +422,39 @@ static void update_cat_star(struct cat_star *ocats, struct cat_star *cats)
 	strcpy(ocats->name, cats->name);
 	if (cats->comments != NULL) {
 		if (ocats->comments != NULL) {
-			asprintf(&t, "%s %s", cats->comments, ocats->comments);
-			free(ocats->comments);
-			ocats->comments = t;
+			if (-1 == asprintf(&t, "%s %s", cats->comments, ocats->comments)) {
+				free(ocats->comments);
+				ocats->comments = NULL;
+			} else {
+				free(ocats->comments);
+				ocats->comments = t;
+			}
 		} else {
 			ocats->comments = strdup(cats->comments);
 		}
 	}
 	if (cats->smags != NULL) {
 		if (ocats->smags != NULL) {
-			asprintf(&t, "%s %s", cats->smags, ocats->smags);
-			free(ocats->smags);
-			ocats->smags = t;
+			if (-1 == asprintf(&t, "%s %s", cats->smags, ocats->smags)) {
+				free(ocats->smags);
+				ocats->smags = NULL;
+			} else {
+				free(ocats->smags);
+				ocats->smags = t;
+			}
 		} else {
 			ocats->smags = strdup(cats->smags);
 		}
 	}
 	if (cats->imags != NULL) {
 		if (ocats->imags != NULL) {
-			asprintf(&t, "%s %s", cats->imags, ocats->imags);
-			free(ocats->imags);
-			ocats->imags = t;
+			if (-1 == asprintf(&t, "%s %s", cats->imags, ocats->imags)) {
+				free(ocats->imags);
+				ocats->imags = NULL;
+			} else {
+				free(ocats->imags);
+				cats->imags = t;
+			}
 		} else {
 			ocats->imags = strdup(cats->imags);
 		}
@@ -685,7 +702,7 @@ void cat_star_release(struct cat_star *cats)
 	}
 }
 
-/* we dont really need a close, but may later */
+/* we don't really need a close, but may later */
 void close_catalog(struct catalog *cat)
 {
 }
@@ -932,6 +949,7 @@ int update_band_by_name(char **mags, char *band, double mag, double err)
 	char *bn1, *bn2=NULL, *bqual;
 	int n1l, n2l, quall;
 	int bn1l, bn2l, bquall;
+	int ret;
 
 	if (band == NULL)
 		return -1;
@@ -940,11 +958,13 @@ int update_band_by_name(char **mags, char *band, double mag, double err)
 
 	if (*mags == NULL || *mags[0] == 0) {
 		if (err == 0.0)
-			asprintf(&nb, "%s=%.3f", band, mag);
+			ret = asprintf(&nb, "%s=%.3f", band, mag);
 		else 
-			asprintf(&nb, "%s=%.3f/%.3f", band, mag, err);
+			ret = asprintf(&nb, "%s=%.3f/%.3f", band, mag, err);
 		if (*mags)
 			free(*mags);
+		if (ret == -1)
+			nb = NULL;
 		*mags = nb;
 		return 0;
 	}
@@ -973,21 +993,27 @@ int update_band_by_name(char **mags, char *band, double mag, double err)
 
 	if (bs == NULL) {
 		if (err == 0.0)
-			asprintf(&nb, "%s %s=%.3f", *mags, band, mag);
+			ret = asprintf(&nb, "%s %s=%.3f", *mags, band, mag);
 		else 
-			asprintf(&nb, "%s %s=%.3f/%.3f", *mags, band, mag, err);
+			ret = asprintf(&nb, "%s %s=%.3f/%.3f", *mags, band, mag, err);
 		if (*mags)
 			free(*mags);
+		if (ret == -1)
+			nb = NULL;
 		*mags = nb;
 		return 0;
 	}
 	if (err == 0.0)
-		asprintf(&nb, "%s %s=%.3f", *mags, band, mag);
+		ret = asprintf(&nb, "%s %s=%.3f", *mags, band, mag);
 	else 
-		asprintf(&nb, "%s %s=%.3f/%.3f", *mags, band, mag, err);
-	do {
-		nb[(bs++) - *mags] = nb[be - *mags];
-	} while (nb[(be++) - *mags] != 0);
+		ret = asprintf(&nb, "%s %s=%.3f/%.3f", *mags, band, mag, err);
+	if (ret != -1) {
+		do {
+			nb[(bs++) - *mags] = nb[be - *mags];
+		} while (nb[(be++) - *mags] != 0);
+	} else {
+		nb = NULL;
+	}
 	free(*mags);
 	*mags = nb;
 	return 0;
