@@ -22,8 +22,8 @@
 *******************************************************************************/
 
 // aphot.c: aperture photometry routines
-// $Revision: 1.19 $
-// $Date: 2005/12/22 22:28:54 $
+// $Revision: 1.20 $
+// $Date: 2009/06/20 20:02:34 $
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -133,7 +133,6 @@ int ring_stats(struct ccd_frame *fr, double x, double y,
 	int xc, yc;
 	int w;
 	int nring = 0, nskipped = 0;
-	float *dp;
 	float v;
 	double sum, sumsq, r;
 //	d3_printf("x:%f y:%f r1:%f r2:%f\n", x, y, r1, r2);
@@ -175,11 +174,9 @@ int ring_stats(struct ccd_frame *fr, double x, double y,
 	sumsq = 0.0;
 	rs->min = HUGE;
 	rs->max = -HUGE;
-	dp = ((float *)(fr->dat)) + xs + ys * w;
 	for (iy = ys; iy < ye; iy++) {
 		for (ix = xs; ix < xe; ix++) {
-			v = *dp;
-			dp ++;
+			v = get_pixel_luminence(fr, ix, iy);
 			if (((r = sqr(ix - xc + 0.0) + sqr(iy - yc + 0.0)) 
 			     < sqr(r1) || r >= sqr(r2)))
 				continue;
@@ -208,7 +205,6 @@ int ring_stats(struct ccd_frame *fr, double x, double y,
 			if (v > rs->max) 
 				rs->max = v;
 		}
-		dp += w - xe + xs;
 	}
 // fill the result structure with the stats
 	rs->all = nring;
@@ -298,7 +294,6 @@ static double aperture_flux(struct ccd_frame *fr, double r, double x, double y,
 	int xe, ye;
 	int xc, yc;
 	int w;
-	float *dp;
 	float v;
 	double sum, sumsq;
 	double min, max;
@@ -331,10 +326,9 @@ static double aperture_flux(struct ccd_frame *fr, double r, double x, double y,
 	sum = 0.0;
 	sumsq = 0.0;
 	n = 0.0;
-	dp = ((float *)(fr->dat)) + xs + ys * w;
 	for (iy = ys; iy < ye; iy++) {
 		for (ix = xs; ix < xe; ix++) {
-			v = *dp;
+			v = get_pixel_luminence(fr, ix, iy);
 			d = sqrt(sqr(1.0 * ix - x) + sqr(1.0 * iy - y));
 			if (d > r + 0.5)
 				we = 0.0;
@@ -349,10 +343,7 @@ static double aperture_flux(struct ccd_frame *fr, double r, double x, double y,
 			if (v > max && we > 0.5) 
 				max = v;
 			n += we;
-//			*dp = 100 + we * 200;
-			dp ++;
 		}
-		dp += w - xe + xs;
 	}
 	if (minv)
 		*minv = min;
@@ -377,7 +368,6 @@ static double aperture_centroid(struct ccd_frame *fr, double r, double x, double
 	int xe, ye;
 	int xc, yc;
 	int w;
-	float *dp;
 	float v;
 	double sum;
 	double sx, sy, xsq, ysq, xw, yw;
@@ -405,10 +395,9 @@ static double aperture_centroid(struct ccd_frame *fr, double r, double x, double
 	sx = 0.0; sy = 0.0;
 	xsq = 0.0; ysq = 0.0;
 	n = 0.0;
-	dp = ((float *)(fr->dat)) + xs + ys * w;
 	for (iy = ys; iy < ye; iy++) {
 		for (ix = xs; ix < xe; ix++) {
-			v = *dp;
+			v = get_pixel_luminence(fr, ix, iy);
 			d = sqrt(sqr(1.0 * ix - x) + sqr(1.0 * iy - y));
 			if (d > r + 0.5)
 				we = 0.0;
@@ -424,9 +413,7 @@ static double aperture_centroid(struct ccd_frame *fr, double r, double x, double
 				ysq += sqr(iy) * (v - threshold) * we;
 				n += we;
 			}
-			dp ++;
 		}
-		dp += w - xe + xs;
 	}
 	if (n == 0 || sum == 0) {
 		if (cx)
