@@ -7,8 +7,8 @@
 /*  aperture photometry basic functions */
 /*  misc helper functions */
 /*  extracted from the former cx.h file */
-/*  $Revision: 1.22 $ */
-/*  $Date: 2005/12/12 02:42:08 $ */
+/*  $Revision: 1.37 $ */
+/*  $Date: 2009/09/01 10:15:38 $ */
 
 #ifndef _CCD_H_
 #define _CCD_H_
@@ -64,13 +64,13 @@ struct point {		// a point in a frame or window
 // values describing the geometry of a frame from a ccd chip
 struct ccd_geometry {
 	unsigned w; /* total size of the frame */
-	unsigned h; 
-	int x_skip; /* number of pixels skipped at the beginning of each line 
+	unsigned h;
+	int x_skip; /* number of pixels skipped at the beginning of each line
 		     * (ref: first active pixel) */
 	int y_skip; /* number of lines skipped at the beginning of each frame */
 };
 
-// structure describing a ccd exposure 
+// structure describing a ccd exposure
 struct exp_data {
 	int datavalid;
 	int bin_x; // number of pixels binned in the x direction
@@ -82,7 +82,7 @@ struct exp_data {
 	double flat_noise; // relative noise of flatfield
 //	double airmass; // airmass of exposure
 //	int temperature; // sensor temperature
-//	unsigned cooling_power; // actual cooling power 
+//	unsigned cooling_power; // actual cooling power
 //	struct timeval exp_start;
 //	double exp_start_jd; // exposure start time
 };
@@ -110,7 +110,7 @@ struct im_stats {
 	double max;	// highest pixel
 	double avg; 	// average pixel val
 	double sigma; 	// standard deviation
-	double cavg; 	// avg of center of population 
+	double cavg; 	// avg of center of population
 	double csigma;  // robust sigma estimate
 	double median;	// median of frame
 	struct im_histogram hist; // the histogram for the current image
@@ -126,13 +126,13 @@ struct im_stats {
 #define WCS_JD_VALID 0x02 	/* the jd value is valid for the target frame */
 #define WCS_LOC_VALID 0x04 	/* location is valid (lat and long) */
 #define WCS_USE_LIN 0x08 	/* wcs uses linear transform; in this case rot is set
-				   to a "mean" value, but is _not_ actually used in the 
-				   transformation. the xinc/yinc are however, according to the 
+				   to a "mean" value, but is _not_ actually used in the
+				   transformation. the xinc/yinc are however, according to the
 				   fits convention. */
 struct wcs{
 	int ref_count;
 	int wcsset;	/* non-zero when following are ok */
-	int flags;	
+	int flags;
 	double xref;	/* x reference coordinate value (deg) */
 	double yref;	/* y reference coordinate value (deg) */
 	double xrefpix;	/* x reference pixel */
@@ -141,7 +141,7 @@ struct wcs{
 	double yinc;	/* y coordinate increment (deg) */
 	double rot;	/* rotation (deg)  (from N through E) */
 	double pc[2][2];/* linear matrix transform (used together with xinc, yinc
-			   when the full linear model is enabled. 
+			   when the full linear model is enabled.
 			   the storage order is {PC1_1, PC1_2, PC2_1, PC2_2}, so
 			   m_i,j = pc[i][j] */
 			/* N.B. the matrix sets the pixel->projection plane transform */
@@ -171,6 +171,7 @@ struct bad_pixel {
 // bad pixel map
 struct bad_pix_map {
 	int ref_count;
+	char *filename;         // filename of this map, if any
 	unsigned int x_skip;	// from frame geometry when the map was created
 	unsigned int y_skip;
 	unsigned int bin_x;
@@ -204,27 +205,52 @@ struct bad_pix_map {
 
 #ifndef LITTLE_ENDIAN
 #ifndef BIG_ENDIAN
-#define LITTLE_ENDIAN	
+#define LITTLE_ENDIAN
 #endif
 #endif
 
+struct raw_metadata {
+	double wbr;		/* white balance gains */
+	double wbg;
+	double wbgp;            /* g' */
+	double wbb;
+	unsigned int color_matrix; /* rgb color matrix */
+};
+
+/* encodings of color_matrix */
+#define FRAME_CFA_RGGB		0x94949494
+#define FRAME_CFA_GBRG          0x49494949
+#define FRAME_CFA_BGGR          0x16161616
+#define FRAME_CFA_GRBG          0x61616161
+
+#define CFA_RED    0
+#define CFA_GREEN1 1
+#define CFA_BLUE   2
+#define CFA_GREEN2 3
+
+/* gives one of the above CFA_* colors */
+#define CFA_COLOR(pattern, x, y) \
+	((pattern) >> (((((y) << 1) & 14) + ((x) & 1)) << 1) & 3)
+
+
 struct ccd_frame {
-	int ref_count;	
+	int ref_count;
 	unsigned loaded; /* how much of the frame's data is loaded (for incremental
 			  * loads and readout - in bytes */
 /*   how many need this frame. When ref_count is 1, a release_frame */
 /*   deallocates the frame. The frame is created with ref_count=1 */
 	int w;		// width of frame in pixels
 	int h;		// height of frame in pixels
-	int x_skip; 	// number of pixels skipped at the beginning of each line 
+	int x_skip; 	// number of pixels skipped at the beginning of each line
 			// (ref: first active pixel)
 	int y_skip; 	// number of lines skipped at the beginning of each frame
 	unsigned magic; // an unique number identifying the frame (science, dark, flatfield etc)
 	struct exp_data exp;
 	struct im_stats stats;
 	struct wcs fim;
+	struct raw_metadata rmeta;
 	int data_valid;	/* non-zero shows the data in the array is valid
-			 * normally, it's the amount of data (in bytes) 
+			 * normally, it's the amount of data (in bytes)
 			 * in the array */
 	int pix_size;	// bytes/pixel
 	int pix_format;	// format pixels are stored in
@@ -232,12 +258,15 @@ struct ccd_frame {
 	FITS_row *var;	/* malloced array of all unrecognized header lines */
 	int nvar;	/* number of var[] */
 	char name[256];	// name of frame
+	void *rdat;     // rgb image planes
+	void *gdat;
+	void *bdat;
 };
 
 // magic numbers for ccd frame
 
 // basic values denoting the frame's origin
-#define UNDEF_FRAME	0x0000000	
+#define UNDEF_FRAME	0x0000000
 #define SCIENCE_FRAME 	0x0000001
 #define BIAS_FRAME	0x0000002
 #define FLAT_FRAME	0x0000004
@@ -253,6 +282,11 @@ struct ccd_frame {
 
 #define FRAME_UNCERTAIN 0x1000000	// frame containins uncertain data
 
+/* frame image structure */
+
+#define FRAME_HAS_CFA     0x0100000 /* frame contains CFA data */
+#define FRAME_VALID_RGB   0x0200000 /* rgb data was generated from bayer */
+
 /////// this is how we represent bad pixels
 
 // bad_pix types
@@ -269,7 +303,6 @@ struct ccd_frame {
 #define BADCOLUMN_SUM 0x20
 #define BADCOLUMN_POLFIT 0x20
 
-
 ///////// structures for star extraction and aperture photometry
 
 
@@ -288,7 +321,7 @@ struct ph_star {
 	double sky_max; // sky maximum
 	double sky;     // estimated sky flux / pixel
 	double sky_err; // estimated random error of sky
-	
+
 	double star_all; // number of pixels used for star flux measurements
 	double tflux;	 // total flux (incl. sky)
 	double flux_err; // total flux estimated error
@@ -296,7 +329,7 @@ struct ph_star {
 	double star; 	 // sky-substracted flux
 	double star_err; // star_flux estimated error
 			 // excluding scintillation
-	double star_max; // max pixel value in star area 
+	double star_max; // max pixel value in star area
 	double star_min; // min pixel value in star area
 
 	double rd_noise; // read noise contribution (in abs value)
@@ -305,13 +338,13 @@ struct ph_star {
 	double scint; // scintillation noise
 
 	double stdmag;	// standard magnitude of star (std star) or computed magnitude
-	double residual; // residual when fitting a mag solution 
+	double residual; // residual when fitting a mag solution
 	int flags; 	// flags qualifying measurements results
 };
 
 #define AP_FAINT 1 	// magnitude is lower than the limiting mag
 #define AP_BPSTAR 2 	// there have been bad pixels in the star aperture (serious problem)
-#define AP_BPSKY 4 	// there have been bad pixels in the sky annulus (not a real problem, 
+#define AP_BPSKY 4 	// there have been bad pixels in the sky annulus (not a real problem,
 			// since they can be excluded
 #define AP_ERR_ACTUAL 8 // the sky error calculation is based on the sigma of the sky region
 #define AP_PAR_CHANGED 16 // some photometry params were invalid and were changed
@@ -337,12 +370,12 @@ struct ph_star {
 
 // star's position and magnitude
 struct star {
-	int ref_count;	
+	int ref_count;
 	double x;	// coords of centroid within frame
 	double y;
 	double xerr;		/* estimated errors from centroiding */
 	double yerr;
-	int wcs_set;	/* shows ra and dec are valid. 
+	int wcs_set;	/* shows ra and dec are valid.
 			 * also tells where they came from */
 	double ra;	// placeholder for wcs coords of object
 	double dec;
@@ -366,7 +399,7 @@ struct ap_params {
 	double std_err;  // combined error of standard stars
 	int quads;	// quadrants for sky measurement
 	int center;		/* attempt to center aperture */
-	int sky_method;	//method of obtaining the sky value (0=avg, 1=median 2=kappa-sigma, 
+	int sky_method;	//method of obtaining the sky value (0=avg, 1=median 2=kappa-sigma,
 			//3=median-mean --- defines below)
 //	int ap_shape; 		/* shape of central aperture */
 	double sigmas; // allowable error for kappa-sigma and mmedian (in sigmas)
@@ -396,7 +429,7 @@ struct ap_params {
 // length of text fields in the recipy structure
 #define RECIPY_TEXT 256
 
-// structure defining a relative photometry problem 
+// structure defining a relative photometry problem
 struct vs_recipy {
 	int ref_count;		/* refcount for the recipy structure */
  	char objname[RECIPY_TEXT+1];
@@ -412,7 +445,7 @@ struct vs_recipy {
 	struct ap_params p;	// measurement parameters
 	int max_cnt;		/* max number of stars in recipy */
 	int cnt;		// number of stars in structure
-	struct star *s;		/* malloced array of stars, freed when the recipy 
+	struct star *s;		/* malloced array of stars, freed when the recipy
 				   is freed */
 };
 
@@ -439,7 +472,7 @@ struct rstats {
 };
 
 struct moments {
-	double mx;	// centroid shift (first-order moment relative 
+	double mx;	// centroid shift (first-order moment relative
 	double my;	// to the reference pixel
 	double mx2;	// second-order moments (relative to centroid)
 	double my2;
@@ -450,7 +483,7 @@ struct moments {
 
 // structure describing the frame's sources
 struct sources {
-	int ref_count;	/* for the sources structure 
+	int ref_count;	/* for the sources structure
 			   s is also freed when the structure is destroyed */
 	int maxn; // maximum number of sources (size of s)
 	int ns;	// number of sources
@@ -475,6 +508,28 @@ struct ctrans {
 };
 
 
+//define inline functions for performance-critical stuff
+enum {
+	PLANE_NULL  = 0,
+	PLANE_RAW   = 1,
+	PLANE_RED   = 2,
+	PLANE_GREEN = 3,
+	PLANE_BLUE  = 4,
+};
+
+static inline float get_pixel_luminence(struct ccd_frame *fr, int x, int y) {
+	float val;
+	int offset = y * fr->w + x;
+
+	if (fr->magic & FRAME_VALID_RGB) {
+		val = *((float *)fr->rdat + offset)
+		    + *((float *)fr->gdat + offset)
+		    + *((float *)fr->bdat + offset);
+		val /= 3;
+		return val;
+	}
+	return *((float *)fr->dat + offset);
+}
 
 //////////// Function declarations
 
@@ -491,13 +546,20 @@ int deb_printf(int level, const char *fmt, ...);
 
 extern void get_frame(struct ccd_frame *fr);
 extern void release_frame(struct ccd_frame *fr);
+extern struct ccd_frame *new_frame(unsigned size_x, unsigned size_y);
+extern struct ccd_frame *new_frame_fr(struct ccd_frame* fr, unsigned size_x, unsigned size_y);
+extern struct ccd_frame *new_frame_head_fr(struct ccd_frame* cam, unsigned size_x, unsigned size_y);
+extern void free_frame(struct ccd_frame *fr);
+extern int alloc_frame_data(struct ccd_frame *fr);
+extern int frame_to_float(struct ccd_frame *fr);
 extern struct ccd_frame *clone_frame(struct ccd_frame *fr);
-struct ccd_frame *new_frame(unsigned size_x, unsigned size_y);
-struct ccd_frame *read_fits_file(char *filename, int force_unsigned);
-struct ccd_frame *read_gz_fits_file(char *filename, char *ungz, int force_unsigned);
 extern int frame_stats(struct ccd_frame *fr);
+
+struct ccd_frame *read_fits_file(char *filename, int force_unsigned, char *default_cfa);
+struct ccd_frame *read_gz_fits_file(char *filename, char *ungz, int force_unsigned,
+				    char *default_cfa);
 extern int write_fits_frame(struct ccd_frame *fr, char *filename);
-int write_gz_fits_frame(struct ccd_frame *fr, char *fn, char *gzcmd);
+extern int write_gz_fits_frame(struct ccd_frame *fr, char *fn, char *gzcmd);
 extern int scale_shift_frame(struct ccd_frame *fr, double m, double s);
 extern int madd_frames (struct ccd_frame *fr, struct ccd_frame *fr1, double m);
 //extern int libip_head(struct ccd_frame *fr);
@@ -514,22 +576,40 @@ int fits_get_int(struct ccd_frame *fr, char *kwd, int *v);
 int sub_frames (struct ccd_frame *fr, struct ccd_frame *fr1);
 
 
-// from badpix.c
+struct ccd_frame *read_image_file(char *filename, char *ungz, int force_unsigned,
+				  char *default_cfa);
+struct ccd_frame *read_fits_file_from_mem(const unsigned char *data, unsigned long len, char *fn,
+                                     int force_unsigned, char *default_cfa);
 
-extern int find_bad_pixels(struct bad_pix_map *map, struct ccd_frame *fr, 
-			   double sig);
-extern int save_bad_pix(struct bad_pix_map *map, char *filename);
-extern int load_bad_pix(struct bad_pix_map *map, char *filename);
-extern int fix_bad_pixels (struct ccd_frame *fr, struct bad_pix_map *map, int method);
+extern float *get_color_plane(struct ccd_frame *fr, int plane_iter);
+extern float **get_color_planeptr(struct ccd_frame *fr, int plane_iter);
+extern int color_plane_iter(struct ccd_frame *fr, int plane_iter);
+
+extern int alloc_frame_rgb_data(struct ccd_frame *fr);
+extern int remove_bayer_info(struct ccd_frame *fr);
+
+// from dslr.c
+extern int raw_filename(char *filename);
+extern struct ccd_frame *read_raw_file(char *filename);
+extern int parse_color_field(struct ccd_frame *fr, char *default_cfa);
+extern int set_color_field(struct ccd_frame *fr);
+
+// from badpix.c
+extern int save_bad_pix(struct bad_pix_map *map);
+extern int load_bad_pix(struct bad_pix_map *map);
+extern int free_bad_pix(struct bad_pix_map *map);
+
+extern int find_bad_pixels(struct bad_pix_map *map, struct ccd_frame *fr, double sig);
+extern int fix_bad_pixels (struct ccd_frame *fr, struct bad_pix_map *map);
 
 // from aphot.c
 
-extern int ring_stats(struct ccd_frame *fr, double x, double y, 
+extern int ring_stats(struct ccd_frame *fr, double x, double y,
 		      double r1, double r2, int quads, struct rstats *rs,
 		      double min, double max);
-double hist_clip_avg(struct rstats *rs, double *median, double sigmas, 
+double hist_clip_avg(struct rstats *rs, double *median, double sigmas,
 		     double *lclip, double *hclip);
-extern int aperture_photometry(struct ccd_frame *fr, struct star *s, 
+extern int aperture_photometry(struct ccd_frame *fr, struct star *s,
 			struct ap_params *p, struct bad_pix_map *bp);
 extern int measure_stars(struct ccd_frame *fr, struct vs_recipy *vs);
 extern void get_ph_solution(struct vs_recipy *vs);
@@ -540,7 +620,7 @@ extern void get_scint_pars(struct ccd_frame *fr, struct vs_recipy *vs, double *t
 extern double scintillation(double t, double d, double am);
 
 // from sources.c
-extern int locate_star(struct ccd_frame *fr, double x, double y, double r, double min_flux, 
+extern int locate_star(struct ccd_frame *fr, double x, double y, double r, double min_flux,
 		       struct star *s);
 extern int get_star_near(struct ccd_frame *fr, int x, int y, double min_flux, struct star *s);
 extern int follow_star(struct ccd_frame *fr, double r, struct star *old_star, struct star *s);
@@ -553,12 +633,12 @@ extern void release_star(struct star *s);
 
 // from worldpos.c
 
-extern int xypix(double xpos, double ypos, double xref, double yref, double xrefpix, 
-		 double yrefpix, double xinc, double yinc, double rot, 
+extern int xypix(double xpos, double ypos, double xref, double yref, double xrefpix,
+		 double yrefpix, double xinc, double yinc, double rot,
 		 char *type, double *xpix, double *ypix);
 
-extern int worldpos(double xpos, double ypos, double xref, double yref, double xrefpix, 
-		 double yrefpix, double xinc, double yinc, double rot, 
+extern int worldpos(double xpos, double ypos, double xref, double yref, double xrefpix,
+		 double yrefpix, double xinc, double yinc, double rot,
 		 char *type, double *xpix, double *ypix);
 
 extern double airmass (double aa);
@@ -598,7 +678,7 @@ extern int make_shift_ctrans(struct ctrans *ct, double dx, double dy);
 extern int make_roto_translate(struct ctrans *ct, double dx, double dy, double xs, double ys, double rot);
 extern int shift_frame(struct ccd_frame *fr, double dx, double dy);
 extern int linear_x_shear(struct ccd_frame *in, struct ccd_frame *out, double a, double c);
-extern int linear_y_shear(struct ccd_frame *in, struct ccd_frame *out, double b, double c, 
+extern int linear_y_shear(struct ccd_frame *in, struct ccd_frame *out, double b, double c,
 			  double u0, double v0);
 extern int filter_frame(struct ccd_frame *fr, struct ccd_frame *fro, float *kern, int size);
 extern int filter_frame_inplace(struct ccd_frame *fr, float *kern, int size);
@@ -607,8 +687,10 @@ extern int make_gaussian(float sigma, int size, float *kern);
 /* from edb.c */
 int locate_edb(char name[], double *ra, double *dec, double *mag, char *edbdir);
 
+int try_dcraw(char *filename);
+struct ccd_frame *read_file_from_dcraw(char *filename);
 
-#endif 
+#endif
 
 
 
