@@ -22,8 +22,8 @@
 *******************************************************************************/
 
 // sources.c: find/track sources
-// $Revision: 1.18 $
-// $Date: 2005/02/12 20:29:47 $
+// $Revision: 1.20 $
+// $Date: 2009/07/14 00:45:53 $
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -63,7 +63,6 @@ static int thin_ring_stats(struct ccd_frame *fr, int x, int y,
 	int xc, yc;
 	int w;
 	int nring = 0, nskipped = 0;
-	float *dp;
 	float v;
 	double sum, sumsq, r;
 	int dy2, rsq1, rsq2;
@@ -100,18 +99,12 @@ static int thin_ring_stats(struct ccd_frame *fr, int x, int y,
 	sumsq = 0.0;
 	rs->min = HUGE;
 	rs->max = -HUGE;
-	dp = ((float *)(fr->dat)) + xs + ys * w;
 	rsq1 = sqr(r2);
 	rsq2 = sqr(r2+1.001);
 	for (iy = ys; iy < ye; iy++) {
 		dy2 = (iy - yc) * (iy - yc);
 		for (ix = xs; ix < xe; ix++) {
-			if (dp >= ((float *)(fr->dat)) + all) {
-				err_printf("scanning past end of frame!\n");
-				break;
-			}
-			v = *dp;
-			dp ++;
+			v = get_pixel_luminence(fr, ix, iy);
 			if (((r = (ix - xc) * (ix - xc) + dy2)) 
 			    < rsq1 || r > rsq2)
 				continue;
@@ -131,7 +124,6 @@ static int thin_ring_stats(struct ccd_frame *fr, int x, int y,
 				rs->max_y = iy;
 			}
 		}
-		dp += w - xe + xs;
 	}
 
 // fill the result structure with the stats
@@ -159,7 +151,6 @@ static int star_moments(struct ccd_frame *fr, double x, double y,
 	int xc, yc;
 	int w;
 	int nring = 0, nskipped = 0;
-	float *dp;
 	float v;
 	double sum, rn;
 	double mx, my, mxy, mx2, my2;
@@ -200,11 +191,9 @@ static int star_moments(struct ccd_frame *fr, double x, double y,
 	mx2 = 0;
 	my2 = 0;
 	mxy = 0;
-	dp = ((float *)(fr->dat)) + xs + ys * w;
 	for (iy = ys; iy < ye; iy++) {
 		for (ix = xs; ix < xe; ix++) {
-			v = *dp;
-			dp ++;
+			v = get_pixel_luminence(fr, ix, iy);
 			if (((rn = sqr(ix - xc) + sqr(iy - yc)) > sqr(r)))
 				continue;
 			nring ++;
@@ -225,7 +214,6 @@ static int star_moments(struct ccd_frame *fr, double x, double y,
 			my2 += v * sqr(iy);
 			sum += v;
 		}
-		dp += w - xe + xs;
 	}
 // fill the result structure with the stats
 	mx /= sum;
@@ -600,16 +588,25 @@ int extract_stars(struct ccd_frame *fr, struct region *reg, double min_flux, dou
 	struct star st;
 	int ret;
 	int x, y;
+	int xs, ys, w, h;
 	int rn;
 	double minpk, pk;
 	double skycut, sky;
 	int starr;
 	double fwhm;
-	float *dp;
+	float dp;
 	int candidates = 0;
 
 	if (reg != NULL) {
-		err_printf("extract_stars: i dont know how to do regions yet\n");
+		xs = reg->xs;
+		ys = reg->ys;
+		w = reg->w;
+		h = reg->h;
+	} else {
+		xs = 0;
+		ys = 0;
+		w = fr->w;
+		h = fr->h;
 	}
 
 	if (!fr->stats.statsok)
@@ -620,13 +617,12 @@ int extract_stars(struct ccd_frame *fr, struct region *reg, double min_flux, dou
 
 	minpk = fr->stats.cavg + 2 * sigmas * fr->stats.csigma;
 
-	dp = fr->dat;
-
 	rsn = malloc(sizeof(struct rstats));
 
-	for (y = 0; y < fr->h; y++) {
-		for (x = 0; x < fr->w; x++) {
-			if ((pk = *dp++) <= minpk) {// below detection threshold
+	for (y = ys; y < ys + h; y++) {
+		for (x = xs; x < xs + w; x++) {
+			dp = get_pixel_luminence(fr, x, y);
+			if ((pk = dp) <= minpk) {// below detection threshold
 				continue;
 			}
 			candidates ++;
