@@ -1,31 +1,31 @@
 /*******************************************************************************
   Copyright(c) 2000 - 2003 Radu Corlan. All rights reserved.
-  
-  This program is free software; you can redistribute it and/or modify it 
-  under the terms of the GNU General Public License as published by the Free 
-  Software Foundation; either version 2 of the License, or (at your option) 
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the Free
+  Software Foundation; either version 2 of the License, or (at your option)
   any later version.
-  
-  This program is distributed in the hope that it will be useful, but WITHOUT 
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
-  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
+
+  This program is distributed in the hope that it will be useful, but WITHOUT
+  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.
-  
+
   You should have received a copy of the GNU General Public License along with
-  this program; if not, write to the Free Software Foundation, Inc., 59 
+  this program; if not, write to the Free Software Foundation, Inc., 59
   Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
   The full GNU General Public License is included in this distribution in the
   file called LICENSE.
-  
+
   Contact Information: radu@corlan.net
 *******************************************************************************/
 
 // warp.c: non-linear image coordinates transform functions
 // and other geometric transforms and filters
 
-// $Revision: 1.19 $
-// $Date: 2006/07/17 22:24:09 $
+// $Revision: 1.23 $
+// $Date: 2009/08/19 18:40:19 $
 
 #include <math.h>
 
@@ -42,7 +42,7 @@
 
 float smooth3[9] = {0.25, 0.5, 0.25, 0.5, 1.0, 0.5, 0.25, 0.5, 0.25};
 
-// fast 7x7 convolution; dp points to the upper-left of the convolution box 
+// fast 7x7 convolution; dp points to the upper-left of the convolution box
 // in the image, ker is a linear vector that contains the kerner
 // scanned l->r and t>b; w is the width of the image
 static inline float conv7(float *dp, float *ker, int w)
@@ -109,14 +109,14 @@ static inline float conv7(float *dp, float *ker, int w)
 	return d;
 }
 
-// fast 5x5 convolution; dp points to the upper-left of the convolution box 
+// fast 5x5 convolution; dp points to the upper-left of the convolution box
 // in the image, ker is a linear vector that contains the kerner
 // scanned l->r and t>b; w is the width of the image
 static inline float conv5(float *dp, float *ker, int w)
 {
 	int nl;
 	float d;
- 
+
 	nl = w - 4;
 	d = *dp++ * *ker++;
 	d += *dp++ * *ker++;
@@ -150,7 +150,7 @@ static inline float conv5(float *dp, float *ker, int w)
 	return d;
 }
 
-// fast 3x3 convolution; dp points to the upper-left of the convolution box 
+// fast 3x3 convolution; dp points to the upper-left of the convolution box
 // in the image, ker is a linear vector that contains the kerner
 // scanned l->r and t>b; w is the width of the image
 static inline float conv3(float *dp, float *ker, int w)
@@ -183,62 +183,66 @@ int filter_frame(struct ccd_frame *fr, struct ccd_frame *fro, float *kern, int s
 	float *dpi;
 	float *dpo;
 	int i, all;
+	int plane_iter = 0;
 
 	if (size % 2 != 1 || size < 3 || size > 7) {
 		err_printf("filter_frame: bad size %d\n");
 		return -1;
-	} 
+	}
 
 	w = fr->w;
 
 	all = w * (fr->h - (size - 1)) - (size) ;
-	dpi = (float *)fr->dat;
-	dpo = (float *)fro->dat;
+
+	while ((plane_iter = color_plane_iter(fr, plane_iter))) {
+		dpi = get_color_plane(fr, plane_iter);
+		dpo = get_color_plane(fro, plane_iter);
 
 
-//	for(i=0; i<all; i++)
-//		*dpo++ = *dpi++;
-//	return;
+//		for(i=0; i<all; i++)
+//			*dpo++ = *dpi++;
+//		return;
 
-//	d3_printf("dpi %x dpo %x\n", dpi, dpo);
+//		d3_printf("dpi %x dpo %x\n", dpi, dpo);
 
-	for (i = 0; i < (size / 2) * w + (size / 2); i++) {
-		*dpo++ = fr->stats.cavg;
-	}
-
-//	d3_printf("all: %d (%d) w: %d allo: %d\n", all, w * fr->h, w, 
-//		  fro->w*fro->h);
-
-//	d3_printf("dpi %x dpo %x\n", dpi, dpo);
-
-	switch(size) {
-	case 3:
-		for (i=0; i<all; i++) {
-			*dpo = conv3(dpi, kern, w);
-			dpi ++;
-			dpo ++;
+		for (i = 0; i < (size / 2) * w + (size / 2); i++) {
+			*dpo++ = fr->stats.cavg;
 		}
-		break;
-	case 5:
-		for (i=0; i<all; i++) {
-			*dpo = conv5(dpi, kern, w);
-			dpi ++;
-  			dpo ++;
-		}
-		break;
-	case 7:
-		for (i=0; i<all; i++) {
-			*dpo = conv7(dpi, kern, w);
-			dpi ++;
-			dpo ++;
-		}
-		break;
-	}
 
-//	d3_printf("dpi %x dpo %x\n", dpi, dpo);
+//		d3_printf("all: %d (%d) w: %d allo: %d\n", all, w * fr->h, w,
+//			  fro->w*fro->h);
 
-	for (i = 0; i < (size / 2) * w + (size / 2); i++) {
-		*dpo++ = fr->stats.cavg;
+//		d3_printf("dpi %x dpo %x\n", dpi, dpo);
+
+		switch(size) {
+		case 3:
+			for (i=0; i<all; i++) {
+				*dpo = conv3(dpi, kern, w);
+				dpi ++;
+				dpo ++;
+			}
+			break;
+		case 5:
+			for (i=0; i<all; i++) {
+				*dpo = conv5(dpi, kern, w);
+				dpi ++;
+  				dpo ++;
+			}
+			break;
+		case 7:
+			for (i=0; i<all; i++) {
+				*dpo = conv7(dpi, kern, w);
+				dpi ++;
+				dpo ++;
+			}
+			break;
+		}
+
+//		d3_printf("dpi %x dpo %x\n", dpi, dpo);
+
+		for (i = 0; i < (size / 2) * w + (size / 2); i++) {
+			*dpo++ = fr->stats.cavg;
+		}
 	}
 
 	fr->stats.statsok = 0;
@@ -252,7 +256,9 @@ int filter_frame(struct ccd_frame *fr, struct ccd_frame *fro, float *kern, int s
 int filter_frame_inplace(struct ccd_frame *fr, float *kern, int size)
 {
 	struct ccd_frame *new;
+	float *dpi, *dpo;
 	int ret, all;
+	int plane_iter = 0;
 
 	new = clone_frame(fr);
 	if (new == NULL)
@@ -264,7 +270,11 @@ int filter_frame_inplace(struct ccd_frame *fr, float *kern, int size)
 		return ret;
 	}
 	all = fr->w * fr->h;
-	memcpy(fr->dat, new->dat, all * fr->pix_size);
+	while ((plane_iter = color_plane_iter(fr, plane_iter))) {
+		dpi = get_color_plane(fr, plane_iter);
+		dpo = get_color_plane(new, plane_iter);
+		memcpy(dpo, dpi, all * fr->pix_size);
+	}
 	fr->stats.statsok = 0;
 	release_frame(new);
 	return ret;
@@ -358,6 +368,7 @@ int linear_x_shear(struct ccd_frame *in, struct ccd_frame *out, double a, double
 {
 	int wi, hi, wo, ho; // frame geometries
 	float *dpi, *dpo; // frame pointers
+	int plane_iter = 0;
 
 	double dwi; // double input frame w/h to avoid repeated transforms
 
@@ -365,7 +376,7 @@ int linear_x_shear(struct ccd_frame *in, struct ccd_frame *out, double a, double
 	double xl0; // pixel slice factors
 
 	int ui, vi; //coordinates in the output frame
-	double x; //coordinates in the input frame 
+	double x; //coordinates in the input frame
 	double ox;
 	double v;
 
@@ -391,40 +402,44 @@ int linear_x_shear(struct ccd_frame *in, struct ccd_frame *out, double a, double
 		return -1;
 	}
 
-//	d3_printf("wi:%d, wo:%d, ho:%d, hi:%d, filler:%.1f\n", wi, wo, ho, hi, filler); 
+//	d3_printf("wi:%d, wo:%d, ho:%d, hi:%d, filler:%.1f\n", wi, wo, ho, hi, filler);
 
-	for (vi = 0; vi < ho; vi++) { //output lines 
-		dpo = (float *)(out->dat) + wo * vi; // line origins
-		dpi = (float *)(in->dat) + wi * vi;
-		ox = - a * c * 1.0 * vi; // x of pixel with u = 0;
-		x = ox - c;
-		xl = c;
-//		read = 0;
-//		d3_printf("vi=%d ", vi);
-		for (ui = 0; ui < wo; ui++) { //output pixels
+	while ((plane_iter = color_plane_iter(in, plane_iter))) {
+		for (vi = 0; vi < ho; vi++) { //output lines
+			dpo = get_color_plane(out, plane_iter);
+			dpi = get_color_plane(in, plane_iter);
+			dpo += wo * vi; // line origins
+			dpi += wi * vi;
+			ox = - a * c * 1.0 * vi; // x of pixel with u = 0;
+			x = ox - c;
 			xl = c;
-			x += xl;
-			if ((x <= 0) || (x > dwi - xl) || (vi >= hi)) { // we are outside the input frame
-				*dpo++ = filler;
-				continue;
- 			}
-			xl0 = ceil(x) - x;
-			if (xl0 > xl) { // it's all within one input pixel
-				*dpo++ = *dpi * xl;
-				continue;
-			}
-			v = (*dpi++) * xl0; // get the first slice
-//			read ++;
-			xl -= xl0;
-			while (xl > 1.0) { // add the whole pixels
-				v += (*dpi++);
+//			read = 0;
+//			d3_printf("vi=%d ", vi);
+			for (ui = 0; ui < wo; ui++) { //output pixels
+				xl = c;
+				x += xl;
+				if ((x <= 0) || (x > dwi - xl) || (vi >= hi)) { // we are outside the input frame
+					*dpo++ = filler;
+					continue;
+ 				}
+				xl0 = ceil(x) - x;
+				if (xl0 > xl) { // it's all within one input pixel
+					*dpo++ = *dpi * xl;
+					continue;
+				}
+				v = (*dpi++) * xl0; // get the first slice
 //				read ++;
-				xl -= 1.0;
+				xl -= xl0;
+				while (xl > 1.0) { // add the whole pixels
+					v += (*dpi++);
+//					read ++;
+					xl -= 1.0;
+				}
+				v += (*dpi) * xl; // add the last bit
+				*dpo ++ = v;
 			}
-			v += (*dpi) * xl; // add the last bit 
-			*dpo ++ = v;
+//			d3_printf("read %d vi=%d\n", read, vi);
 		}
-//		d3_printf("read %d vi=%d\n", read, vi);
 	}
 //	d3_printf("x_shear: exiting\n");
 	return 0;
@@ -433,11 +448,12 @@ int linear_x_shear(struct ccd_frame *in, struct ccd_frame *out, double a, double
 // do linear shear transform in the y direction
 // x = u - u0
 // y = c (v - v0 + b * (u - u0))
-int linear_y_shear(struct ccd_frame *in, struct ccd_frame *out, double b, double c, 
+int linear_y_shear(struct ccd_frame *in, struct ccd_frame *out, double b, double c,
 		   double u0, double v0)
 {
 	int wi, wo, ho; // frame geometries
 	float *dpi, *dpo; // frame pointers
+	int plane_iter = 0;
 
 	double dhi; // double input frame w/h to avoid repeated transforms
 
@@ -445,7 +461,7 @@ int linear_y_shear(struct ccd_frame *in, struct ccd_frame *out, double b, double
 	double yl0;
 
 	int ui, uip, vi; //coordinates in the output frame
-	double y; //coordinates in the input frame 
+	double y; //coordinates in the input frame
 	double oy;
 	double v;
 
@@ -466,39 +482,41 @@ int linear_y_shear(struct ccd_frame *in, struct ccd_frame *out, double b, double
 		return -1;
 	}
 
-	dpo = out->dat; // output pointer
-
-	for (ui = 0; ui < wo; ui++) { //output columns
-		uip = (int)floor(ui - u0);
-		dpo = (float *)(out->dat) + uip; // line origins
-		dpi = (float *)(in->dat) + uip;
-		oy = c * (b * (1.0 * ui - u0) - v0); // y of pixel with v = 0;
-		y = oy - c;
-		for (vi = 0; vi < ho; vi++) { //output pixels
-			yl = c;
-			y += yl;
-			if (y <= 0 || y > dhi - yl || ui >= wi) { // we are outside the input frame
-				*dpo = filler;
-				dpo += wo;
-				continue;
- 			}
-			yl0 = ceil(y) - y;
-			if (yl0 > yl) { // it's all within one input pixel
-				*dpo = *dpi * yl;
-				dpo += wo;
-				continue;
-			}
-			v = (*dpi) * yl0; // get the first slice
-			dpi += wi;
-			yl -= yl0;
-			while (yl > 1.0) { // add the whole pixels
-				v += (*dpi);
+	while ((plane_iter = color_plane_iter(in, plane_iter))) {
+		for (ui = 0; ui < wo; ui++) { //output columns
+			uip = (int)floor(ui - u0);
+			dpo = get_color_plane(out, plane_iter);
+			dpi = get_color_plane(in, plane_iter);
+			dpo += uip; // line origins
+			dpi += uip;
+			oy = c * (b * (1.0 * ui - u0) - v0); // y of pixel with v = 0;
+			y = oy - c;
+			for (vi = 0; vi < ho; vi++) { //output pixels
+				yl = c;
+				y += yl;
+				if (y <= 0 || y > dhi - yl || ui >= wi) { // we are outside the input frame
+					*dpo = filler;
+					dpo += wo;
+					continue;
+	 			}
+				yl0 = ceil(y) - y;
+				if (yl0 > yl) { // it's all within one input pixel
+					*dpo = *dpi * yl;
+					dpo += wo;
+					continue;
+				}
+				v = (*dpi) * yl0; // get the first slice
 				dpi += wi;
-				yl -= 1.0;
+				yl -= yl0;
+				while (yl > 1.0) { // add the whole pixels
+					v += (*dpi);
+					dpi += wi;
+					yl -= 1.0;
+				}
+				v += (*dpi) * yl; // add the last bit
+				*dpo = v;
+				dpo += wo;
 			}
-			v += (*dpi) * yl; // add the last bit 
-			*dpo = v;
-			dpo += wo;
 		}
 	}
 	return 0;
@@ -530,7 +548,7 @@ static void shift_right(float *dat, int w, int h, int dn, double a, double b)
 		*dp-- = *sp-- * b;
 		for (x = 0; x < dn; x++)
 			*dp-- = 0.0;
-	} 
+	}
 }
 
 // shift frame left
@@ -549,7 +567,7 @@ static void shift_left(float *dat, int w, int h, int dn, double a, double b)
 		*dp++ = *sp * b;
 		for (x = 0; x < dn; x++)
 			*dp++ = 0.0;
-	} 
+	}
 }
 
 // shift frame up
@@ -599,7 +617,7 @@ static void shift_down(float *dat, int w, int h, int dn, double a, double b)
 	for (y = 0; y < dn; y++)
 		for (x = 0; x < w; x++)
 			*dp-- = 0.0;
- 
+
 }
 // shift_frame rebins a frame in-place maintaining it's orientation and scale
 // frame size is maintained; new pixels are 0-filled
@@ -610,35 +628,37 @@ int shift_frame(struct ccd_frame *fr, double dx, double dy)
 	int dn;
 	int h, w;
 	float *dat;
+	int plane_iter = 0;
 
 	w = fr->w;
 	h = fr->h;
-	dat = (float *)fr->dat;
 
-	if (dx > 0) { // shift right
-		a = dx - floor(dx);
-		b = 1 - a;
-		dn = floor(dx);
-		shift_right(dat, w, h, dn, a, b);
-	} else if (dx < 0) { // shift left
-		dx = -dx;
-		a = dx - floor(dx);
-		b = 1 - a;
-		dn = floor(dx);
-		shift_left(dat, w, h, dn, a, b);
-	}
+	while ((plane_iter = color_plane_iter(fr, plane_iter))) {
+		dat = get_color_plane(fr, plane_iter);
+		if (dx > 0) { // shift right
+			a = dx - floor(dx);
+			b = 1 - a;
+			dn = floor(dx);
+			shift_right(dat, w, h, dn, a, b);
+		} else if (dx < 0) { // shift left
+			a = -dx - floor(-dx);
+			b = 1 - a;
+			dn = floor(-dx);
+			shift_left(dat, w, h, dn, a, b);
+		}
 
-	if (dy < 0.0) { // shift up
-		dy = -dy;
-		a = dy - floor(dy);
-		b = 1 - a;
-		dn = floor(dy);
-		shift_up(dat, w, h, dn, a, b);
-	} else if (dy > 0.0) {
-		a = dy - floor(dy);
-		b = 1 - a;
-		dn = floor(dy);
-		shift_down(dat, w, h, dn, a, b);
+		if (dy < 0.0) { // shift up
+			a = -dy - floor(-dy);
+			b = 1 - a;
+			dn = floor(-dy);
+			shift_up(dat, w, h, dn, a, b);
+
+		} else if (dy > 0.0) {
+			a = dy - floor(dy);
+			b = 1 - a;
+			dn = floor(dy);
+			shift_down(dat, w, h, dn, a, b);
+		}
 	}
 
 	return 0;
@@ -664,7 +684,7 @@ int make_gaussian(float sigma, int size, float *kern)
 	if (size % 2 != 1 || size < 3 || size > 127) {
 		err_printf("make_gaussian: bad size %d\n");
 		return -1;
-	} 
+	}
 	mid = size / 2;
 	mp = kern + mid * size + mid;
 	sum = 0;
