@@ -1,23 +1,23 @@
 /*******************************************************************************
   Copyright(c) 2000 - 2003 Radu Corlan. All rights reserved.
-  
-  This program is free software; you can redistribute it and/or modify it 
-  under the terms of the GNU General Public License as published by the Free 
-  Software Foundation; either version 2 of the License, or (at your option) 
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the Free
+  Software Foundation; either version 2 of the License, or (at your option)
   any later version.
-  
-  This program is distributed in the hope that it will be useful, but WITHOUT 
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
-  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
+
+  This program is distributed in the hope that it will be useful, but WITHOUT
+  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.
-  
+
   You should have received a copy of the GNU General Public License along with
-  this program; if not, write to the Free Software Foundation, Inc., 59 
+  this program; if not, write to the Free Software Foundation, Inc., 59
   Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
   The full GNU General Public License is included in this distribution in the
   file called LICENSE.
-  
+
   Contact Information: radu@corlan.net
 *******************************************************************************/
 
@@ -51,72 +51,64 @@
 
 #define FILE_FILTER_SZ 255
 
+#define F_OPEN GTK_FILE_CHOOSER_ACTION_OPEN
+#define F_SAVE GTK_FILE_CHOOSER_ACTION_SAVE
+
 struct file_action {
-	GtkWidget *filesel;
+	GtkWidget *chooser;
 	GtkWidget *window;
 	void *data;
 	int arg1;
 	char file_filter[FILE_FILTER_SZ+1];
+
+	void (*action)(GtkWidget *, gpointer user_data);
 };
 
-gint destroy_filesel(gpointer data)
+
+gboolean chooser_response_cb(GtkWidget *widget, gint response, struct file_action *fa)
 {
-	gtk_object_set_data(GTK_OBJECT(data), "file_selector", NULL);
+	if (response == GTK_RESPONSE_ACCEPT) {
+
+		/* call the action on the selected files */
+		fa->action(GTK_WIDGET(widget), fa);
+	}
+
+	gtk_object_set_data(GTK_OBJECT(fa->window), "file_chooser", NULL);
+	gtk_widget_destroy(widget);
+	
+	free(fa);
+
 	return TRUE;
 }
 
-/*
-static void filter_files (GtkCList *clist,
-			  gint row,
-			  gint column,
-			  GdkEventButton *event,
-			  gpointer user_data)
+GtkWidget * create_file_chooser(char *title, GtkFileChooserAction chooser_type, struct file_action *fa)
 {
-	struct file_action *fa = user_data;
-	d3_printf("filter files\n");
-	if (fa->file_filter[0] != 0)
-		gtk_file_selection_complete(GTK_FILE_SELECTION(fa->filesel), fa->file_filter);
-}
-*/
+	GtkWidget *chooser;
 
-GtkWidget * create_file_selection(char *title, GtkSignalFunc file_action,
-				  struct file_action *action) 
-{
-	GtkWidget *file_selector;
-	file_selector = gtk_file_selection_new(title);
-	gtk_window_set_position (GTK_WINDOW (file_selector), GTK_WIN_POS_CENTER);
+	if (chooser_type == F_OPEN)
+		chooser = gtk_file_chooser_dialog_new(title, NULL, chooser_type,
+						      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+						      NULL);
+	else
+		chooser = gtk_file_chooser_dialog_new(title, NULL, chooser_type,
+						      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+						      NULL);
 
-	gtk_clist_set_selection_mode (GTK_CLIST (GTK_FILE_SELECTION(file_selector)->file_list), 
-				     GTK_SELECTION_EXTENDED);
-   
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
-			    "clicked", GTK_SIGNAL_FUNC (file_action), action);
+	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(chooser), TRUE);
 
-//	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->dir_list),
-//			    "select-row", GTK_SIGNAL_FUNC (filter_files), action);
+	g_object_set_data(G_OBJECT(fa->window), "file_chooser", chooser);
 
-	gtk_signal_connect_object (GTK_OBJECT (file_selector), "delete_event",
-                        GTK_SIGNAL_FUNC (destroy_filesel), (gpointer) action->window);
-   			   
-	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
-				   "clicked", GTK_SIGNAL_FUNC (destroy_filesel),
-				   (gpointer) action->window);
+	g_signal_connect(G_OBJECT(chooser), "response",
+			 G_CALLBACK(chooser_response_cb), fa);
 
-	gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->cancel_button),
-				   "clicked", GTK_SIGNAL_FUNC (destroy_filesel),
-				   (gpointer) action->window);
-      
-	gtk_object_set_data_full(GTK_OBJECT(file_selector), "file_action", action, 
-			    GTK_SIGNAL_FUNC (free));
+	gtk_widget_show (chooser);
 
-	gtk_object_set_data_full(GTK_OBJECT(action->window), "file_selector", file_selector, 
-			    GTK_SIGNAL_FUNC (gtk_widget_destroy));
-
-	gtk_widget_show (file_selector);
-	return file_selector;
+	return chooser;
 }
 
-/* return a malloced string that contains the supplied 
+/* return a malloced string that contains the supplied
  * filename with ".ext" added (if it didn;t already end in '.ext')
  */
 char *add_extension(char *fn, char *ext)
@@ -148,7 +140,7 @@ char *add_extension(char *fn, char *ext)
 }
 
 
-/* return a malloced string that contains the supplied 
+/* return a malloced string that contains the supplied
  * filename with the extention (if any) replaced by ext
  */
 char *force_extension(char *fn, char *ext)
@@ -189,59 +181,72 @@ char *force_extension(char *fn, char *ext)
 static void set_last_open(gpointer object, char *file_class, char *path)
 {
 	char *op;
-	gtk_object_set_data_full(GTK_OBJECT(object), file_class, path, 
-			    GTK_SIGNAL_FUNC (free));
+
+	g_object_set_data_full(G_OBJECT(object), file_class, path,
+			       (GDestroyNotify) free);
+
 	op = gtk_object_get_data(GTK_OBJECT(object), "last_open_fits");
 	if (op == NULL) {
-		gtk_object_set_data_full(GTK_OBJECT(object), "last_open_fits", strdup(path), 
-					 GTK_SIGNAL_FUNC (free));
+		g_object_set_data_full(G_OBJECT(object), "last_open_fits", strdup(path),
+				       (GDestroyNotify) free);
 	}
+
 	op = gtk_object_get_data(GTK_OBJECT(object), "last_open_etc");
 	if (op == NULL) {
-		gtk_object_set_data_full(GTK_OBJECT(object), "last_open_etc", strdup(path), 
-					 GTK_SIGNAL_FUNC (free));
+		g_object_set_data_full(G_OBJECT(object), "last_open_etc", strdup(path),
+				       (GDestroyNotify) free);
 	}
+
 	op = gtk_object_get_data(GTK_OBJECT(object), "last_open_rcp");
 	if (op == NULL) {
-		gtk_object_set_data_full(GTK_OBJECT(object), "last_open_rcp", strdup(path), 
-					 GTK_SIGNAL_FUNC (free));
-	}	
+		g_object_set_data_full(G_OBJECT(object), "last_open_rcp", strdup(path),
+				       (GDestroyNotify) free);
+	}
+
 	op = gtk_object_get_data(GTK_OBJECT(object), "last_open_mband");
 	if (op == NULL) {
-		gtk_object_set_data_full(GTK_OBJECT(object), "last_open_mband", strdup(path), 
-					 GTK_SIGNAL_FUNC (free));
-	}	
+		g_object_set_data_full(G_OBJECT(object), "last_open_mband", strdup(path),
+				       (GDestroyNotify) free);
+	}
 }
 
 
 /* file action functions */
 
-/* create a pnm file from the channel-mapped image 
+/* create a pnm file from the channel-mapped image
  * if fa->arg1 = 1, we create a 16-bit pnm file (still TBD)
  */
-static void export_pnm(GtkFileSelection *selector, gpointer user_data) 
+static void export_pnm(GtkWidget *chooser, gpointer user_data)
 {
 	char *fn, *fne;
 	struct file_action *fa = user_data;
-	fn = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fa->filesel));
+
+	fn = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
+	g_return_if_fail(fn != NULL);
+
 	fne = add_extension(fn, "pnm");
+
 	d3_printf("Saving pnm file: %s\n", fne);
-	channel_to_pnm_file(fa->data, fa->window, fne);
+
+	channel_to_pnm_file(fa->data, fa->window, fne, fa->arg1);
 	free(fne);
+	g_free(fn);
 }
 
 
-/* 
+/*
  * save to a fits file
  */
-static void save_fits(GtkFileSelection *selector, gpointer user_data) 
+static void save_fits(GtkWidget *chooser, gpointer user_data)
 {
 	char *fn;
 	struct file_action *fa = user_data;
 	struct image_channel *ich;
 	int i;
 
-	fn = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fa->filesel));
+	fn = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
+	g_return_if_fail(fn != NULL);
+
 	d3_printf("Saving fits file: %s\n", fn);
 	ich = fa->data;
 	if (file_is_zipped(fn)) {
@@ -249,24 +254,30 @@ static void save_fits(GtkFileSelection *selector, gpointer user_data)
 			if (fn[i] == '.') {
 				fn[i] = 0;
 				break;
-			}		
+			}
 		write_gz_fits_frame(ich->fr, fn, P_STR(FILE_COMPRESS));
 	} else {
 		write_fits_frame(ich->fr, fn);
 	}
+
+	g_free(fn);
 }
 
 
 /* open a fits file and display it */
-static void open_fits(GtkFileSelection *selector, gpointer user_data) 
+static void open_fits(GtkWidget *chooser, gpointer user_data)
 {
 	char *fn, *fnd;
 	struct ccd_frame *fr;
 	struct file_action *fa = user_data;
 
-	fn = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fa->filesel));
+	fn = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
 	d3_printf("opening fits file: %s\n", fn);
-	fr = read_gz_fits_file(fn, P_STR(FILE_UNCOMPRESS), P_INT(FILE_UNSIGNED_FITS));
+	fr = read_image_file(fn, P_STR(FILE_UNCOMPRESS), P_INT(FILE_UNSIGNED_FITS),
+			     default_cfa[P_INT(FILE_DEFAULT_CFA)]);
+
+	g_free(fn);
+
 	if (fr == NULL) {
 		err_printf_sb2(fa->window, "Error trying to open fits file\n");
 		return;
@@ -285,7 +296,7 @@ int load_rcp_to_window(gpointer window, char *name, char *object)
 	struct stf *stf = NULL;
 	GList *rsl;
 	int p=0;
-	FILE *rfn;
+	FILE *rfn = NULL;
 	char *cmd;
 	struct wcs *wcs;
 	char *text, *file=NULL;
@@ -306,7 +317,7 @@ int load_rcp_to_window(gpointer window, char *name, char *object)
 			err_printf("no image - cannot load/create dynamic recipe\n");
 			return -1;
 		}
-		if (object && object[0]) 
+		if (object && object[0])
 			strncpy(obj, object, 70);
 		else if (fits_get_string(channel->fr, P_STR(FN_OBJECT), obj, 70) <= 0) {
 			err_printf("no object - cannot load/create dynamic recipe\n");
@@ -340,9 +351,10 @@ int load_rcp_to_window(gpointer window, char *name, char *object)
 	} else {
 		/* just load the specified file */
 		if (file_is_zipped(name)) {
-			asprintf(&cmd, "%s %s ", P_STR(FILE_UNCOMPRESS), name);
-			rfn = popen(cmd, "r");
-			free(cmd);
+			if (-1 != asprintf(&cmd, "%s %s ", P_STR(FILE_UNCOMPRESS), name)) {
+				rfn = popen(cmd, "r");
+				free(cmd);
+			}
 			p=1;
 		} else {
 			rfn = fopen(name, "r");
@@ -356,7 +368,7 @@ int load_rcp_to_window(gpointer window, char *name, char *object)
 		if (file)
 			free(file);
 		stf = stf_read_frame(rfn);
-		
+
 		if (p) {
 			pclose(rfn);
 		} else {
@@ -367,7 +379,7 @@ int load_rcp_to_window(gpointer window, char *name, char *object)
 	if (stf == NULL)
 		return -1;
 
-	if (wcs != NULL && ((wcs->wcsset < WCS_INITIAL) 
+	if (wcs != NULL && ((wcs->wcsset < WCS_INITIAL)
 			    || ((wcs->wcsset <= WCS_FITTED) && (wcs->flags & WCS_HINTED)))) {
 		d3_printf("try wcs from rcp\n");
 		if (!stf_find_double(stf, &v, 1, SYM_RECIPE, SYM_EQUINOX))
@@ -391,13 +403,13 @@ int load_rcp_to_window(gpointer window, char *name, char *object)
 				wcs->wcsset = WCS_INITIAL;
 			}
 		}
-	} 
+	}
 
 	rsl = stf_find_glist(stf, 0, SYM_STARS);
 	if (rsl == NULL)
 		return -1;
 
-	gtk_object_set_data_full(GTK_OBJECT(window), "recipe", stf, 
+	gtk_object_set_data_full(GTK_OBJECT(window), "recipe", stf,
 				 (GtkDestroyNotify)stf_free_all);
 	merge_cat_star_list_to_window(window, rsl);
 	return g_list_length(rsl);
@@ -409,7 +421,7 @@ int load_gsc2_to_window(gpointer window, char *name)
 {
 	struct cat_star *csl[MAX_GSC2_STARS];
 	int ret, p=0;
-	FILE *rfn;
+	FILE *rfn = NULL;
 	char *cmd;
 
 	if (name == NULL)
@@ -417,9 +429,10 @@ int load_gsc2_to_window(gpointer window, char *name)
 	d3_printf("read_gsc2: looking for %s\n", name);
 
 	if (file_is_zipped(name)) {
-		asprintf(&cmd, "%s %s ", P_STR(FILE_UNCOMPRESS), name);
-		rfn = popen(cmd, "r");
-		free(cmd);
+		if (-1 != asprintf(&cmd, "%s %s ", P_STR(FILE_UNCOMPRESS), name)) {
+			rfn = popen(cmd, "r");
+			free(cmd);
+		}
 		p=1;
 	} else {
 		rfn = fopen(name, "r");
@@ -448,12 +461,12 @@ int load_gsc2_to_window(gpointer window, char *name)
 
 
 /* open a rcp file and load stars; */
-static void open_rcp(GtkFileSelection *selector, gpointer user_data) 
+static void open_rcp(GtkWidget *chooser, gpointer user_data)
 {
 	char *fn, *fnd;
 	struct file_action *fa = user_data;
 
-	fn = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fa->filesel));
+	fn = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
 	g_return_if_fail(fn != NULL);
 
 	load_rcp_to_window(fa->window, fn, NULL);
@@ -461,16 +474,18 @@ static void open_rcp(GtkFileSelection *selector, gpointer user_data)
 	set_last_open(fa->window, "last_open_rcp", fnd = strdup(fn));
 	d3_printf("lastopen_rcp set to %s\n", fnd);
 	gtk_widget_queue_draw(fa->window);
+
+	g_free(fn);
 }
 
 /* open a report file */
-static void open_mband(GtkFileSelection *selector, gpointer user_data) 
+static void open_mband(GtkWidget *chooser, gpointer user_data)
 {
 	char *fn, *fnd;
 	struct file_action *fa = user_data;
 	gpointer mwin;
 
-	fn = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fa->filesel));
+	fn = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
 	g_return_if_fail(fn != NULL);
 
 	add_to_mband(fa->window, fn);
@@ -478,21 +493,23 @@ static void open_mband(GtkFileSelection *selector, gpointer user_data)
 	mwin = gtk_object_get_data(GTK_OBJECT(fa->window), "im_window");
 	if (mwin != NULL)
 		set_last_open(mwin, "last_open_mband", fnd = strdup(fn));
-	else 
+	else
 		set_last_open(fa->window, "last_open_mband", fnd = strdup(fn));
 	d3_printf("lastopen_mband set to %s\n", fnd);
 	gtk_widget_queue_draw(fa->window);
+
+	g_free(fn);
 }
 
 
 /* open a gsc2 file and load stars; star that have v photometry are
  * marked as object, while the other are made field stars */
-static void open_gsc2(GtkFileSelection *selector, gpointer user_data) 
+static void open_gsc2(GtkWidget *chooser, gpointer user_data)
 {
 	char *fn, *fnd;
 	struct file_action *fa = user_data;
 
-	fn = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fa->filesel));
+	fn = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
 	g_return_if_fail(fn != NULL);
 
 	load_gsc2_to_window(fa->window, fn);
@@ -500,16 +517,18 @@ static void open_gsc2(GtkFileSelection *selector, gpointer user_data)
 	set_last_open(fa->window, "last_open_etc", fnd = strdup(fn));
 	d3_printf("lastopen_etc set to %s\n", fnd);
 	gtk_widget_queue_draw(fa->window);
+
+	g_free(fn);
 }
 
 
 /* set the entry with the file name */
-static void set_entry(GtkFileSelection *selector, gpointer user_data) 
+static void set_entry(GtkWidget *chooser, gpointer user_data)
 {
 	char *fn, *fnd;
 	struct file_action *fa = user_data;
 
-	fn = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fa->filesel));
+	fn = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
 	g_return_if_fail(fn != NULL);
 
 	gtk_entry_set_text(GTK_ENTRY(fa->data), fn);
@@ -518,62 +537,49 @@ static void set_entry(GtkFileSelection *selector, gpointer user_data)
 
 	set_last_open(fa->window, "last_open_fits", fnd = strdup(fn));
 	d3_printf("lastopen_fits set to %s\n", fnd);
+
+	g_free(fn);
 }
 
 
 /* get the file list and call the callback in fa->data */
-static void get_list(GtkFileSelection *selector, gpointer user_data) 
+static void get_list(GtkWidget *chooser, gpointer user_data)
 {
-	GtkWidget *clist;
-	GList *sl=NULL;
-	GList *fl=NULL;
-	char *text;
-	char *fn, *fo, *fnd;
-	char *dn;
 	struct file_action *fa = user_data;
 	get_file_list_type cb;
+	char *fn = NULL;
+	GSList *fl, *sl;
+	
 
-	fn = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fa->filesel));
-	g_return_if_fail(fn != NULL);
+	fl = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(chooser));
+	g_return_if_fail(fl != NULL);
 
-	fnd = strdup(fn);
-	dn = dirname(fn);
-
-	clist = GTK_FILE_SELECTION(fa->filesel)->file_list;
-	sl = GTK_CLIST(clist) -> selection;
-	while(sl != NULL) {
-		if (gtk_clist_get_text(GTK_CLIST(clist), GPOINTER_TO_INT(sl->data), 0, &text)) {
-			if ((asprintf(&fo, "%s/%s", dn, text)) > 0)
-				fl = g_list_append(fl, fo);
-		}
-		sl = g_list_next(sl);
-	}
 	cb = fa->data;
 	if (cb)
 		(* cb)(fl, fa->window);
-	if (fl != NULL) {
-		fnd = strdup(fl->data);
-		set_last_open(fa->window, "last_open_fits", fnd);
-		d3_printf("lastopen_fits set to %s\n", fnd);
-	} 
-	sl = fl; 		/* we reuse sl here */
-	while(sl != NULL) {
-		free(sl->data);
-		sl = g_list_next(sl);
-	}
-	g_list_free(fl);
 
+
+	for (sl = fl; sl; sl = sl->next) {
+		if (!sl->next) {
+			fn = strdup(sl->data);
+			set_last_open(fa->window, "last_open_fits", fn);
+			d3_printf("lastopen_fits set to %s\n", fn);
+		}
+
+		g_free(sl->data);
+	}
+	g_slist_free(fl);
 }
 
 /* get the file name and call the callback in fa->data */
-static void get_file(GtkFileSelection *selector, gpointer user_data) 
+static void get_file(GtkWidget *chooser, gpointer user_data)
 {
 	char *fn, *fnd;
 	struct file_action *fa = user_data;
 	get_file_type cb;
 	gpointer mwin;
 
-	fn = gtk_file_selection_get_filename (GTK_FILE_SELECTION(fa->filesel));
+	fn = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
 	g_return_if_fail(fn != NULL);
 
 	fnd = strdup(fn);
@@ -584,9 +590,11 @@ static void get_file(GtkFileSelection *selector, gpointer user_data)
 	mwin = gtk_object_get_data(GTK_OBJECT(fa->window), "im_window");
 	if (mwin != NULL)
 		set_last_open(mwin, "last_open_etc", fnd = strdup(fn));
-	else 
+	else
 		set_last_open(fa->window, "last_open_etc", fnd = strdup(fn));
 	d3_printf("lastopen_etc set to %s\n", fnd);
+
+	g_free(fn);
 }
 
 
@@ -601,60 +609,63 @@ void file_select_to_entry(gpointer data, GtkWidget *entry, char *title, char *fi
 			  int activate)
 {
 	GtkWidget *window = data;
-	GtkWidget *filesel;
+	GtkWidget *chooser;
 	struct file_action *fa;
-
 	char *lastopen;
-	lastopen = gtk_object_get_data(GTK_OBJECT(window), "last_open_fits");
+
 	fa = calloc(1, sizeof(struct file_action));
+
+	fa->action = set_entry;
+
 	fa->window = window;
 	fa->data = entry;
 	fa->arg1 = activate;
 
 	if (title == NULL)
-		filesel = create_file_selection("Select file", set_entry, fa);
+		chooser = create_file_chooser("Select file", F_OPEN, fa);
 	else
-		filesel = create_file_selection(title, set_entry, fa);
+		chooser = create_file_chooser(title, F_OPEN, fa);
 
 	strncpy(fa->file_filter, filter, FILE_FILTER_SZ);
-	fa->filesel = filesel;
-	if (lastopen != NULL) {
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION(filesel), 
-						 lastopen);
-	}
+	fa->chooser = chooser;
+
+	lastopen = gtk_object_get_data(GTK_OBJECT(window), "last_open_fits");
+	if (lastopen)
+		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(chooser), lastopen);
 }
 
-/* run the file selector configured to call cb on ok with the 
- * selected file; arg is passed on to the called function 
+/* run the file selector configured to call cb on ok with the
+ * selected file; arg is passed on to the called function
  * the file selector is run in a modal window */
 void file_select(gpointer data, char *title, char *filter,
 		 get_file_type cb, unsigned arg)
 {
 	GtkWidget *window = data;
-	GtkWidget *filesel;
+	GtkWidget *chooser;
 	struct file_action *fa;
-
 	char *lastopen;
-	lastopen = gtk_object_get_data(GTK_OBJECT(window), "last_open_etc");
+
+
 	fa = calloc(1, sizeof(struct file_action));
+	fa->action = get_file;
+
 	fa->window = window;
 	fa->data = cb;
 	fa->arg1 = arg;
 
 	if (title == NULL)
-		filesel = create_file_selection("Select file", get_file, fa);
+		chooser = create_file_chooser("Select file", F_OPEN, fa);
 	else
-		filesel = create_file_selection(title, get_file, fa);
+		chooser = create_file_chooser(title, F_OPEN, fa);
 
 	if (filter != NULL)
 		strncpy(fa->file_filter, filter, FILE_FILTER_SZ);
 	else
 		fa->file_filter[0] = 0;
-	fa->filesel = filesel;
-	if (lastopen != NULL) {
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION(filesel), 
-						 lastopen);
-	}
+	fa->chooser = chooser;
+	lastopen = gtk_object_get_data(GTK_OBJECT(window), "last_open_etc");
+	if (lastopen)
+		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(chooser), lastopen);
 }
 
 
@@ -665,121 +676,132 @@ void file_select_list(gpointer data, char *title, char *filter,
 		      get_file_list_type cb)
 {
 	GtkWidget *window = data;
-	GtkWidget *filesel;
+	GtkWidget *chooser;
 	struct file_action *fa;
-
 	char *lastopen;
-	lastopen = gtk_object_get_data(GTK_OBJECT(window), "last_open_fits");
+
 	fa = calloc(1, sizeof(struct file_action));
+
+	fa->action = get_list;
 	fa->window = window;
 	fa->data = cb;
 
 	if (title == NULL)
-		filesel = create_file_selection("Select file", get_list, fa);
+		chooser = create_file_chooser("Select file", F_OPEN, fa);
 	else
-		filesel = create_file_selection(title, get_list, fa);
+		chooser = create_file_chooser(title, F_OPEN, fa);
 
 	strncpy(fa->file_filter, filter, FILE_FILTER_SZ);
-	fa->filesel = filesel;
-	if (lastopen != NULL) {
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION(filesel), 
-						 lastopen);
-	}
+	fa->chooser = chooser;
+
+	lastopen = gtk_object_get_data(GTK_OBJECT(window), "last_open_etc");
+	if (lastopen)
+		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(chooser), lastopen);
 }
 
 
 void file_popup_cb(gpointer data, guint action, GtkWidget *menu_item)
 {
 	GtkWidget *window = data;
-	GtkWidget *filesel;
+	GtkWidget *chooser;
 	struct image_channel *channel;
 	struct file_action *fa;
 	char *fn;
 	char *lastopen;
+
+	fa = calloc(1, sizeof(struct file_action));
+	fa->window = window;
+
 	switch(action) {
 	case FILE_OPEN:
-		lastopen = gtk_object_get_data(GTK_OBJECT(window), "last_open_fits");
-		fa = calloc(1, sizeof(struct file_action));
-		fa->window = window;
-		filesel = create_file_selection("Select fits file to open", open_fits, fa);
-		fa->filesel = filesel;
+		fa->action = open_fits;
+
+		chooser = create_file_chooser("Select fits file to open", F_OPEN, fa);
+		fa->chooser = chooser;
 		strncpy(fa->file_filter, "*.fits*", FILE_FILTER_SZ);
-		if (lastopen != NULL) {
-			gtk_file_selection_set_filename (GTK_FILE_SELECTION(filesel), 
-							 lastopen);
-		}
+
+		lastopen = gtk_object_get_data(GTK_OBJECT(window), "last_open_fits");
+		if (lastopen)
+			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(chooser), lastopen);
 		break;
+
 	case FILE_OPEN_RCP:
+		fa->action = open_rcp;
+
+		chooser = create_file_chooser("Select fits file to open", F_OPEN, fa);
+		fa->chooser = chooser;
+
 		lastopen = gtk_object_get_data(GTK_OBJECT(window), "last_open_rcp");
-		fa = calloc(1, sizeof(struct file_action));
-		fa->window = window;
-		filesel = create_file_selection("Select fits file to open", open_rcp, fa);
-		fa->filesel = filesel;
-		if (lastopen != NULL) {
-			gtk_file_selection_set_filename (GTK_FILE_SELECTION(filesel), 
-							 lastopen);
-		}
-//		gtk_file_selection_complete(GTK_FILE_SELECTION(filesel), "*.fits*");
+		if (lastopen)
+			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(chooser), lastopen);
 		break;
+
 	case FILE_ADD_TO_MBAND:
+		fa->action = open_mband;
+
+		chooser = create_file_chooser("Select report file to open", F_OPEN, fa);
+		fa->chooser = chooser;
+
 		lastopen = gtk_object_get_data(GTK_OBJECT(window), "last_open_mband");
-		fa = calloc(1, sizeof(struct file_action));
-		fa->window = window;
-		filesel = create_file_selection("Select report file to open", open_mband, fa);
-		fa->filesel = filesel;
-		if (lastopen != NULL) {
-			gtk_file_selection_set_filename (GTK_FILE_SELECTION(filesel), 
-							 lastopen);
-		}
-//		gtk_file_selection_complete(GTK_FILE_SELECTION(filesel), "*.fits*");
+		if (lastopen)
+			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(chooser), lastopen);
 		break;
+
 	case FILE_LOAD_GSC2:
+		fa->action = open_gsc2;
+
+		chooser = create_file_chooser("Select GSC-2 data file to open", F_OPEN, fa);
+		fa->chooser = chooser;
+
 		lastopen = gtk_object_get_data(GTK_OBJECT(window), "last_open_etc");
-		fa = calloc(1, sizeof(struct file_action));
-		fa->window = window;
-		filesel = create_file_selection("Select GSC-2 data file to open", open_gsc2, fa);
-		fa->filesel = filesel;
-		if (lastopen != NULL) {
-			gtk_file_selection_set_filename (GTK_FILE_SELECTION(filesel), 
-							 lastopen);
-		}
-//		gtk_file_selection_complete(GTK_FILE_SELECTION(filesel), "*.fits*");
+		if (lastopen)
+			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(chooser), lastopen);
 		break;
+
 	case FILE_EXPORT_PNM8:
 	case FILE_EXPORT_PNM16:
+		fa->action = export_pnm;
+		
 		channel = gtk_object_get_data(GTK_OBJECT(window), "i_channel");
 		if (channel == NULL || channel->fr == NULL) {
 			err_printf_sb2(window, "No frame loaded");
 			error_beep();
+			free(fa);
 			return ;
 		}
+
 		fn = force_extension(channel->fr->name, "pnm");
-		fa = calloc(1, sizeof(struct file_action));
-		fa->window = window;
 		fa->data = channel;
-//		fa->arg1 = (action == FILE_EXPORT_PNM16);
-		filesel = create_file_selection("Select pnm file name", export_pnm, fa);
-		fa->filesel = filesel;
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION(filesel), fn);
+		fa->arg1 = (action == FILE_EXPORT_PNM16);
+
+		chooser = create_file_chooser("Select pnm file name", F_SAVE, fa);
+		fa->chooser = chooser;
+		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(chooser), fn);
 		free(fn);
 		break;
+
 	case FILE_SAVE_AS:
+		fa->action = save_fits;
+		
 		channel = gtk_object_get_data(GTK_OBJECT(window), "i_channel");
 		if (channel == NULL || channel->fr == NULL) {
 			err_printf_sb2(window, "No frame");
 			error_beep();
+			free(fa);
 			return ;
 		}
+
+		fa->data = channel;		
 		wcs_to_fits_header(channel->fr);
 		fn = channel->fr->name;
-		fa = calloc(1, sizeof(struct file_action));
-		fa->window = window;
-		fa->data = channel;
-		filesel = create_file_selection("Select output file name", save_fits, fa);
-		fa->filesel = filesel;
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION(filesel), fn);
+		
+		chooser = create_file_chooser("Select output file name", F_SAVE, fa);
+		fa->chooser = chooser;
+		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER(chooser), fn);
 		break;
+
 	default:
+		free(fa);
 		err_printf("unknown action %d in file_popup_cb\n", action);
 	}
 
@@ -814,7 +836,7 @@ char *find_file_in_path(char *pattern, char *path)
 			return fn;
 		}
 		globfree(&gl);
-		dir = strtok(NULL, ":");	
+		dir = strtok(NULL, ":");
 	}
 	return NULL;
 }
@@ -859,61 +881,37 @@ FILE * open_expanded(char *path, char *mode)
 
 /* modally prompt for a file name */
 
-/* modal dialog */
-
-static void fsel_yes( GtkWidget *widget, gpointer data )
-{
-	char **retv;
-	char *fn;
-
-	retv = gtk_object_get_data(GTK_OBJECT(data), "return_val");
-	if (retv != NULL) {
-		fn = gtk_file_selection_get_filename (GTK_FILE_SELECTION(data));
-		*retv = strdup(fn);
-	}
-	gtk_widget_destroy(data);
-}
-
-static void fsel_cancel( GtkWidget *widget, gpointer data )
-{
-	gtk_widget_destroy(data);
-}
-
-static void fsel_close( GtkWidget *widget, gpointer data )
-{
-	gtk_main_quit();
-}
-
 /* prompt the user to select a file; return the malloced file name or NULL */
-
 char * modal_file_prompt(char *title, char *initial)
 {
-	GtkWidget *file_selector;
+	GtkWidget *chooser;
 	char *retval = NULL;
 
-	file_selector = gtk_file_selection_new(title);
-	gtk_window_set_position (GTK_WINDOW (file_selector), GTK_WIN_POS_CENTER);
-	gtk_window_set_modal (GTK_WINDOW (file_selector), TRUE);
+	chooser = gtk_file_chooser_dialog_new(title, NULL,
+					      GTK_FILE_CHOOSER_ACTION_OPEN,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+					      NULL);
+
+	gtk_window_set_title(GTK_WINDOW(chooser), title);
+
+	gtk_window_set_position(GTK_WINDOW(chooser), GTK_WIN_POS_CENTER);
+	gtk_window_set_modal(GTK_WINDOW(chooser), TRUE);
+
+	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(chooser), TRUE);
 
-	gtk_clist_set_selection_mode (GTK_CLIST (GTK_FILE_SELECTION(file_selector)->file_list), 
-				     GTK_SELECTION_EXTENDED);
 	if (initial)
-		gtk_file_selection_set_filename (GTK_FILE_SELECTION(file_selector), initial);
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
-			    "clicked", GTK_SIGNAL_FUNC (fsel_yes), file_selector);
-//	gtk_signal_connect (GTK_OBJECT (file_selector), "delete_event",
-//			    GTK_SIGNAL_FUNC (fsel_cancel), file_selector);
-	gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->cancel_button),
-				   "clicked", GTK_SIGNAL_FUNC (fsel_cancel), file_selector);
-	gtk_object_set_data(GTK_OBJECT(file_selector), "return_val", &retval); 
-	gtk_signal_connect (GTK_OBJECT (file_selector), "destroy",
-			    GTK_SIGNAL_FUNC (fsel_close), file_selector);
-	if (title != NULL) {
-		gtk_window_set_title(GTK_WINDOW(file_selector), title);
+		gtk_file_chooser_select_filename(GTK_FILE_CHOOSER(chooser), initial);
+
+	if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_ACCEPT) {
+		char *filename;
+		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
+
+		retval = strdup(filename);
+		g_free(filename);
 	}
-	gtk_widget_show(file_selector);
-	gtk_main();
+
+	gtk_widget_destroy(GTK_WIDGET(chooser));
+
 	return retval;
 }
-
-
