@@ -1,7 +1,7 @@
 #ifndef _FILEGUI_H_
 #define _FILEGUI_H_
 
-typedef void (* get_file_list_type) (GList *fl, gpointer window);
+typedef void (* get_file_list_type) (GSList *fl, gpointer window);
 typedef void (* get_file_type) (char *fn, gpointer window, unsigned arg);
 
 
