@@ -1,29 +1,29 @@
 /*******************************************************************************
   Copyright(c) 2000 - 2003 Radu Corlan. All rights reserved.
-  
-  This program is free software; you can redistribute it and/or modify it 
-  under the terms of the GNU General Public License as published by the Free 
-  Software Foundation; either version 2 of the License, or (at your option) 
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the Free
+  Software Foundation; either version 2 of the License, or (at your option)
   any later version.
-  
-  This program is distributed in the hope that it will be useful, but WITHOUT 
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
-  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
+
+  This program is distributed in the hope that it will be useful, but WITHOUT
+  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.
-  
+
   You should have received a copy of the GNU General Public License along with
-  this program; if not, write to the Free Software Foundation, Inc., 59 
+  this program; if not, write to the Free Software Foundation, Inc., 59
   Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
   The full GNU General Public License is included in this distribution in the
   file called LICENSE.
-  
+
   Contact Information: radu@corlan.net
 *******************************************************************************/
 
-// gcx.c: main file for camera control program 
-// $Revision: 1.23 $
-// $Date: 2007/07/04 13:26:50 $
+// gcx.c: main file for camera control program
+// $Revision: 1.34 $
+// $Date: 2009/09/27 15:29:51 $
 
 #define _GNU_SOURCE
 
@@ -115,7 +115,7 @@ int load_params_rc(void)
 	uid_t my_uid;
 	struct passwd *passwd;
 	char *rcname;
-	int ret;
+	int ret = -1;
 
 	my_uid = getuid();
 	passwd = getpwuid(my_uid);
@@ -123,9 +123,10 @@ int load_params_rc(void)
 		err_printf("load_params_rc: cannot determine home directoy\n");
 		return -1;
 	}
-	asprintf(&rcname, "%s/.gcxrc", passwd->pw_dir);
-	ret = load_par_file(rcname, PAR_NULL);
-	free(rcname);
+	if (-1 != asprintf(&rcname, "%s/.gcxrc", passwd->pw_dir)) {
+		ret = load_par_file(rcname, PAR_NULL);
+		free(rcname);
+	}
 	return ret;
 }
 
@@ -134,7 +135,7 @@ int save_params_rc(void)
 	uid_t my_uid;
 	struct passwd *passwd;
 	char *rcname;
-	int ret;
+	int ret = -1;
 
 	my_uid = getuid();
 	passwd = getpwuid(my_uid);
@@ -142,9 +143,10 @@ int save_params_rc(void)
 		err_printf("save_params_rc: cannot determine home directoy\n");
 		return -1;
 	}
-	asprintf(&rcname, "%s/.gcxrc", passwd->pw_dir);
-	ret = save_par_file(rcname, PAR_NULL);
-	free(rcname);
+	if (-1 != asprintf(&rcname, "%s/.gcxrc", passwd->pw_dir)) {
+		ret = save_par_file(rcname, PAR_NULL);
+		free(rcname);
+	}
 	return ret;
 }
 
@@ -155,7 +157,7 @@ static void recipe_file_convert(char *rcf, char *outf)
 	if (rcf[0] != 0) {
 		infp = fopen(rcf, "r");
 		if (infp == NULL) {
-			err_printf("Cannot open file %s for reading\n%s\n", 
+			err_printf("Cannot open file %s for reading\n%s\n",
 				   rcf, strerror(errno));
 			exit(1);
 		}
@@ -163,7 +165,7 @@ static void recipe_file_convert(char *rcf, char *outf)
 	if (outf[0] != 0) {
 		outfp = fopen(outf, "w");
 		if (outfp == NULL) {
-			err_printf("Cannot open file %s for writing\n%s\n", 
+			err_printf("Cannot open file %s for writing\n%s\n",
 				   outf, strerror(errno));
 			fclose(infp);
 			exit(1);
@@ -203,7 +205,7 @@ static void recipe_aavso_convert(char *rcf, char *outf)
 	if (rcf[0] != 0) {
 		infp = fopen(rcf, "r");
 		if (infp == NULL) {
-			err_printf("Cannot open file %s for reading\n%s\n", 
+			err_printf("Cannot open file %s for reading\n%s\n",
 				   rcf, strerror(errno));
 			exit(1);
 		}
@@ -211,7 +213,7 @@ static void recipe_aavso_convert(char *rcf, char *outf)
 	if (outf[0] != 0) {
 		outfp = fopen(outf, "w");
 		if (outfp == NULL) {
-			err_printf("Cannot open file %s for writing\n%s\n", 
+			err_printf("Cannot open file %s for writing\n%s\n",
 				   outf, strerror(errno));
 			fclose(infp);
 			exit(1);
@@ -252,7 +254,7 @@ static void report_convert(char *rcf, char *outf)
 	if (rcf[0] != 0) {
 		infp = fopen(rcf, "r");
 		if (infp == NULL) {
-			err_printf("Cannot open file %s for reading\n%s\n", 
+			err_printf("Cannot open file %s for reading\n%s\n",
 				   rcf, strerror(errno));
 			exit(1);
 		}
@@ -260,7 +262,7 @@ static void report_convert(char *rcf, char *outf)
 	if (outf[0] != 0) {
 		outfp = fopen(outf, "w");
 		if (outfp == NULL) {
-			err_printf("Cannot open file %s for writing\n%s\n", 
+			err_printf("Cannot open file %s for writing\n%s\n",
 				   outf, strerror(errno));
 			if (infp)
 				fclose(infp);
@@ -277,7 +279,7 @@ static void report_convert(char *rcf, char *outf)
 		report_to_table(stdin, stdout, NULL);
 	if (infp)
 		fclose(infp);
-	if (outfp) 
+	if (outfp)
 		fclose(outfp);
 	exit (0);
 }
@@ -290,7 +292,7 @@ static void recipe_merge(char *rcf, char *mergef, char *outf, double mag_limit)
 	if (rcf[0] != '-' || rcf[1] != 0) {
 		infp = fopen(rcf, "r");
 		if (infp == NULL) {
-			err_printf("Cannot open file %s for reading\n%s\n", 
+			err_printf("Cannot open file %s for reading\n%s\n",
 				   rcf, strerror(errno));
 			exit(1);
 		}
@@ -300,7 +302,7 @@ static void recipe_merge(char *rcf, char *mergef, char *outf, double mag_limit)
 	if (mergef[0] != '-' || mergef[1] != 0) {
 		mfp = fopen(mergef, "r");
 		if (mfp == NULL) {
-			err_printf("Cannot open file %s for reading\n%s\n", 
+			err_printf("Cannot open file %s for reading\n%s\n",
 				   mergef, strerror(errno));
 			exit(1);
 		}
@@ -310,7 +312,7 @@ static void recipe_merge(char *rcf, char *mergef, char *outf, double mag_limit)
 	if (outf[0] != 0) {
 		outfp = fopen(outf, "w");
 		if (outfp == NULL) {
-			err_printf("Cannot open file %s for writing\n%s\n", 
+			err_printf("Cannot open file %s for writing\n%s\n",
 				   outf, strerror(errno));
 			exit(1);
 		}
@@ -319,7 +321,7 @@ static void recipe_merge(char *rcf, char *mergef, char *outf, double mag_limit)
 	}
 	if (merge_rcp(infp, mfp, outfp, mag_limit) >= 0)
 		exit (0);
-	else 
+	else
 		exit (1);
 }
 
@@ -330,7 +332,7 @@ static void recipe_set_tobj(char *rcf, char *obj, char *outf, double mag_limit)
 	if (rcf[0] != '-' || rcf[1] != 0) {
 		infp = fopen(rcf, "r");
 		if (infp == NULL) {
-			err_printf("Cannot open file %s for reading\n%s\n", 
+			err_printf("Cannot open file %s for reading\n%s\n",
 				   rcf, strerror(errno));
 			exit(1);
 		}
@@ -340,7 +342,7 @@ static void recipe_set_tobj(char *rcf, char *obj, char *outf, double mag_limit)
 	if (outf[0] != 0) {
 		outfp = fopen(outf, "w");
 		if (outfp == NULL) {
-			err_printf("Cannot open file %s for writing\n%s\n", 
+			err_printf("Cannot open file %s for writing\n%s\n",
 				   outf, strerror(errno));
 			exit(1);
 		}
@@ -349,7 +351,7 @@ static void recipe_set_tobj(char *rcf, char *obj, char *outf, double mag_limit)
 	}
 	if (rcp_set_target(infp, obj, outfp, mag_limit) >= 0)
 		exit (0);
-	else 
+	else
 		exit (1);
 }
 
@@ -363,7 +365,7 @@ static void catalog_file_convert(char *rcf, char *outf, double mag_limit)
 	}
 	if (outfp)
 		nc = convert_catalog(stdin, outfp, rcf, mag_limit);
-	else 
+	else
 		nc = convert_catalog(stdin, stdout, rcf, mag_limit);
 	if (nc < 0) {
 		err_printf("Error converting catalog table\n");
@@ -389,7 +391,7 @@ static void mb_reduce(char *mband, char *outf)
 	if (outf[0] != 0) {
 		outfp = fopen(outf, "w");
 		if (outfp == NULL) {
-			err_printf("Cannot open file %s for writing\n%s\n", 
+			err_printf("Cannot open file %s for writing\n%s\n",
 				   outf, strerror(errno));
 			exit(1);
 		}
@@ -397,7 +399,7 @@ static void mb_reduce(char *mband, char *outf)
 	if (mband[0] != '-' || mband[1] != 0) {
 		infp = fopen(mband, "r");
 		if (infp == NULL) {
-			err_printf("Cannot open file %s for reading\n%s\n", 
+			err_printf("Cannot open file %s for reading\n%s\n",
 				   mband, strerror(errno));
 			exit(1);
 		}
@@ -409,12 +411,15 @@ static void mb_reduce(char *mband, char *outf)
 }
 
 
-static void add_image_file_to_list(struct image_file_list *imfl, char *filename)
+static void add_image_file_to_list(struct image_file_list *imfl, char *filename, int flags)
 {
 	struct image_file *imf;
 
 	imf = image_file_new();
+
 	imf->filename = strdup(filename);
+	imf->flags = flags;
+
 	imfl->imlist = g_list_append(imfl->imlist, imf);
 }
 
@@ -441,14 +446,14 @@ static int set_wcs_from_object (struct ccd_frame *fr, char *name, double spp)
 
 }
 
-static void make_tycho_rcp(char *obj, double tycrcp_box, char *outf, double mag_limit) 
+static void make_tycho_rcp(char *obj, double tycrcp_box, char *outf, double mag_limit)
 {
 	FILE *of = NULL;
 	if (obj[0] == 0) {
 		err_printf("Please specify an object (with -j)\n");
 		exit(1);
 	}
-	if (outf[0] != 0) 
+	if (outf[0] != 0)
 		of = fopen(outf, "w");
 	if (of == NULL)
 		of = stdout;
@@ -457,14 +462,14 @@ static void make_tycho_rcp(char *obj, double tycrcp_box, char *outf, double mag_
 	exit(1);
 }
 
-static void cat_rcp(char *obj, double cat_box, char *outf, double mag_limit) 
+static void cat_rcp(char *obj, double cat_box, char *outf, double mag_limit)
 {
 	FILE *of = NULL;
 	if (obj[0] == 0) {
 		err_printf("Please specify an object (with -j)\n");
 		exit(1);
 	}
-	if (outf[0] != 0) 
+	if (outf[0] != 0)
 		of = fopen(outf, "w");
 	if (of == NULL)
 		of = stdout;
@@ -474,12 +479,38 @@ static void cat_rcp(char *obj, double cat_box, char *outf, double mag_limit)
 }
 
 
+
+int extract_bad_pixels(char *badpix, char *outf)
+{
+	struct image_file *imf;
+	struct bad_pix_map *map;
+
+	imf = image_file_new();
+	imf->filename = strdup(badpix);
+
+	if (load_image_file(imf) < 0)
+		goto out_err;
+
+	map = calloc(1, sizeof(struct bad_pix_map));
+	map->filename = strdup(outf);
+
+	if (find_bad_pixels(map, imf->fr, P_DBL(CCDRED_BADPIX_SIGMAS)) < 0)
+		goto out_err;
+
+	save_bad_pix(map);
+
+out_err:
+	exit(0);
+}
+
+
+
 static struct ccd_frame *make_blank_obj_fr(char *obj)
 {
 	struct ccd_frame *fr;
 	fr = new_frame(P_INT(FILE_NEW_WIDTH), P_INT(FILE_NEW_HEIGHT));
-	set_wcs_from_object(fr, obj, P_DBL(FILE_NEW_SECPERPIX)) == 0;
-	return fr;	
+	set_wcs_from_object(fr, obj, P_DBL(FILE_NEW_SECPERPIX));
+	return fr;
 }
 
 #define NUM_AIRMASSES 8
@@ -490,13 +521,13 @@ static void print_scint_table(char *apert)
 {
 	double d;
 	int i, j;
-	
+
 	d = strtod(apert, NULL);
 	if (d == 0) {
 		err_printf("invalid aperture %s\n", apert);
 		return;
 	}
-	
+
 	printf("\n    Scintillation table for D=%.0fcm, sea level\n\n", d);
 	printf("exp/am");
 	for (i=0; i<NUM_AIRMASSES; i++)
@@ -527,6 +558,7 @@ int main(int ac, char **av)
 	char of[1024]=""; /* obsfile */
 	char obj[1024] = ""; /* object */
 	char mband[1024] = ""; /* argument to mband */
+	char badpixf[1024] = ""; /* dark for badpix extraction */
 	int to_pnm = 0;
 	int run_phot = 0;
 	int convert_recipe = 0;
@@ -535,6 +567,7 @@ int main(int ac, char **av)
 	int no_reduce = 0;
 	int save_internal_cat = 0;
 	int update_files = 0;
+	int extract_badpix = 0;
 	struct ccd_reduce *ccdr = NULL;
 	struct image_file_list *imfl = NULL; /* list of frames to load / process */
 	float mag_limit = 99.0; /* mag limit when generating rcp and cat files */
@@ -556,13 +589,16 @@ int main(int ac, char **av)
 
 	GtkWidget *window;
 
-	char *shortopts = "D:p:hP:V:vo:id:b:f:B:M:A:O:usa:T:S:nG:Nj:F";
+	char *shortopts = "D:p:hP:V:vo:id:b:f:B:M:A:O:usa:T:S:nG:Nj:Fc";
 	struct option longopts[] = {
 		{"debug", required_argument, NULL, 'D'},
 		{"dark", required_argument, NULL, 'd'},
 		{"bias", required_argument, NULL, 'b'},
 		{"flat", required_argument, NULL, 'f'},
+
 		{"badpix", required_argument, NULL, 'B'},
+		{"extract-badpix", required_argument, NULL, 'X'},
+
 		{"add-bias", required_argument, NULL, 'A'},
 		{"multiply", required_argument, NULL, 'M'},
 		{"gaussian-blur", required_argument, NULL, 'G'},
@@ -574,7 +610,6 @@ int main(int ac, char **av)
 		{"superflat", no_argument, NULL, 'F'},
 		{"align", required_argument, NULL, 'a'},
 		{"no-reduce", no_argument, NULL, 'N'},
-
 		{"set", required_argument, NULL, 'S'},
 
 		{"rcfile", required_argument, NULL, 'r'},
@@ -605,9 +640,14 @@ int main(int ac, char **av)
 		{"multi-band-reduce", required_argument, NULL, '^'},
 		{"options-doc", no_argument, NULL, '`'},
 		{"scintillation", required_argument, NULL, ';'},
+
+		{"demosaic", optional_argument, NULL, 'c' },
+
 		{NULL, 0, NULL, 0}
 	};
 
+	setenv("LANG", "C", 1);
+
 	init_ptable();
 	load_params_rc();
 	gtk_init (&ac, &av);
@@ -807,7 +847,7 @@ int main(int ac, char **av)
 			break;
 		case 'G':
 			v = strtod(optarg, &endp);
-			if (endp != optarg) {		
+			if (endp != optarg) {
 				if (ccdr == NULL)
 					ccdr = ccd_reduce_new();
 				ccdr->blurv = v;
@@ -817,7 +857,7 @@ int main(int ac, char **av)
 			break;
 		case 'M':
 			v = strtod(optarg, &endp);
-			if (endp != optarg) {		
+			if (endp != optarg) {
 				if (ccdr == NULL)
 					ccdr = ccd_reduce_new();
 				ccdr->mulv = v;
@@ -827,7 +867,7 @@ int main(int ac, char **av)
 			break;
 		case 'A':
 			v = strtod(optarg, &endp);
-			if (endp != optarg) {		
+			if (endp != optarg) {
 				if (ccdr == NULL)
 					ccdr = ccd_reduce_new();
 				ccdr->addv = v;
@@ -835,6 +875,27 @@ int main(int ac, char **av)
 			}
 			batch = 1;
 			break;
+
+		case 'B':
+			if (ccdr == NULL)
+				ccdr = ccd_reduce_new();
+			ccdr->bad_pix_map = calloc(1, sizeof(struct bad_pix_map));
+			ccdr->bad_pix_map->filename = strdup(optarg);
+			ccdr->ops |= IMG_OP_BADPIX;
+			batch = 1;
+			break;
+
+		case 'X':
+			extract_badpix = 1;
+			strncpy(badpixf, optarg, 1023);
+			break;
+
+		case 'c':
+			if (ccdr == NULL)
+				ccdr = ccd_reduce_new();
+			ccdr->ops |= IMG_OP_DEMOSAIC;
+			batch = 1;
+			break;
 		}
 	}
 
@@ -845,6 +906,10 @@ int main(int ac, char **av)
 		load_par_file(rc_fn, PAR_NULL);
 	}
 
+	if (extract_badpix) {
+		extract_bad_pixels(badpixf, outf); /* does not return */
+	}
+
 	if (mband[0] != 0) {
 		mb_reduce(mband, outf); /* does not return */
 	}
@@ -879,7 +944,7 @@ int main(int ac, char **av)
 		if (outf[0] != 0) {
 			outfp = fopen(outf, "w");
 			if (outfp == NULL) {
-				err_printf("Cannot open file %s for writing\n%s\n", 
+				err_printf("Cannot open file %s for writing\n%s\n",
 					   outf, strerror(errno));
 				exit(1);
 			}
@@ -931,7 +996,7 @@ int main(int ac, char **av)
 	if (ac > optind) { /* we build the image list */
 		imfl = image_file_list_new();
 		for (i = optind; i < ac; i++)
-			add_image_file_to_list(imfl, av[i]);
+			add_image_file_to_list(imfl, av[i], 0);
 	}
 
 	if (ccdr != NULL) {
@@ -951,7 +1016,7 @@ int main(int ac, char **av)
 				info_printf("no output file name: going interactive\n");
 				interactive = 1;
 			}
-		} 
+		}
 	}
 
 	if ((imfl != NULL) && (ccdr != NULL) && !no_reduce) {
@@ -976,8 +1041,9 @@ int main(int ac, char **av)
 
 	if ((ac > optind)) {
 		if (fr == NULL) {
-			fr = read_gz_fits_file(av[optind], P_STR(FILE_UNCOMPRESS),
-					       P_INT(FILE_UNSIGNED_FITS));
+			fr = read_image_file(av[optind], P_STR(FILE_UNCOMPRESS),
+					     P_INT(FILE_UNSIGNED_FITS),
+					     default_cfa[P_INT(FILE_DEFAULT_CFA)]);
 			if (fr) {
 				rescan_fits_exp(fr, &fr->exp);
 				rescan_fits_wcs(fr, &fr->fim);
@@ -989,7 +1055,7 @@ int main(int ac, char **av)
 		}
 		if (obj[0] != 0)
 			set_wcs_from_object(fr, obj, -P_DBL(WCS_SEC_PER_PIX));
-		
+
 		frame_to_channel(fr, window, "i_channel");
 		release_frame(fr);
 		if (to_pnm) {
@@ -1001,9 +1067,9 @@ int main(int ac, char **av)
 					exit(1);
 			}
 			if (outf[0] != 0) {
-				ret = channel_to_pnm_file(channel, NULL, outf);
+				ret = channel_to_pnm_file(channel, NULL, outf, 0);
 			} else {
-				ret = channel_to_pnm_file(channel, NULL, NULL);
+				ret = channel_to_pnm_file(channel, NULL, NULL, 0);
 			}
 			if (ret) {
 				err_printf("error writing pnm file\n");
@@ -1022,7 +1088,7 @@ int main(int ac, char **av)
 					output_file = fopen(outf, "a");
 					if (output_file == NULL)
 						err_printf("Cannot open file %s "
-							   "for appending\n%s\n", 
+							   "for appending\n%s\n",
 							   outf, strerror(errno));
 				}
 				if (output_file == NULL)
