@@ -28,8 +28,10 @@ char *get_header(fp,h)
 
 	lheader = atoi(w);
 	w += 4;
-	read(fp,w,lheader-4);
-	w[lheader] = '\0';
+	if (read(fp,w,lheader-4) > 0)
+		w[lheader] = '\0';
+	else
+		*w = '\0';
 
 	h->len = lheader;
 	h->vers = atoi(w);
