@@ -1,23 +1,23 @@
 /*******************************************************************************
   Copyright(c) 2000 - 2003 Radu Corlan. All rights reserved.
-  
-  This program is free software; you can redistribute it and/or modify it 
-  under the terms of the GNU General Public License as published by the Free 
-  Software Foundation; either version 2 of the License, or (at your option) 
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the Free
+  Software Foundation; either version 2 of the License, or (at your option)
   any later version.
-  
-  This program is distributed in the hope that it will be useful, but WITHOUT 
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
-  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
+
+  This program is distributed in the hope that it will be useful, but WITHOUT
+  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.
-  
+
   You should have received a copy of the GNU General Public License along with
-  this program; if not, write to the Free Software Foundation, Inc., 59 
+  this program; if not, write to the Free Software Foundation, Inc., 59
   Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
   The full GNU General Public License is included in this distribution in the
   file called LICENSE.
-  
+
   Contact Information: radu@corlan.net
 *******************************************************************************/
 
@@ -33,7 +33,6 @@
 #include "sourcesdraw.h"
 #include "params.h"
 #include "wcs.h"
-#include "camera.h"
 #include "cameragui.h"
 #include "filegui.h"
 #include "interface.h"
@@ -63,7 +62,7 @@ static void close_yes_no( GtkWidget *widget, gpointer data )
 
 
 /* create the modal y/n dialog and run the mainloop until it exits
- * display text as a bunch of labels. newlines inside the text 
+ * display text as a bunch of labels. newlines inside the text
  * return 0/1 for n/y, or -1 for errors */
 /* title is the window title; if NULL, 'gcx question' is used */
 int modal_yes_no(char *text, char *title)
@@ -72,7 +71,7 @@ int modal_yes_no(char *text, char *title)
 
 	GtkWidget *dialog, *label, *vbox;
 	dialog = create_yes_no();
-	gtk_object_set_data(GTK_OBJECT(dialog), "return_val", &retval); 
+	gtk_object_set_data(GTK_OBJECT(dialog), "return_val", &retval);
 	vbox = gtk_object_get_data(GTK_OBJECT(dialog), "vbox");
 	g_return_val_if_fail(vbox != NULL, 0);
 	gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
@@ -127,8 +126,8 @@ int modal_entry_prompt(char *text, char *title, char *initial, char **value)
 
 	GtkWidget *dialog, *label, *entry;
 	dialog = create_entry_prompt();
-	gtk_object_set_data(GTK_OBJECT(dialog), "return_val", &retval); 
-	gtk_object_set_data(GTK_OBJECT(dialog), "entry_val", value); 
+	gtk_object_set_data(GTK_OBJECT(dialog), "return_val", &retval);
+	gtk_object_set_data(GTK_OBJECT(dialog), "entry_val", value);
 	entry = gtk_object_get_data(GTK_OBJECT(dialog), "entry");
 	g_return_val_if_fail(entry != NULL, 0);
 	label = gtk_object_get_data(GTK_OBJECT(dialog), "label");
@@ -325,7 +324,7 @@ void star_rm_cb(gpointer data, guint action, GtkWidget *menu_item)
 	case STAR_RM_OFF:
 		ret = remove_off_frame_stars(window);
 		if (ret >= 0) {
-			info_printf_sb2(window, "stars", 5000, 
+			info_printf_sb2(window, "stars", 5000,
 				       " %d off-frame stars removed", ret);
 		}
 		break;
@@ -353,10 +352,11 @@ static GtkItemFactoryEntry image_popup_menu_items[] = {
 	{ "/_File",		NULL,         	NULL,  		0, "<Branch>" },
 /*  	{ "/File/tear",  	NULL,         	NULL,  		0, "<Tearoff>" }, */
 	{ "/File/_New Frame",	"<control>n", 	new_frame_cb, 	0, "<Item>" },
-	{ "/File/_Open Fits",	"<control>o", 	file_popup_cb, 	FILE_OPEN, "<Item>" },
+	{ "/File/_Open Image...",	"<control>o", 	file_popup_cb, 	FILE_OPEN, "<Item>" },
 	{ "/File/_Save Fits As...", "<control>s", file_popup_cb, FILE_SAVE_AS, "<Item>" },
 	{ "/File/_Export Image",	NULL, 	NULL, 	0, "<Branch>" },
 	{ "/File/_Export Image/_8-bit pnm", NULL, file_popup_cb, FILE_EXPORT_PNM8, "<Item>" },
+	{ "/File/_Export Image/_16-bit pnm", NULL, file_popup_cb, FILE_EXPORT_PNM16, "<Item>" },
 	{ "/File/sep",		NULL,         	NULL,  		0, "<Separator>" },
 	{ "/File/Load _Recipe",	"r", file_popup_cb, FILE_OPEN_RCP, "<Item>" },
 	{ "/File/_Create Recipe",	NULL, create_recipe_cb, 0, "<Item>" },
@@ -468,12 +468,12 @@ static GtkItemFactoryEntry image_popup_menu_items[] = {
 	{ "/Processing/_Multi-frame Reduction...", "<control>m", mband_open_cb, 1, "<Item>" },
 
 	{ "/_Catalogs",         	NULL,         	NULL, 		0, "<Branch>" },
-	{ "/Catalogs/Load Field Stars From _GSC Catalog", "g", find_stars_cb, 
+	{ "/Catalogs/Load Field Stars From _GSC Catalog", "g", find_stars_cb,
 	  ADD_STARS_GSC, "<Item>"},
-	{ "/Catalogs/Load Field Stars From _Tycho2 Catalog", "<control>t", find_stars_cb, 
+	{ "/Catalogs/Load Field Stars From _Tycho2 Catalog", "<control>t", find_stars_cb,
 	  ADD_STARS_TYCHO2, "<Item>"},
 	{ "/Catalogs/sep",		NULL,         	NULL,  		0, "<Separator>" },
-	{ "/Catalogs/Load Field Stars From GSC-_2 File", "<control>g", file_popup_cb, 
+	{ "/Catalogs/Load Field Stars From GSC-_2 File", "<control>g", file_popup_cb,
 	  FILE_LOAD_GSC2, "<Item>"},
 	{ "/Catalogs/sep",		NULL,         	NULL,  		0, "<Separator>" },
 	{ "/Catalogs/Download GSC-_ACT stars from CDS", NULL, cds_query_cb, QUERY_GSC_ACT, "<Item>"},
@@ -534,7 +534,7 @@ static gboolean image_clicked_cb(GtkWidget *w, GdkEventButton *event, gpointer d
 		}
 		if(menu) {
 			// d4_printf("menu=%p\n", menu);
-			gtk_menu_popup(menu, NULL, NULL, NULL, NULL, 
+			gtk_menu_popup(menu, NULL, NULL, NULL, NULL,
 				       event->button, event->time);
 		}
 	}
@@ -546,7 +546,7 @@ static gboolean image_clicked_cb(GtkWidget *w, GdkEventButton *event, gpointer d
 	}
 	if (event->button == 1) {
 		found = stars_under_click(GTK_WIDGET(data), event);
-		if (!(event->state & GDK_SHIFT_MASK) && (found == NULL)) 
+		if (!(event->state & GDK_SHIFT_MASK) && (found == NULL))
 			gsl_unselect_all(data);
 		if ((event->state & GDK_CONTROL_MASK) || (found != NULL)) {
 			g_slist_free(found);
@@ -583,20 +583,20 @@ static gint motion_event_cb (GtkWidget *widget, GdkEventMotion *event, gpointer
 				dx -= DRAG_MIN_MOVE;
 			else if (dx < -DRAG_MIN_MOVE)
 				dx += DRAG_MIN_MOVE;
-			else 
+			else
 				dx = 0;
 			if (dy > DRAG_MIN_MOVE)
 				dy -= DRAG_MIN_MOVE;
 			else if (dy < -DRAG_MIN_MOVE)
 				dy += DRAG_MIN_MOVE;
-			else 
+			else
 				dy = 0;
 			drag_adjust_cuts(window, dx, dy);
-			ox = x; 
+			ox = x;
 			oy = y;
 		}
 	} else {
-		ox = x; 
+		ox = x;
 		oy = y;
 	}
 	return TRUE;
@@ -606,13 +606,13 @@ GtkItemFactory *get_star_popup_menu(gpointer data)
 {
 	GtkItemFactory *item_factory;
 	GtkAccelGroup *accel_group;
-	gint nmenu_items = sizeof (star_popup_menu_items) / 
+	gint nmenu_items = sizeof (star_popup_menu_items) /
 		sizeof (star_popup_menu_items[0]);
 	accel_group = gtk_accel_group_new ();
 
-	item_factory = gtk_item_factory_new (GTK_TYPE_MENU, 
+	item_factory = gtk_item_factory_new (GTK_TYPE_MENU,
 					     "<star_popup>", accel_group);
-	gtk_item_factory_create_items (item_factory, nmenu_items, 
+	gtk_item_factory_create_items (item_factory, nmenu_items,
 				       star_popup_menu_items, data);
 
 	return item_factory;
@@ -625,21 +625,21 @@ GtkWidget *get_image_popup_menu(GtkWidget *window)
 	GtkWidget *ret;
 	GtkItemFactory *item_factory;
 	GtkAccelGroup *accel_group;
-	gint nmenu_items = sizeof (image_popup_menu_items) / 
+	gint nmenu_items = sizeof (image_popup_menu_items) /
 		sizeof (image_popup_menu_items[0]);
 	accel_group = gtk_accel_group_new ();
 
-	item_factory = gtk_item_factory_new (GTK_TYPE_MENU, 
+	item_factory = gtk_item_factory_new (GTK_TYPE_MENU,
 					     "<image_popup>", accel_group);
 	gtk_object_set_data_full(GTK_OBJECT(window), "image_popup_if", item_factory,
 				 (GtkDestroyNotify) gtk_object_unref);
-	gtk_item_factory_create_items (item_factory, nmenu_items, 
+	gtk_item_factory_create_items (item_factory, nmenu_items,
 				       image_popup_menu_items, window);
 
   /* Attach the new accelerator group to the window. */
 	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
 
-    /* Finally, return the actual menu bar created by the item factory. */ 
+    /* Finally, return the actual menu bar created by the item factory. */
 	ret = gtk_item_factory_get_widget (item_factory, "<image_popup>");
 	return ret;
 }
@@ -649,28 +649,28 @@ static GtkWidget *get_main_menu_bar(GtkWidget *window)
 	GtkWidget *ret;
 	GtkItemFactory *item_factory;
 	GtkAccelGroup *accel_group;
-	gint nmenu_items = sizeof (image_popup_menu_items) / 
+	gint nmenu_items = sizeof (image_popup_menu_items) /
 		sizeof (image_popup_menu_items[0]);
 	accel_group = gtk_accel_group_new ();
 
-	item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, 
+	item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR,
 					     "<main_menu>", accel_group);
 	gtk_object_set_data_full(GTK_OBJECT(window), "main_menu_if", item_factory,
 				 (GtkDestroyNotify) gtk_object_unref);
-	gtk_item_factory_create_items (item_factory, nmenu_items, 
+	gtk_item_factory_create_items (item_factory, nmenu_items,
 				       image_popup_menu_items, window);
 
   /* Attach the new accelerator group to the window. */
 	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
 
-    /* Finally, return the actual menu bar created by the item factory. */ 
+    /* Finally, return the actual menu bar created by the item factory. */
 	ret = gtk_item_factory_get_widget (item_factory, "<main_menu>");
 	return ret;
 }
 
 
 GtkWidget * create_image_window()
-{	
+{
 	GtkWidget *window;
 	GtkWidget *scrolled_window;
 	GtkWidget *image;
@@ -687,6 +687,8 @@ GtkWidget * create_image_window()
 	image = gtk_drawing_area_new();
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	g_object_ref_sink(window);
+
 	gtk_signal_connect (GTK_OBJECT (window), "destroy",
 			  GTK_SIGNAL_FUNC (destroy_cb), NULL);
 
@@ -717,9 +719,9 @@ GtkWidget * create_image_window()
 	gtk_object_set_data_full (GTK_OBJECT (window), "statuslabel2", statuslabel2,
 				  (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (statuslabel1);
-	
+
 	menubar = get_main_menu_bar(window);
-	gtk_menu_bar_set_shadow_type(GTK_MENU_BAR(menubar), GTK_SHADOW_NONE);
+	//gtk_menu_bar_set_shadow_type(GTK_MENU_BAR(menubar), GTK_SHADOW_NONE);
 
 	gtk_widget_show(menubar);
 	gtk_box_pack_start(GTK_BOX(hbox), menubar, TRUE, TRUE, 0);
@@ -732,16 +734,16 @@ GtkWidget * create_image_window()
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), alignment);
 	gtk_container_add(GTK_CONTAINER(alignment), image);
 
-	gtk_signal_connect(GTK_OBJECT(scrolled_window), "button_press_event", 
+	gtk_signal_connect(GTK_OBJECT(scrolled_window), "button_press_event",
 			 GTK_SIGNAL_FUNC(sources_clicked_cb), window);
-	gtk_signal_connect(GTK_OBJECT(scrolled_window), "button_press_event", 
+	gtk_signal_connect(GTK_OBJECT(scrolled_window), "button_press_event",
 			 GTK_SIGNAL_FUNC(image_clicked_cb), window);
-	gtk_signal_connect(GTK_OBJECT(image), "motion_notify_event", 
+	gtk_signal_connect(GTK_OBJECT(image), "motion_notify_event",
 			 GTK_SIGNAL_FUNC(motion_event_cb), window);
-	gtk_signal_connect(GTK_OBJECT(image), "expose_event", 
+	gtk_signal_connect(GTK_OBJECT(image), "expose_event",
 			 GTK_SIGNAL_FUNC(image_expose_cb), window);
 
-	gtk_widget_set_events(image,  GDK_BUTTON_PRESS_MASK 
+	gtk_widget_set_events(image,  GDK_BUTTON_PRESS_MASK
 			      | GDK_POINTER_MOTION_MASK
 			      | GDK_POINTER_MOTION_HINT_MASK);
 
@@ -749,7 +751,7 @@ GtkWidget * create_image_window()
 	gtk_object_set_data(GTK_OBJECT(window), "image_alignment", alignment);
 	gtk_object_set_data(GTK_OBJECT(window), "image", image);
 
-  	gtk_window_set_default_size(GTK_WINDOW(window), 700, 500); 
+  	gtk_window_set_default_size(GTK_WINDOW(window), 700, 500);
 
 	gtk_widget_show(image);
 	gtk_widget_show (scrolled_window);
@@ -782,6 +784,8 @@ GtkWidget* create_about_cx (void)
 	GtkWidget *label7;
 	GtkWidget *label12;
 	GtkWidget *label13;
+	GtkWidget *label14;
+	GtkWidget *label15;
 	GtkWidget *hseparator1;
 	GtkWidget *label8;
 	GtkWidget *label9;
@@ -838,7 +842,7 @@ GtkWidget* create_about_cx (void)
 	gtk_widget_show (label4);
 	gtk_box_pack_start (GTK_BOX (vbox2), label4, FALSE, FALSE, 0);
 
-	label5 = gtk_label_new (("(c) 2002-2005 Radu Corlan"));
+	label5 = gtk_label_new (("(c) 2002-2009 Radu Corlan"));
 	gtk_widget_ref (label5);
 	gtk_object_set_data_full (GTK_OBJECT (about_cx), "label5", label5,
 				  (GtkDestroyNotify) gtk_widget_unref);
@@ -873,6 +877,20 @@ GtkWidget* create_about_cx (void)
 	gtk_widget_show (label13);
 	gtk_box_pack_start (GTK_BOX (vbox2), label13, FALSE, FALSE, 0);
 
+	label14 = gtk_label_new (("gtk2 port and DSLR raw file support (c) 2009 Matei Conovici"));
+	gtk_widget_ref (label14);
+	gtk_object_set_data_full (GTK_OBJECT (about_cx), "label14", label14,
+				  (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (label14);
+	gtk_box_pack_start (GTK_BOX (vbox2), label14, FALSE, FALSE, 0);
+
+	label15 = gtk_label_new (("Full-color enhancements (c) 2009 Geoffrey Hausheer"));
+	gtk_widget_ref (label15);
+	gtk_object_set_data_full (GTK_OBJECT (about_cx), "label15", label15,
+				  (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (label15);
+	gtk_box_pack_start (GTK_BOX (vbox2), label15, FALSE, FALSE, 0);
+
 	hseparator1 = gtk_hseparator_new ();
 	gtk_widget_ref (hseparator1);
 	gtk_object_set_data_full (GTK_OBJECT (about_cx), "hseparator1", hseparator1,
@@ -880,7 +898,7 @@ GtkWidget* create_about_cx (void)
 	gtk_widget_show (hseparator1);
 	gtk_box_pack_start (GTK_BOX (vbox2), hseparator1, FALSE, FALSE, 0);
 
-	label8 = gtk_label_new (("gcx comes with ABSOLUTELY NO WARANTY"));
+	label8 = gtk_label_new (("gcx comes with ABSOLUTELY NO WARRANTY"));
 	gtk_widget_ref (label8);
 	gtk_object_set_data_full (GTK_OBJECT (about_cx), "label8", label8,
 				  (GtkDestroyNotify) gtk_widget_unref);
