@@ -1,23 +1,23 @@
 /*******************************************************************************
   Copyright(c) 2000 - 2003 Radu Corlan. All rights reserved.
-  
-  This program is free software; you can redistribute it and/or modify it 
-  under the terms of the GNU General Public License as published by the Free 
-  Software Foundation; either version 2 of the License, or (at your option) 
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the Free
+  Software Foundation; either version 2 of the License, or (at your option)
   any later version.
-  
-  This program is distributed in the hope that it will be useful, but WITHOUT 
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
-  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
+
+  This program is distributed in the hope that it will be useful, but WITHOUT
+  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.
-  
+
   You should have received a copy of the GNU General Public License along with
-  this program; if not, write to the Free Software Foundation, Inc., 59 
+  this program; if not, write to the Free Software Foundation, Inc., 59
   Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
   The full GNU General Public License is included in this distribution in the
   file called LICENSE.
-  
+
   Contact Information: radu@corlan.net
 *******************************************************************************/
 
@@ -84,7 +84,7 @@ void drag_adjust_cuts(GtkWidget *window, int dx, int dy)
 		dx = -30;
 
 	if (dy > 0 || span > MIN_SPAN)
-		span = span * (1.0 + dy * CONTRAST_STEP_DRAG); 
+		span = span * (1.0 + dy * CONTRAST_STEP_DRAG);
 	channel->lcut = base - span * channel->avg_at - dx * BRIGHT_STEP_DRAG * span;
 	channel->hcut = base + span * (1 - channel->avg_at)- dx * BRIGHT_STEP_DRAG * span;
 
@@ -123,12 +123,12 @@ static void channel_cuts_action(struct image_channel *channel, int action)
 		channel->hcut = fr->stats.max;
 		break;
 	case CUTS_FLATTER:
-		span = span * CONTRAST_STEP; 
+		span = span * CONTRAST_STEP;
 		channel->lcut = base - span * channel->avg_at;
 		channel->hcut = base + span * (1 - channel->avg_at);
 		break;
 	case CUTS_SHARPER:
-		span = span / CONTRAST_STEP; 
+		span = span / CONTRAST_STEP;
 		channel->lcut = base - span * channel->avg_at;
 		channel->hcut = base + span * (1 - channel->avg_at);
 		break;
@@ -169,9 +169,9 @@ static void channel_cuts_action(struct image_channel *channel, int action)
 			val = 0;
 		if (val >= SIGMAS_VALS)
 			val = SIGMAS_VALS - 1;
-		channel->lcut = channel->davg - channel->avg_at 
+		channel->lcut = channel->davg - channel->avg_at
 			* sigmas[val] * channel->dsigma;
-		channel->hcut = channel->davg + (1.0 - channel->avg_at) 
+		channel->hcut = channel->davg + (1.0 - channel->avg_at)
 			* sigmas[val] * channel->dsigma;
 		break;
 	default:
@@ -188,11 +188,11 @@ void set_default_channel_cuts(struct image_channel* channel)
 }
 
 
-/* pan the image in the scrolled window 
- * so that the center of the viewable area is at xc and yc 
+/* pan the image in the scrolled window
+ * so that the center of the viewable area is at xc and yc
  * of the image's width/height
- */ 
-int set_scrolls(GtkWidget *window, double xc, double yc) 
+ */
+int set_scrolls(GtkWidget *window, double xc, double yc)
 {
 	GtkScrolledWindow *scw;
 	GtkAdjustment *hadj, *vadj;
@@ -223,11 +223,11 @@ int set_scrolls(GtkWidget *window, double xc, double yc)
 	return 0;
 }
 
-/* return the position of the scrollbars as 
+/* return the position of the scrollbars as
  * the fraction of the image's dimention the center of the
  * visible area is at
  */
-int get_scrolls(GtkWidget *window, double *xc, double *yc) 
+int get_scrolls(GtkWidget *window, double *xc, double *yc)
 {
 	GtkScrolledWindow *scw;
 	GtkAdjustment *hadj, *vadj;
@@ -271,26 +271,26 @@ void set_darea_size(GtkWidget *window, struct map_geometry *geom, double xc, dou
 		return;
 	}
 
-	if (GTK_WIDGET(scw)->window == NULL) 
+	if (GTK_WIDGET(scw)->window == NULL)
 		return;
 	gdk_window_get_size(GTK_WIDGET(scw)->window, &w, &h);
 	d3_printf("scw size id %d %d\n", w, h);
-	
+
 //	gdk_window_freeze_updates(GTK_WIDGET(darea)->window);
 
 #ifndef GTK2
-	gtk_drawing_area_size(GTK_DRAWING_AREA(darea), 
+	gtk_drawing_area_size(GTK_DRAWING_AREA(darea),
 			      geom->width * zi / zo, geom->height * zi / zo);
 #else
-	gtk_widget_set_size_request(GTK_WIDGET(darea), geom->width * zi / zo, 
+	gtk_widget_set_size_request(GTK_WIDGET(darea), geom->width * zi / zo,
 				    geom->height * zi / zo);
 #endif
 
-/* we need this to make sure the drawing area contracts properly 
- * when we go from scrollbars to no scrollbars, we have to be sure 
+/* we need this to make sure the drawing area contracts properly
+ * when we go from scrollbars to no scrollbars, we have to be sure
  * everything happens in sequnece */
 
-	if (w >= geom->width * zi / zo || 
+	if (w >= geom->width * zi / zo ||
 	    h >= geom->height * zi / zo) {
 		gtk_widget_queue_resize(window);
 		while (gtk_events_pending ())
@@ -302,7 +302,7 @@ void set_darea_size(GtkWidget *window, struct map_geometry *geom, double xc, dou
 
 	if (hadj->page_size < geom->width * zi / zo) {
 		hadj->upper = geom->width * zi / zo;
-		if (hadj->upper < hadj->page_size) 
+		if (hadj->upper < hadj->page_size)
 			hadj->upper = hadj->page_size;
 		hadj->value = (hadj->upper - hadj->lower) * xc + hadj->lower
 			- hadj->page_size / 2;
@@ -314,7 +314,7 @@ void set_darea_size(GtkWidget *window, struct map_geometry *geom, double xc, dou
 
 	if (vadj->page_size < geom->height * zi / zo) {
 		vadj->upper = geom->height * zi / zo;
-		if (vadj->upper < vadj->page_size) 
+		if (vadj->upper < vadj->page_size)
 			vadj->upper = vadj->page_size;
 		vadj->value = (vadj->upper - vadj->lower) * yc + vadj->lower
 			- vadj->page_size / 2;
@@ -348,7 +348,7 @@ void step_zoom(struct map_geometry *geom, int step)
 	if (zoom == 1.0) {
 		if (step > 0)
 			geom->zoom = 2.0;
-		else 
+		else
 			geom->zoom = 0.5;
 		return;
 	}
@@ -359,12 +359,12 @@ void step_zoom(struct map_geometry *geom, int step)
 	if (step > 0) {
 		if (zoom < ZOOM_STEP_TRSHD)
 			zoom += 1.0;
-		else 
+		else
 			zoom += ZOOM_STEP_LARGE;
 	} else {
 		if (zoom > ZOOM_STEP_TRSHD)
 			zoom -= ZOOM_STEP_LARGE;
-		else 
+		else
 			zoom -= 1.0;
 	}
 	if (zoom > MAX_ZOOM)
@@ -395,18 +395,18 @@ void pan_cursor(GtkWidget *window)
 	if (image->window) {
 		gdk_window_get_pointer(image->window, &x, &y, &mask);
 		gdk_window_get_size(image->window, &w, &h);
-		
+
 		set_scrolls(window, 1.0 * x / w, 1.0 * y / h);
 	}
 }
 
 
-/********************** 
- * callbacks from gui 
+/**********************
+ * callbacks from gui
  **********************/
 
 /*
- * changing cuts (brightness/contrast) 
+ * changing cuts (brightness/contrast)
  */
 void cuts_option_cb(gpointer data, guint action, GtkWidget *menu_item)
 {
@@ -427,7 +427,7 @@ void cuts_option_cb(gpointer data, guint action, GtkWidget *menu_item)
 	gtk_widget_queue_draw(window);
 }
 
-/* 
+/*
  * zoom/pan
  */
 void view_option_cb(gpointer window, guint action, GtkWidget *menu_item)
@@ -448,7 +448,7 @@ void view_option_cb(gpointer window, guint action, GtkWidget *menu_item)
 
 	gdk_window_get_pointer(image->window, &x, &y, &mask);
 	gdk_window_get_size(image->window, &w, &h);
-	d2_printf("view action %d at x:%d y:%d  (w:%d h:%d) mask: %d\n", 
+	d2_printf("view action %d at x:%d y:%d  (w:%d h:%d) mask: %d\n",
 	       action, x, y, w, h, mask);
 
 	switch(action) {
@@ -509,7 +509,7 @@ void stats_cb(gpointer data, guint action, GtkWidget *menu_item)
 	if (!(i_channel->fr)->stats.statsok)
 		frame_stats(i_channel->fr);
 
-	info_printf_sb2(data, "Frame: %dx%d   avg:%.0f sigma:%.1f min:%.0f max:%.0f", 
+	info_printf_sb2(data, "Frame: %dx%d   avg:%.0f sigma:%.1f min:%.0f max:%.0f",
 		i_channel->fr->w, i_channel->fr->h,
 		i_channel->fr->stats.cavg, i_channel->fr->stats.csigma,
 		i_channel->fr->stats.min, i_channel->fr->stats.max );
@@ -547,22 +547,33 @@ void show_region_stats(GtkWidget *window, double x, double y)
 	yi = y;
 
 	if (i_channel->fr->pix_format == PIX_FLOAT) {
-	val = *((float *)(i_channel->fr->dat) + xi + yi * i_channel->fr->w);
-	ring_stats(i_channel->fr, x, y, 0, 10, 0xf, &rs, -HUGE, HUGE);
-	sprintf(buf, "[%d,%d]=%.1f  Region: Avg:%.0f  Sigma:%.1f  Min:%.0f  Max:%.0f", 
-		xi, yi, val, rs.avg, rs.sigma, rs.min, rs.max );
+		val = get_pixel_luminence(i_channel->fr, xi, yi);
+		ring_stats(i_channel->fr, x, y, 0, 10, 0xf, &rs, -HUGE, HUGE);
+
+		if (i_channel->fr->magic & FRAME_VALID_RGB) {
+			sprintf(buf, "[%d,%d]=%.1f (%.1f, %.1f, %.1f)  Region: Avg:%.0f  Sigma:%.1f  Min:%.0f  Max:%.0f",
+				xi, yi, val,
+				*(((float *) i_channel->fr->rdat) + xi + yi * i_channel->fr->w),
+				*(((float *) i_channel->fr->gdat) + xi + yi * i_channel->fr->w),
+				*(((float *) i_channel->fr->bdat) + xi + yi * i_channel->fr->w),
+				rs.avg, rs.sigma, rs.min, rs.max );
+		} else {
+			sprintf(buf, "[%d,%d]=%.1f  Region: Avg:%.0f  Sigma:%.1f  Min:%.0f  Max:%.0f",
+				xi, yi, val, rs.avg, rs.sigma, rs.min, rs.max );
+		}
 	} else if (i_channel->fr->pix_format == PIX_BYTE) {
+		/* FIXME: This won't work with RGB */
 		val = 1.0 * *((unsigned char *)(i_channel->fr->dat) + xi + yi * i_channel->fr->w);
-		sprintf(buf, "Pixel [%d,%d]=%.1f", 
+		sprintf(buf, "Pixel [%d,%d]=%.1f",
 		xi, yi, val);
 	} else {
-		sprintf(buf, "Pixel [%d,%d] unknown format", xi, yi); 
+		sprintf(buf, "Pixel [%d,%d] unknown format", xi, yi);
 	}
 	info_printf_sb2(window, "%s\n", buf);
 }
 
 /*
- * show zoom/cuts in statusbar 
+ * show zoom/cuts in statusbar
  */
 void show_zoom_cuts(GtkWidget * window)
 {
@@ -585,7 +596,7 @@ void show_zoom_cuts(GtkWidget * window)
 	if (geom == NULL)
 		return;
 
-	sprintf(buf, " Z:%.2f  Lcut:%.0f  Hcut:%.0f", 
+	sprintf(buf, " Z:%.2f  Lcut:%.0f  Hcut:%.0f",
 		geom->zoom, i_channel->lcut, i_channel->hcut);
 
 	gtk_label_set_text(GTK_LABEL(statuslabel), buf);
@@ -620,13 +631,13 @@ void channel_set_lut_from_gamma(struct image_channel *channel)
 		x1 = channel->gamma * channel->toe;
 //		d3_printf("channel offset is %f\n", channel->offset);
 		if (x0 < 0.001) {
-			y = 65535 * (channel->offset + (1 - channel->offset) * 
+			y = 65535 * (channel->offset + (1 - channel->offset) *
 				     pow(x, 1.0 / channel->gamma));
 		} else {
 			g = (x + x1) * x0 / ((x + x0) * x1);
 			y = 65535 * (channel->offset + (1 - channel->offset) * pow(x, g));
 		}
-		if (!channel->invert) 
+		if (!channel->invert)
 			channel->lut[i] = y;
 		else
 			channel->lut[LUT_SIZE - i - 1] = y;
@@ -637,11 +648,11 @@ void channel_set_lut_from_gamma(struct image_channel *channel)
 /* draw the histogram/curve */
 #define CUTS_FACTOR 0.8 /* how much of the histogram is between the cuts */
 /* rebin a histogram region from low to high into dbins bins */
-void rebin_histogram(struct im_histogram *hist, double *rbh, int dbins, 
+void rebin_histogram(struct im_histogram *hist, double *rbh, int dbins,
 		double low, double high, double *max)
 {
 	int i, hp, dp;
-	int leader = 0; 
+	int leader = 0;
 	double hbinsize, dbinsize;
 	double v, ri, dri;
 
@@ -722,7 +733,7 @@ void scale_histogram(double *rbh, int dbins, double max, int logh)
 	}
 }
 
-void plot_histogram(GtkWidget *darea, GdkRectangle *area, 
+void plot_histogram(GtkWidget *darea, GdkRectangle *area,
 		    int dskip, double *rbh, int dbins)
 {
 	int firstx, lcx, hcx;
@@ -788,12 +799,12 @@ void plot_curve(GtkWidget *darea, GdkRectangle *area, struct image_channel *chan
 	span = hcx - lcx;
 	points = calloc((span+1), sizeof(GdkPoint));
 	points[0].x = 0;
-	points[0].y = darea->allocation.height 
+	points[0].y = darea->allocation.height
 			- darea->allocation.height * channel->lut[0] / 65536;
 	for (i = 0; i < span; i++) {
-		ci = LUT_SIZE * i / span; 
+		ci = LUT_SIZE * i / span;
 		points[i+1].x = lcx+i;
-		points[i+1].y = darea->allocation.height 
+		points[i+1].y = darea->allocation.height
 			- darea->allocation.height * channel->lut[ci] / 65536;
 	}
 	gdk_draw_lines(darea->window, greengc, points, span+1);
@@ -901,7 +912,7 @@ void imadj_cuts_updated (GtkWidget *spinbutton, gpointer dialog)
 	channel = gtk_object_get_data(GTK_OBJECT(dialog), "i_channel");
 	if (channel == NULL) /* no channel */
 		return;
-	window = gtk_object_get_data(GTK_OBJECT(dialog), "image_window"); 
+	window = gtk_object_get_data(GTK_OBJECT(dialog), "image_window");
 
 	channel->lcut = spin_get_value(dialog, "low_cut_spin");
 	channel->hcut = spin_get_value(dialog, "high_cut_spin");
@@ -929,7 +940,7 @@ void imadj_lut_updated (GtkWidget *spinbutton, gpointer dialog)
 	channel = gtk_object_get_data(GTK_OBJECT(dialog), "i_channel");
 	if (channel == NULL) /* no channel */
 		return;
-	window = gtk_object_get_data(GTK_OBJECT(dialog), "image_window"); 
+	window = gtk_object_get_data(GTK_OBJECT(dialog), "image_window");
 
 	channel->gamma = spin_get_value(dialog, "gamma_spin");
 	channel->toe = spin_get_value(dialog, "toe_spin");
@@ -1107,7 +1118,7 @@ void histogram_cb(gpointer data, guint action, GtkWidget *menu_item)
 		gtk_widget_ref(dialog);
 		gtk_object_set_data_full(GTK_OBJECT(window), "imadj_dialog", dialog,
 					 (GtkDestroyNotify)gtk_widget_destroy);
-		gtk_object_set_data(GTK_OBJECT(dialog), "image_window", window); 
+		gtk_object_set_data(GTK_OBJECT(dialog), "image_window", window);
 		close = gtk_object_get_data(GTK_OBJECT(dialog), "hist_close");
 		gtk_signal_connect (GTK_OBJECT (dialog), "destroy",
 				    GTK_SIGNAL_FUNC (close_imadj_dialog), window);
@@ -1123,7 +1134,7 @@ void histogram_cb(gpointer data, guint action, GtkWidget *menu_item)
 		gdk_window_raise(dialog->window);
 	}
 	ref_image_channel(i_channel);
-	gtk_object_set_data_full(GTK_OBJECT(dialog), "i_channel", i_channel, 
+	gtk_object_set_data_full(GTK_OBJECT(dialog), "i_channel", i_channel,
 				 (GtkDestroyNotify)release_image_channel);
 	imadj_dialog_update(dialog);
 	imadj_dialog_edit(dialog);
