@@ -1,29 +1,30 @@
+
 /*******************************************************************************
   Copyright(c) 2000 - 2003 Radu Corlan. All rights reserved.
-  
-  This program is free software; you can redistribute it and/or modify it 
-  under the terms of the GNU General Public License as published by the Free 
-  Software Foundation; either version 2 of the License, or (at your option) 
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the Free
+  Software Foundation; either version 2 of the License, or (at your option)
   any later version.
-  
-  This program is distributed in the hope that it will be useful, but WITHOUT 
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
-  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
+
+  This program is distributed in the hope that it will be useful, but WITHOUT
+  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.
-  
+
   You should have received a copy of the GNU General Public License along with
-  this program; if not, write to the Free Software Foundation, Inc., 59 
+  this program; if not, write to the Free Software Foundation, Inc., 59
   Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
   The full GNU General Public License is included in this distribution in the
   file called LICENSE.
-  
+
   Contact Information: radu@corlan.net
 *******************************************************************************/
 
 /* Multiband reduction gui and menus */
 
-/* we hold references to o_frames, sobs and o_stars without them being ref_counted. 
+/* we hold references to o_frames, sobs and o_stars without them being ref_counted.
    so while the clists in the dialog are alive, the mbds must not be destroyed */
 
 #include <stdio.h>
@@ -38,7 +39,6 @@
 #include "sourcesdraw.h"
 #include "params.h"
 #include "wcs.h"
-#include "camera.h"
 #include "cameragui.h"
 #include "filegui.h"
 #include "interface.h"
@@ -51,7 +51,6 @@
 
 
 static void bands_list_update_vals(GtkWidget *dialog, struct mband_dataset *mbds);
-static void sob_list_set_row_vals(GtkWidget *sob_list, int row, struct star_obs *sob);
 static void mb_rebuild_sob_list(gpointer dialog, GList *sol);
 static void sob_list_update_vals(GtkWidget *sob_list);
 static void mb_rebuild_ofr_list(gpointer dialog);
@@ -110,21 +109,27 @@ static int fit_progress(char *msg, void *data)
 static void rep_file_cb(char *fn, gpointer data, unsigned action)
 {
 	GtkWidget *ofr_list;
-	GList *gl;
+	GtkTreeModel *ofr_store;
 	GList *ofrs = NULL, *sl;
 	FILE *repfp = NULL;
 	struct mband_dataset *mbds;
 	struct o_frame *ofr;
 	char qu[1024];
 	int ret;
+	GList *gl, *glh;
+	GtkTreeSelection *sel;
+	GtkTreeIter iter;
 
 	d3_printf("Report action %x fn:%s\n", action, fn);
 
 	mbds = gtk_object_get_data(GTK_OBJECT(data), "mbds");
 	g_return_if_fail(mbds != NULL);
+
 	ofr_list = gtk_object_get_data(GTK_OBJECT(data), "ofr_list");
 	g_return_if_fail(ofr_list != NULL);
 
+	ofr_store = gtk_tree_view_get_model(GTK_TREE_VIEW(ofr_list));
+
 	if ((repfp = fopen(fn, "r")) != NULL) { /* file exists */
 		snprintf(qu, 1023, "File %s exists\nAppend?", fn);
 		if (!modal_yes_no(qu, "gcx: file exists")) {
@@ -141,14 +146,18 @@ static void rep_file_cb(char *fn, gpointer data, unsigned action)
 		return;
 	}
 
-	gl = GTK_CLIST(ofr_list)->selection;
-	if (gl != NULL) {
-		for (gl = GTK_CLIST(ofr_list)->selection; gl != NULL; gl = g_list_next(gl)) {
-//			d3_printf("row %d is selected\n", (int) gl->data);
-			ofr = gtk_clist_get_row_data(GTK_CLIST(ofr_list), (int) gl->data); 
+	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(ofr_list));
+	glh = gtk_tree_selection_get_selected_rows(sel, NULL);
+	if (glh != NULL) {
+		for (gl = g_list_first(glh); gl != NULL; gl = g_list_next(gl)) {
+			if (!gtk_tree_model_get_iter(ofr_store, &iter, gl->data))
+				continue;
+
+			gtk_tree_model_get(ofr_store, &iter, 0, &ofr, -1);
 			if (ofr->skip)
 				continue;
-			if (((action & FMT_MASK) != REP_DATASET) && 
+
+			if (((action & FMT_MASK) != REP_DATASET) &&
 			    (ofr == NULL || ZPSTATE(ofr) <= ZP_FIT_ERR)) {
 //				d3_printf("skipping frame with 0 fitted stars\n");
 				continue;
@@ -156,12 +165,15 @@ static void rep_file_cb(char *fn, gpointer data, unsigned action)
 			ofrs = g_list_prepend(ofrs, ofr);
 		}
 		ofrs = g_list_reverse(ofrs);
+
+		g_list_foreach(glh, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free(glh);
 	} else {
 		for (sl = mbds->ofrs; sl != NULL; sl = g_list_next(sl)) {
-			ofr = O_FRAME(sl->data); 
+			ofr = O_FRAME(sl->data);
 			if (ofr->skip)
 				continue;
-			if (((action & FMT_MASK) != REP_DATASET) && 
+			if (((action & FMT_MASK) != REP_DATASET) &&
 			    (ofr == NULL || ZPSTATE(ofr) <= ZP_FIT_ERR)) {
 				continue;
 			}
@@ -169,19 +181,21 @@ static void rep_file_cb(char *fn, gpointer data, unsigned action)
 		}
 		ofrs = g_list_reverse(ofrs);
 	}
+
 	if (ofrs == NULL) {
 		error_beep();
 		mbds_printf(data, "Nothing to report. Only fitted frames will be reported.\n");
 		return;
 	}
-	
+
 	ret = mbds_report_from_ofrs(mbds, repfp, ofrs, action);
 	if (ret < 0) {
 		mbds_printf(data, "%s", last_err());
 	} else {
-		mbds_printf(data, "%d frame(s), %d star(s) reported to %s", 
+		mbds_printf(data, "%d frame(s), %d star(s) reported to %s",
 			    g_list_length(ofrs), ret, fn);
 	}
+
 	g_list_free(ofrs);
 	fclose(repfp);
 }
@@ -195,59 +209,82 @@ static void rep_cb(gpointer data, guint action, GtkWidget *menu_item)
 static void select_cb(gpointer data, guint action, GtkWidget *menu_item)
 {
 	GtkWidget *list;
-	GList *gl;
+	GList *gl, *glh;
 	struct mband_dataset *mbds;
 	int n;
+	struct o_frame *ofr;
+	GtkTreeIter iter;
+	GtkTreeModel *ofr_store;
+	GtkTreeSelection *sel;
 
 	switch(action) {
 	case SEL_ALL:
 		list = gtk_object_get_data(GTK_OBJECT(data), "ofr_list");
-		if (list) 
-			gtk_clist_select_all(GTK_CLIST(list));
+		if (list)
+			gtk_tree_selection_select_all(gtk_tree_view_get_selection(GTK_TREE_VIEW(list)));
+
 		list = gtk_object_get_data(GTK_OBJECT(data), "sob_list");
-		if (list) 
-			gtk_clist_select_all(GTK_CLIST(list));
+		if (list)
+			gtk_tree_selection_select_all(gtk_tree_view_get_selection(GTK_TREE_VIEW(list)));
 		break;
+
 	case UNSEL_ALL:
 		list = gtk_object_get_data(GTK_OBJECT(data), "ofr_list");
-		if (list) 
-			gtk_clist_unselect_all(GTK_CLIST(list));
+		if (list)
+			gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(GTK_TREE_VIEW(list)));
+
 		list = gtk_object_get_data(GTK_OBJECT(data), "sob_list");
-		if (list) 
-			gtk_clist_unselect_all(GTK_CLIST(list));
+		if (list)
+			gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(GTK_TREE_VIEW(list)));
 		break;
+
 	case HIDE_SEL:
 		n = 0;
 		list = gtk_object_get_data(GTK_OBJECT(data), "ofr_list");
-		if (list == NULL) 
+		if (list == NULL)
 			return;
-		gtk_clist_freeze(GTK_CLIST(list));
-		while((gl = GTK_CLIST(list)->selection) != NULL) {
-			struct o_frame *ofr;
-			ofr = gtk_clist_get_row_data(GTK_CLIST(list), (int) gl->data); 
+
+		ofr_store = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
+
+		sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
+		glh = gtk_tree_selection_get_selected_rows(sel, NULL);
+
+		for (gl = g_list_first(glh); gl != NULL; gl = g_list_next(gl)) {
+			if (!gtk_tree_model_get_iter(ofr_store, &iter, gl->data))
+				continue;
+
+			gtk_tree_model_get(ofr_store, &iter, 0, &ofr, -1);
+
 			ofr->skip = 1;
-			gtk_clist_remove(GTK_CLIST(list), (int) gl->data);
 			n++;
 		}
-		gtk_clist_thaw(GTK_CLIST(list));
-		if (n > 0)
+
+		g_list_foreach(glh, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free(glh);
+
+		if (n > 0) {
+			mb_rebuild_ofr_list(data);
 			mbds_printf(data, "%d frames hidden: they will not be processed or saved\n", n);
+		}
 		break;
+
 	case UNHIDE_ALL:
 		n = 0;
 		mbds = gtk_object_get_data(GTK_OBJECT(data), "mbds");
 		g_return_if_fail(mbds != NULL);
+
 		for (gl = mbds->ofrs; gl != NULL; gl = gl->next) {
-			struct o_frame *ofr;
 			ofr = gl->data;
 			if (ofr->skip) {
 				ofr->skip = 0;
 				n++;
 			}
 		}
-		mb_rebuild_ofr_list(data);
-		if (n > 0)
+
+		if (n > 0) {
+			mb_rebuild_ofr_list(data);
 			mbds_printf(data, "%d frames restored\n", n);
+		}
 		break;
 	}
 }
@@ -260,8 +297,8 @@ static void mb_close_cb(gpointer data, guint action, GtkWidget *menu_item)
 	mbds = gtk_object_get_data(GTK_OBJECT(data), "mbds");
 
 	if (mbds && !modal_yes_no("Closing the dataset will "
-		 "discard fit information\n"
-		 "Are you sure?", "Close dataset?"))
+				  "discard fit information\n"
+				  "Are you sure?", "Close dataset?"))
 		return;
 
 	list = gtk_object_get_data(GTK_OBJECT(data), "ofr_list");
@@ -287,7 +324,7 @@ static void mb_close_cb(gpointer data, guint action, GtkWidget *menu_item)
 static void plot_cb(gpointer data, guint action, GtkWidget *menu_item)
 {
 	GtkWidget *ofr_list;
-	GList *gl;
+	GList *gl, *glh;
 	GList *ofrs = NULL, *sl;
 	struct mband_dataset *mbds;
 	struct o_frame *ofr;
@@ -295,28 +332,49 @@ static void plot_cb(gpointer data, guint action, GtkWidget *menu_item)
 	int band = -1;
 	FILE *plfp = NULL;
 	int pop;
-	
+	GtkTreeModel *ofr_store;
+	GtkTreeSelection *sel;
+	GtkTreeIter iter;
+
 
 	mbds = gtk_object_get_data(GTK_OBJECT(data), "mbds");
 	g_return_if_fail(mbds != NULL);
 
 	if (action == PLOT_STAR) {
+		/* FIXME: variable reuse is not pretty */
 		ofr_list = gtk_object_get_data(GTK_OBJECT(data), "sob_list");
 		if (ofr_list == NULL) {
 			error_beep();
 			mbds_printf(data, "No star selected\n");
 			return;
 		}
-		gl = GTK_CLIST(ofr_list)->selection;
-		if (gl == NULL) {
+
+		ofr_store = gtk_tree_view_get_model(GTK_TREE_VIEW(ofr_list));
+
+		sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(ofr_list));
+		glh = gtk_tree_selection_get_selected_rows(sel, NULL);
+		if (glh == NULL) {
 			error_beep();
 			mbds_printf(data, "No star selected\n");
 			return;
 		}
-		for (gl = GTK_CLIST(ofr_list)->selection; gl != NULL; gl = g_list_next(gl)) {
-			sob = gtk_clist_get_row_data(GTK_CLIST(ofr_list), (int) gl->data); 
+
+		for (gl = g_list_first(glh); gl != NULL; gl = g_list_next(gl)) {
+			if (!gtk_tree_model_get_iter(ofr_store, &iter, gl->data))
+				continue;
+
+			gtk_tree_model_get(ofr_store, &iter, 0, &sob, -1);
+
+			ofr = sob->ofr;
+			if (ofr == NULL || ofr->skip || ZPSTATE(ofr) <= ZP_FIT_ERR)
+				continue;
+
 			ofrs = g_list_prepend(ofrs, sob);
 		}
+
+		g_list_foreach(glh, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free(glh);
+
 		pop = open_plot(&plfp, NULL);
 		if (pop < 0) {
 			mbds_printf(data, last_err());
@@ -329,38 +387,44 @@ static void plot_cb(gpointer data, guint action, GtkWidget *menu_item)
 
 
 	ofr_list = gtk_object_get_data(GTK_OBJECT(data), "ofr_list");
-	gl = GTK_CLIST(ofr_list)->selection;
-	if (gl != NULL) {
-		for (gl = GTK_CLIST(ofr_list)->selection; gl != NULL; gl = g_list_next(gl)) {
-//			d3_printf("row %d is selected\n", (int) gl->data);
-			ofr = gtk_clist_get_row_data(GTK_CLIST(ofr_list), (int) gl->data); 
-			if (ofr->skip)
+	g_return_if_fail(ofr_list != NULL);
+	ofr_store = gtk_tree_view_get_model(GTK_TREE_VIEW(ofr_list));
+
+	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(ofr_list));
+	glh = gtk_tree_selection_get_selected_rows(sel, NULL);
+	if (glh != NULL) {
+		for (gl = g_list_first(glh); gl != NULL; gl = g_list_next(gl)) {
+			if (!gtk_tree_model_get_iter(ofr_store, &iter, gl->data))
 				continue;
-			if (ofr == NULL || ZPSTATE(ofr) <= ZP_FIT_ERR) {
-//				d3_printf("skipping frame with 0 fitted stars\n");
+
+			gtk_tree_model_get(ofr_store, &iter, 0, &ofr, -1);
+			if (ofr == NULL || ofr->skip || ZPSTATE(ofr) <= ZP_FIT_ERR)
 				continue;
-			}
+
 			ofrs = g_list_prepend(ofrs, ofr);
 		}
 		ofrs = g_list_reverse(ofrs);
+
+		g_list_foreach(glh, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free(glh);
 	} else {
 		for (sl = mbds->ofrs; sl != NULL; sl = g_list_next(sl)) {
-			ofr = O_FRAME(sl->data); 
-			if (ofr->skip)
+			ofr = O_FRAME(sl->data);
+
+			if (ofr == NULL || ofr->skip || ZPSTATE(ofr) <= ZP_FIT_ERR)
 				continue;
-			if (ofr == NULL || ZPSTATE(ofr) <= ZP_FIT_ERR) {
-//				d3_printf("skipping frame with 0 fitted stars\n");
-				continue;
-			}
+
 			ofrs = g_list_prepend(ofrs, ofr);
 		}
 		ofrs = g_list_reverse(ofrs);
 	}
+
 	if (ofrs == NULL) {
 		error_beep();
 		mbds_printf(data, "Nothing to plot. Only fitted frames will be plotted\n");
 		return;
 	}
+
 	for (sl = ofrs; sl != NULL; sl = g_list_next(sl)) {
 		if (O_FRAME(sl->data)->band >= 0) {
 			band = O_FRAME(sl->data)->band;
@@ -378,6 +442,7 @@ static void plot_cb(gpointer data, guint action, GtkWidget *menu_item)
 		ofrs_plot_residual_vs_mag(plfp, ofrs, (action & PLOT_WEIGHTED) != 0);
 		close_plot(plfp, pop);
 		break;
+
 	case PLOT_RES_COL:
 		if (band < 0) {
 			mbds_printf(data, "None of the selected frames has a valid band\n");
@@ -388,10 +453,11 @@ static void plot_cb(gpointer data, guint action, GtkWidget *menu_item)
 			mbds_printf(data, last_err());
 			break;
 		}
-		ofrs_plot_residual_vs_col(mbds, plfp, band, 
+		ofrs_plot_residual_vs_col(mbds, plfp, band,
 					  ofrs, (action & PLOT_WEIGHTED) != 0);
 		close_plot(plfp, pop);
 		break;
+
 	case PLOT_ZP_TIME:
 		pop = open_plot(&plfp, NULL);
 		if (pop < 0) {
@@ -401,6 +467,7 @@ static void plot_cb(gpointer data, guint action, GtkWidget *menu_item)
 		ofrs_plot_zp_vs_time(plfp, ofrs);
 		close_plot(plfp, pop);
 		break;
+
 	case PLOT_ZP_AIRMASS:
 		pop = open_plot(&plfp, NULL);
 		if (pop < 0) {
@@ -411,22 +478,23 @@ static void plot_cb(gpointer data, guint action, GtkWidget *menu_item)
 		close_plot(plfp, pop);
 		break;
 	}
+
 	g_list_free(ofrs);
 }
 
-static void ofr_list_set_row_vals(GtkWidget *ofr_list, int row, struct o_frame *ofr)
+static void ofr_store_set_row_vals(GtkListStore *ofr_store, GtkTreeIter *iter, struct o_frame *ofr)
 {
 	char c[50 * 10];
 	int i;
 	char *obj;
-	char *states[] = {"Not Fitted", "Err", "All-sky", "Diff", "ZP Only", "OK",  
+	char *states[] = {"Not Fitted", "Err", "All-sky", "Diff", "ZP Only", "OK",
 			  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
 			  NULL, NULL, NULL};
 
 	obj = stf_find_string(ofr->stf, 1, SYM_OBSERVATION, SYM_OBJECT);
 	if (obj)
 		snprintf(c, 40, "%s", obj);
-	else 
+	else
 		c[0] = 0;
 	snprintf(c+50, 40, "%s", ofr->trans->bname);
 	if (ofr->zpstate >= ZP_ALL_SKY) {
@@ -438,7 +506,7 @@ static void ofr_list_set_row_vals(GtkWidget *ofr_list, int row, struct o_frame *
 		c[200] = 0;
 		c[350] = 0;
 	}
-	snprintf(c+100, 40, "%s%s", states[ofr->zpstate & ZP_STATE_M], 
+	snprintf(c+100, 40, "%s%s", states[ofr->zpstate & ZP_STATE_M],
 		 ofr->as_zp_valid ? "-AV" : "");
 	if ((ofr->zpstate & ZP_STATE_M) > ZP_NOT_FITTED) {
 		snprintf(c+250, 40, "%d", ofr->vstars);
@@ -449,31 +517,51 @@ static void ofr_list_set_row_vals(GtkWidget *ofr_list, int row, struct o_frame *
 	}
 	snprintf(c+400, 40, "%.2f", ofr->airmass);
 	snprintf(c+450, 40, "%.5f", ofr->mjd);
-	for (i = 0; i < 10; i++)
-		gtk_clist_set_text(GTK_CLIST(ofr_list), row, i, c+50*i);
+
+	gtk_list_store_set(ofr_store, iter, 0, ofr, -1);
+	for (i = 0; i < 10; i++) {
+		gtk_list_store_set(ofr_store, iter, i + 1, c + 50 * i, -1);
+	}
 }
 
 static void fit_cb(gpointer data, guint action, GtkWidget *menu_item)
 {
 	GtkWidget *ofr_list, *sob_list;
-	GList *gl;
+	GList *gl, *glh;
 	GList *ofrs = NULL, *sl;
 	struct mband_dataset *mbds;
 	struct o_frame *ofr;
-	int i;
+	GtkTreeSelection *sel;
+	GtkTreeModel *ofr_store;
+	GtkTreeIter iter;
 
 	ofr_list = gtk_object_get_data(GTK_OBJECT(data), "ofr_list");
-	sob_list = gtk_object_get_data(GTK_OBJECT(data), "sob_list");
-	mbds = gtk_object_get_data(GTK_OBJECT(data), "mbds");
 	g_return_if_fail(ofr_list != NULL);
+	ofr_store = gtk_tree_view_get_model(GTK_TREE_VIEW(ofr_list));
+
+	sob_list = gtk_object_get_data(GTK_OBJECT(data), "sob_list");
 //	g_return_if_fail(sob_list != NULL);
+
+	mbds = gtk_object_get_data(GTK_OBJECT(data), "mbds");
 	g_return_if_fail(mbds != NULL);
-	if (GTK_CLIST(ofr_list)->selection != NULL) {
-		for (gl = GTK_CLIST(ofr_list)->selection; gl != NULL; gl = g_list_next(gl)) {
-			ofr = gtk_clist_get_row_data(GTK_CLIST(ofr_list), (int) gl->data);
-			g_return_if_fail(ofr != NULL);
+
+	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(ofr_list));
+	glh = gtk_tree_selection_get_selected_rows(sel, NULL);
+	if (glh != NULL) {
+		for (gl = g_list_first(glh); gl != NULL; gl = g_list_next(gl)) {
+			if (!gtk_tree_model_get_iter(ofr_store, &iter, gl->data))
+				continue;
+
+			gtk_tree_model_get(ofr_store, &iter, 0, &ofr, -1);
+			if (ofr == NULL)
+				continue;
+
+			d4_printf("ofr = %08x\n", ofr);
 			ofrs = g_list_prepend(ofrs, ofr);
 		}
+
+		g_list_foreach(glh, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free(glh);
 	} else {
 		for (gl = mbds->ofrs; gl != NULL; gl = g_list_next(gl)) {
 			ofr = O_FRAME(gl->data);
@@ -483,6 +571,7 @@ static void fit_cb(gpointer data, guint action, GtkWidget *menu_item)
 		}
 //		ofrs = g_list_copy(mbds->ofrs);
 	}
+
 	switch (action) {
 	case FIT_ZPOINTS:
 		fit_progress("Fitting zero points with no color", data);
@@ -504,6 +593,7 @@ static void fit_cb(gpointer data, guint action, GtkWidget *menu_item)
 		bands_list_update_vals(data, mbds);
 		fit_progress("Done", data);
 		break;
+
 	case FIT_ZP_WTRANS:
 		fit_progress("Fitting zero points with current color coefficients", data);
 		for (sl = ofrs; sl != NULL; sl = g_list_next(sl)) {
@@ -521,6 +611,7 @@ static void fit_cb(gpointer data, guint action, GtkWidget *menu_item)
 		}
 		fit_progress("Done", data);
 		break;
+
 	case FIT_TRANS:
 		mbds_fit_all(ofrs, fit_progress, data);
 		fit_progress("Transforming stars", data);
@@ -528,6 +619,7 @@ static void fit_cb(gpointer data, guint action, GtkWidget *menu_item)
 		bands_list_update_vals(data, mbds);
 		fit_progress("Done", data);
 		break;
+
 	case FIT_ALL_SKY:
 		fit_progress("Fitting all-sky extinction coefficient", data);
 		if (fit_all_sky_zp(mbds, ofrs)) {
@@ -541,29 +633,43 @@ static void fit_cb(gpointer data, guint action, GtkWidget *menu_item)
 		mbds_transform_all(mbds, ofrs, 0);
 		bands_list_update_vals(data, mbds);
 		fit_progress("Done", data);
-
 		break;
+
 	default:
 		return;
 	}
-	if (GTK_CLIST(ofr_list)->selection == NULL) {
-		for (i = 0; i < GTK_CLIST(ofr_list)->rows; i++) {
-			ofr = gtk_clist_get_row_data(GTK_CLIST(ofr_list), i);
+
+	glh  = gtk_tree_selection_get_selected_rows(sel, NULL);
+	if (glh != NULL) {
+		for (gl = g_list_first(glh); gl != NULL; gl = g_list_next(gl)) {
+			if (!gtk_tree_model_get_iter(ofr_store, &iter, gl->data))
+				continue;
+
+			gtk_tree_model_get(ofr_store, &iter, 0, &ofr, -1);
 			if (ofr == NULL)
 				continue;
-			ofr_list_set_row_vals(ofr_list, i, ofr);
-			
-//			sob_list_update_by_ofr(sob_list, ofr);
+
+			ofr_store_set_row_vals(GTK_LIST_STORE(ofr_store), &iter, ofr);
 		}
+
+		g_list_foreach(glh, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free(glh);
 	} else {
-		for (gl = GTK_CLIST(ofr_list)->selection; gl != NULL; gl = g_list_next(gl)) {
-			ofr = gtk_clist_get_row_data(GTK_CLIST(ofr_list), (int) gl->data);
-			ofr_list_set_row_vals(ofr_list, (int) gl->data, ofr);
-//			sob_list_update_by_ofr(sob_list, ofr);
+		/* lovely API */
+		if (gtk_tree_model_get_iter_first(ofr_store, &iter)) {
+			do {
+				gtk_tree_model_get(ofr_store, &iter, 0, &ofr, -1);
+				if (ofr == NULL)
+					continue;
+
+				ofr_store_set_row_vals(GTK_LIST_STORE(ofr_store), &iter, ofr);
+			} while (gtk_tree_model_iter_next(ofr_store, &iter));
 		}
 	}
+
 	if (sob_list != NULL)
 		sob_list_update_vals(sob_list);
+
 	g_list_free(ofrs);
 }
 
@@ -573,90 +679,120 @@ static void mbds_ofr_to_list(GtkWidget *dialog, GtkWidget *list)
 	struct mband_dataset *mbds;
 	GList *sl;
 	struct o_frame *ofr;
-	int row, i;
-	char *line [] = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
+	GtkTreeIter iter;
+	GtkTreeModel *ofr_model;
+
 	mbds = gtk_object_get_data(GTK_OBJECT(dialog), "mbds");
-	if (mbds == NULL)
-		return;
-	gtk_clist_freeze(GTK_CLIST(list));
+	g_return_if_fail(mbds != NULL);
+
+	ofr_model = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
+
 	sl = mbds->ofrs;
 	while(sl != NULL) {
 		ofr = O_FRAME(sl->data);
 		sl = g_list_next(sl);
 		if (ofr->skip)
 			continue;
-		row = gtk_clist_append(GTK_CLIST(list), line);
-		ofr_list_set_row_vals(list, row, ofr);
-		gtk_clist_set_row_data(GTK_CLIST(list), row, ofr);
-		ofr->data = row;
+
+		gtk_list_store_append(GTK_LIST_STORE(ofr_model), &iter);
+		ofr_store_set_row_vals(GTK_LIST_STORE(ofr_model), &iter, ofr);
 	}
-	for (i = 0; i< 10; i++) {
-		gtk_clist_set_column_auto_resize(GTK_CLIST(list), i, 1);
-	}
-	gtk_clist_thaw(GTK_CLIST(list));
 }
 
-void ofr_bpress_cb(GtkCList *clist, GdkEventButton *event, gpointer data)
+
+void ofr_bpress_cb(GtkTreeView *ofr_list, GdkEventButton *event, gpointer data)
 {
-	int row = -1, col = -1, x, y;
+	int x, y;
+	struct o_frame *ofr;
+	GtkTreeModel *ofr_store;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+
+	g_return_if_fail(event->window == gtk_tree_view_get_bin_window(ofr_list));
+
 	x = floor(event->x);
 	y = floor(event->y);
-	if (gtk_clist_get_selection_info(clist, x, y, &row, &col))
-		mb_rebuild_sob_list(data, O_FRAME(gtk_clist_get_row_data(clist, row))->sol);
-}
 
-static void ofr_click_col_cb(GtkCList *clist, gint column, gpointer data)
-{
-	static int lastcol = -1;
-	gtk_clist_set_sort_column(GTK_CLIST(clist), column);
-	if (column == lastcol) {
-		lastcol = -1;
-		gtk_clist_set_sort_type(GTK_CLIST(clist), GTK_SORT_DESCENDING);
-	} else {
-		lastcol = column;
-		gtk_clist_set_sort_type(GTK_CLIST(clist), GTK_SORT_ASCENDING);
+	if (gtk_tree_view_get_path_at_pos(ofr_list, x, y, &path, NULL, NULL, NULL)) {
+
+		ofr_store = gtk_tree_view_get_model(ofr_list);
+		if (gtk_tree_model_get_iter(ofr_store, &iter, path)) {
+
+			gtk_tree_model_get(ofr_store, &iter, 0, &ofr, -1);
+			g_return_if_fail(ofr != NULL);
+
+			mb_rebuild_sob_list(data, ofr->sol);
+		}
 	}
-	gtk_clist_sort(GTK_CLIST(clist));
 }
 
 static void mb_rebuild_ofr_list(gpointer dialog)
 {
-	GtkWidget *list;
+	GtkListStore *ofr_store;
+	GtkWidget *ofr_list;
 	GtkScrolledWindow *scw;
-	char *titles[] = {"Object", "Band", "Status", "Zpoint", "Err", "Fitted", 
+	char *titles[] = {"Object", "Band", "Status", "Zpoint", "Err", "Fitted",
 			  "Outliers", "MEU", "Airmass", "MJD", NULL};
+	int i;
+	GtkTreeViewColumn *column;
 
-	list = gtk_object_get_data(GTK_OBJECT(dialog), "ofr_list");
-	if (list == NULL) {
+	ofr_list = gtk_object_get_data(GTK_OBJECT(dialog), "ofr_list");
+	if (ofr_list == NULL) {
 		scw = gtk_object_get_data(GTK_OBJECT(dialog), "ofr_scw");
 		g_return_if_fail(scw != NULL);
-		list = gtk_clist_new_with_titles(10, titles);
-		gtk_scrolled_window_add_with_viewport(scw, list);
-		gtk_widget_ref(list);
+
+		ofr_store = gtk_list_store_new(11, G_TYPE_POINTER,
+					       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+					       G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+					       G_TYPE_STRING, G_TYPE_STRING);
+
+		ofr_list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(ofr_store));
+		g_object_unref(ofr_store);
+
+		for (i = 0; i < 10; i++) {
+			column = gtk_tree_view_column_new_with_attributes(titles[i],
+									  gtk_cell_renderer_text_new(),
+									  "text", i + 1, NULL);
+
+			gtk_tree_view_column_set_sort_column_id(column, i + 1);
+			gtk_tree_view_append_column(GTK_TREE_VIEW(ofr_list), column);
+		}
+
+		gtk_scrolled_window_add_with_viewport(scw, ofr_list);
+		gtk_widget_ref(ofr_list);
 		gtk_object_set_data_full(GTK_OBJECT(dialog), "ofr_list",
-					 list, (GtkDestroyNotify) gtk_widget_destroy);
-		gtk_clist_set_selection_mode(GTK_CLIST(list), GTK_SELECTION_EXTENDED);
-		gtk_signal_connect(GTK_OBJECT(list), "button-press-event", ofr_bpress_cb, dialog);
-		gtk_signal_connect(GTK_OBJECT(list), "click-column", ofr_click_col_cb, dialog);
-		gtk_widget_show(list);
+					 ofr_list, (GtkDestroyNotify) gtk_widget_destroy);
+
+		gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(ofr_list)),
+					    GTK_SELECTION_MULTIPLE);
+
+		gtk_signal_connect(GTK_OBJECT(ofr_list), "button-press-event",
+				   GTK_SIGNAL_FUNC(ofr_bpress_cb), dialog);
+
+		gtk_widget_show(ofr_list);
 	} else {
-		gtk_clist_clear(GTK_CLIST(list));
+		ofr_store = GTK_LIST_STORE (gtk_tree_view_get_model(GTK_TREE_VIEW(ofr_list)));
+		gtk_list_store_clear(ofr_store);
 	}
-	mbds_ofr_to_list(dialog, list);
+
+	mbds_ofr_to_list(dialog, ofr_list);
 }
 
-static void bands_list_set_row_vals(GtkWidget *bands_list, int row, int band,
-				    struct mband_dataset *mbds)
+
+static void bands_store_set_row_vals(GtkListStore *bands_list, GtkTreeIter *iter, int band,
+				     struct mband_dataset *mbds)
 {
 	char c[50 * 10];
 	int i;
-	
+
+	memset(c, 0, sizeof(c));
 	snprintf(c, 40, "%s", mbds->trans[band].bname);
+	d4_printf("%s\n", c);
 	if (mbds->trans[band].c1 >= 0 && mbds->trans[band].c2 >= 0) {
 		snprintf(c+50, 40, "%s-%s", mbds->trans[mbds->trans[band].c1].bname,
 			 mbds->trans[mbds->trans[band].c2].bname);
 		if (mbds->trans[band].kerr < BIG_ERR) {
-			snprintf(c+100, 40, "%.3f/%.3f", mbds->trans[band].k, 
+			snprintf(c+100, 40, "%.3f/%.3f", mbds->trans[band].k,
 				 mbds->trans[band].kerr);
 		} else {
 			c[100] = 0;
@@ -666,87 +802,99 @@ static void bands_list_set_row_vals(GtkWidget *bands_list, int row, int band,
 		c[100] = 0;
 	}
 	if (mbds->trans[band].zzerr < BIG_ERR) {
-		snprintf(c+150, 40, "%.3f/%.3f", mbds->trans[band].zz, 
+		snprintf(c+150, 40, "%.3f/%.3f", mbds->trans[band].zz,
 			 mbds->trans[band].zzerr);
 		snprintf(c+200, 40, "%.3f", mbds->trans[band].am);
-		snprintf(c+250, 40, "%.3f/%.3f", -mbds->trans[band].e, 
+		snprintf(c+250, 40, "%.3f/%.3f", -mbds->trans[band].e,
 			 mbds->trans[band].eerr);
 		snprintf(c+300, 40, "%.2f", mbds->trans[band].eme1);
 	} else {
 		c[150] = c[200] = c[250] = c[300] = 0;
 	}
 
-	for (i = 0; i < 7; i++)
-		gtk_clist_set_text(GTK_CLIST(bands_list), row, i, c+50*i);
+	gtk_list_store_set(bands_list, iter, 0, &mbds->trans[band], -1);
+	for (i = 0; i < 6; i++) {
+		gtk_list_store_set(bands_list, iter, i + 1, c + 50 * i, -1);
+	}
 }
 
 static void bands_list_update_vals(GtkWidget *dialog, struct mband_dataset *mbds)
 {
 	GtkWidget *list;
+	GtkListStore *store;
+	GtkTreeIter iter;
 	int i;
 
 	list = gtk_object_get_data(GTK_OBJECT(dialog), "bands_list");
 	g_return_if_fail(mbds != NULL);
 	g_return_if_fail(list != NULL);
-	gtk_clist_freeze(GTK_CLIST(list));
-	for (i = 0; i < mbds->nbands; i++) {
-		bands_list_set_row_vals(list, i, i, mbds);
-	}
-	gtk_clist_thaw(GTK_CLIST(list));
-}
 
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(list)));
+
+	gtk_list_store_clear(store);
+	for (i = 0; i < mbds->nbands; i++) {
+
+		gtk_list_store_append(store, &iter);
+		bands_store_set_row_vals(store, &iter, i, mbds);
+	}
+}
 
 static void mbds_bands_to_list(GtkWidget *dialog, GtkWidget *list)
 {
 	struct mband_dataset *mbds;
-	GList *sl;
-	int row, i;
-	char *line [] = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
+
 	mbds = gtk_object_get_data(GTK_OBJECT(dialog), "mbds");
 	if (mbds == NULL)
 		return;
-	gtk_clist_freeze(GTK_CLIST(list));
-	sl = mbds->ofrs;
-	for (i = 0; i < mbds->nbands; i++) {
-		row = gtk_clist_append(GTK_CLIST(list), line);
-		bands_list_set_row_vals(list, row, i, mbds);
-	}
-	for (i = 0; i < 7; i++) {
-		gtk_clist_set_column_auto_resize(GTK_CLIST(list), i, 1);
-		gtk_clist_set_column_justification(GTK_CLIST(list), i, GTK_JUSTIFY_CENTER);
-	}
-	gtk_clist_thaw(GTK_CLIST(list));
-}
 
+	bands_list_update_vals(dialog, mbds);
+}
 
 static void mb_rebuild_bands_list(gpointer dialog)
 {
 	GtkWidget *list;
 	GtkScrolledWindow *scw;
-	char *titles[] = {"Band", "Color", "Color coeff", 
-			  "All-sky zeropoint", "Mean airmass", 
+	GtkListStore *store;
+	GtkTreeViewColumn *column;
+	char *titles[] = {"Band", "Color", "Color coeff",
+			  "All-sky zeropoint", "Mean airmass",
 			  "Extinction coeff", "Extinction me1", NULL};
+	int i;
 
 	list = gtk_object_get_data(GTK_OBJECT(dialog), "bands_list");
 	if (list == NULL) {
 		scw = gtk_object_get_data(GTK_OBJECT(dialog), "bands_scw");
 		g_return_if_fail(scw != NULL);
-		list = gtk_clist_new_with_titles(8, titles);
+
+		store = gtk_list_store_new(8, G_TYPE_POINTER,
+					   G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+					   G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+
+		list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+		g_object_unref(store);
+
+		for (i = 0; i < 7; i++) {
+			column = gtk_tree_view_column_new_with_attributes(titles[i],
+									  gtk_cell_renderer_text_new(),
+									  "text", i + 1, NULL);
+
+			gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
+		}
+
 		gtk_scrolled_window_add_with_viewport(scw, list);
 		gtk_widget_ref(list);
 		gtk_object_set_data_full(GTK_OBJECT(dialog), "bands_list",
 					 list, (GtkDestroyNotify) gtk_widget_destroy);
-		gtk_clist_column_titles_passive(GTK_CLIST(list));
-//		gtk_clist_set_selection_mode(GTK_CLIST(list), GTK_SELECTION_EXTENDED);
 		gtk_widget_show(list);
 	} else {
-		gtk_clist_clear(GTK_CLIST(list));
+		store = GTK_LIST_STORE (gtk_tree_view_get_model(GTK_TREE_VIEW(list)));
+		gtk_list_store_clear(store);
 	}
+
 	mbds_bands_to_list(dialog, list);
 }
 
-
-static void sob_list_set_row_vals(GtkWidget *sob_list, int row, struct star_obs *sob)
+static void sob_store_set_row_vals(GtkListStore *sob_list, GtkTreeIter *iter, struct star_obs *sob)
 {
 	char c[50 * 15];
 	double se;
@@ -780,7 +928,7 @@ static void sob_list_set_row_vals(GtkWidget *sob_list, int row, struct star_obs
 		} else if (sob->ofr->band >= 0 && sob->err < BIG_ERR) {
 			snprintf(c+150, 40, "[%5.2f]", sob->mag);
 			snprintf(c+200, 40, "%.3f", sob->err);
-		} 
+		}
 	} else {
 		if (sob->ofr->band >= 0 && sob->err < BIG_ERR) {
 			snprintf(c+150, 40, "[%5.2f]", sob->mag);
@@ -809,84 +957,100 @@ static void sob_list_set_row_vals(GtkWidget *sob_list, int row, struct star_obs
 		flags >>= 1;
 		i++;
 	}
-	for (i = 0; i < 13; i++)
-		gtk_clist_set_text(GTK_CLIST(sob_list), row, i, c+50*i);
+
+	gtk_list_store_set(sob_list, iter, 0, sob, -1);
+	for (i = 0; i < 13; i++) {
+		gtk_list_store_set(sob_list, iter, i + 1, c + 50 * i, -1);
+	}
 }
 
 static void sob_list_update_vals(GtkWidget *sob_list)
 {
-	int i;
-	for (i = 0; i < GTK_CLIST(sob_list)->rows; i++)
-		sob_list_set_row_vals(sob_list, i, 
-				      STAR_OBS(gtk_clist_get_row_data(GTK_CLIST(sob_list), i)));
-}
+	struct star_obs *sob;
+	GtkTreeModel *store;
+	GtkTreeIter iter;
 
+	store = gtk_tree_view_get_model(GTK_TREE_VIEW(sob_list));
+
+	if (gtk_tree_model_get_iter_first(store, &iter)) {
+		do {
+			gtk_tree_model_get(store, &iter, 0, &sob, -1);
+			if (sob == NULL)
+				continue;
+
+			sob_store_set_row_vals(GTK_LIST_STORE(store), &iter, sob);
+		} while (gtk_tree_model_iter_next(store, &iter));
+	}
+}
 
 static void sobs_to_list(GtkWidget *list, GList *sol)
 {
 	GList *sl;
 	struct star_obs *sob;
-	int row, i;
-	char *line [] = {NULL, NULL, NULL, NULL, NULL, NULL, NULL, 
-			 NULL, NULL, NULL, NULL, NULL, NULL, NULL};
-	gtk_clist_freeze(GTK_CLIST(list));
+	GtkTreeModel *store;
+	GtkTreeIter iter;
+
+	store = gtk_tree_view_get_model(GTK_TREE_VIEW(list));
+
 	sl = sol;
-	while(sl != NULL) {
+	while (sl != NULL) {
 		sob = STAR_OBS(sl->data);
 		sl = g_list_next(sl);
-		row = gtk_clist_append(GTK_CLIST(list), line);
-		sob_list_set_row_vals(list, row, sob);
-		gtk_clist_set_row_data(GTK_CLIST(list), row, sob);
-		sob->data = row;
+
+		gtk_list_store_append(GTK_LIST_STORE(store), &iter);
+		sob_store_set_row_vals(GTK_LIST_STORE(store), &iter, sob);
 	}
-	for (i = 0; i < 13; i++) {
-		gtk_clist_set_column_auto_resize(GTK_CLIST(list), i, 1);
-		if (i > 0 && i < 12)
-			gtk_clist_set_column_justification(GTK_CLIST(list), 
-							   i, GTK_JUSTIFY_CENTER);
-	}
-	gtk_clist_thaw(GTK_CLIST(list));
-}
-
-
-static void sob_click_col_cb(GtkCList *clist, gint column, gpointer data)
-{
-	static int lastcol = -1;
-	gtk_clist_set_sort_column(GTK_CLIST(clist), column);
-	if (column == lastcol) {
-		lastcol = -1;
-		gtk_clist_set_sort_type(GTK_CLIST(clist), GTK_SORT_DESCENDING);
-	} else {
-		lastcol = column;
-		gtk_clist_set_sort_type(GTK_CLIST(clist), GTK_SORT_ASCENDING);
-	}
-	gtk_clist_sort(GTK_CLIST(clist));
 }
 
+/* FIXME */
 static void mb_rebuild_sob_list(gpointer dialog, GList *sol)
 {
 	GtkWidget *list;
 	GtkScrolledWindow *scw;
-	char *titles[] = {"Name", "Type", "Band", "Smag", "Err", "Imag", "Err", "Residual", 
+	GtkListStore *store;
+	GtkTreeViewColumn *column;
+	int i;
+	char *titles[] = {"Name", "Type", "Band", "Smag", "Err", "Imag", "Err", "Residual",
 			  "Std Error", "Outlier", "R.A.", "Declination", "Flags", NULL};
 
 	list = gtk_object_get_data(GTK_OBJECT(dialog), "sob_list");
 	if (list == NULL) {
 		scw = gtk_object_get_data(GTK_OBJECT(dialog), "sob_scw");
 		g_return_if_fail(scw != NULL);
-		list = gtk_clist_new_with_titles(14, titles);
+
+		store = gtk_list_store_new(14, G_TYPE_POINTER,
+					   G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+					   G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+					   G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
+					   G_TYPE_STRING, G_TYPE_STRING);
+
+		list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+		g_object_unref(store);
+
+		for (i = 0; i < 13; i++) {
+			column = gtk_tree_view_column_new_with_attributes(titles[i],
+									  gtk_cell_renderer_text_new(),
+									  "text", i + 1, NULL);
+
+			gtk_tree_view_column_set_sort_column_id(column, i + 1);
+			gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
+		}
+
+		gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(list)),
+					    GTK_SELECTION_MULTIPLE);
+
+
 		gtk_scrolled_window_add_with_viewport(scw, list);
 		gtk_widget_ref(list);
 		gtk_object_set_data_full(GTK_OBJECT(dialog), "sob_list",
 					 list, (GtkDestroyNotify) gtk_widget_destroy);
-		gtk_clist_set_selection_mode(GTK_CLIST(list), GTK_SELECTION_EXTENDED);
-		gtk_signal_connect(GTK_OBJECT(list), "click-column", sob_click_col_cb, dialog);
-//		gtk_clist_set_column_justification(GTK_CLIST(list), 5, GTK_JUSTIFY_RIGHT);
-//		gtk_clist_set_column_justification(GTK_CLIST(list), 3, GTK_JUSTIFY_RIGHT);
+
 		gtk_widget_show(list);
 	} else {
-		gtk_clist_clear(GTK_CLIST(list));
+		store = GTK_LIST_STORE (gtk_tree_view_get_model(GTK_TREE_VIEW(list)));
+		gtk_list_store_clear(store);
 	}
+
 	sobs_to_list(list, sol);
 }
 
@@ -900,11 +1064,11 @@ void add_to_mband(gpointer dialog, char *fn)
 //	d1_printf("loading report file: %s\n", fn);
 	inf = fopen(fn, "r");
 	if (inf == NULL) {
-		mbds_printf(dialog, "Cannot open file %s for reading: %s\n", 
+		mbds_printf(dialog, "Cannot open file %s for reading: %s\n",
 			   fn, strerror(errno));
 		error_beep();
 		return;
-	} 
+	}
 
 	mbds = gtk_object_get_data(GTK_OBJECT(dialog), "mbds");
 	if (mbds == NULL) {
@@ -920,12 +1084,12 @@ void add_to_mband(gpointer dialog, char *fn)
 		ret = mband_dataset_add_stf(mbds, stf);
 		if (ret >= 0) {
 			n++;
-		} 
+		}
 		mbds_printf(dialog, "%d frames read", n);
 		while (gtk_events_pending ())
 			gtk_main_iteration ();
-		
-	} 
+
+	}
 	if (n == 0) {
 		error_beep();
 		mbds_printf(dialog, "%s: %s", fn, last_err());
@@ -1021,7 +1185,7 @@ static void mb_wclose_cb(gpointer data, guint action, GtkWidget *menu_item)
 
 static GtkItemFactoryEntry mband_menu_items[] = {
 	{ "/_File",		NULL,         	NULL,  		0, "<Branch>" },
-	{ "/File/_Add to Dataset", "<control>o", 	file_popup_cb, 	
+	{ "/File/_Add to Dataset", "<control>o", 	file_popup_cb,
 	  		FILE_ADD_TO_MBAND, "<Item>" },
 	{ "/File/_Save Dataset", "<control>s", rep_cb, REP_DATASET | REP_ALL, "<Item>"},
 	{ "/File/_Close Dataset", "<control>w",	mb_close_cb, 	0, "<Item>" },
@@ -1051,7 +1215,7 @@ static GtkItemFactoryEntry mband_menu_items[] = {
 	{ "/_Plot", NULL, NULL,	0, "<Branch>" },
 	{ "/_Plot/_Residuals vs Magnitude", NULL, plot_cb, PLOT_RES_SM, "<Item>" },
 	{ "/_Plot/Residuals vs _Color", NULL, plot_cb, PLOT_RES_COL, "<Item>" },
-	{ "/_Plot/Standard _Errors vs Magnitude", NULL, plot_cb, 
+	{ "/_Plot/Standard _Errors vs Magnitude", NULL, plot_cb,
 	  PLOT_RES_SM | PLOT_WEIGHTED, "<Item>" },
 	{ "/_Plot/_Standard Errors vs Color", NULL, plot_cb,
 	  PLOT_RES_COL | PLOT_WEIGHTED, "<Item>" },
@@ -1073,21 +1237,21 @@ static GtkWidget *get_main_menu_bar(GtkWidget *window)
 	GtkWidget *ret;
 	GtkItemFactory *item_factory;
 	GtkAccelGroup *accel_group;
-	gint nmenu_items = sizeof (mband_menu_items) / 
+	gint nmenu_items = sizeof (mband_menu_items) /
 		sizeof (mband_menu_items[0]);
 	accel_group = gtk_accel_group_new ();
 
-	item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, 
+	item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR,
 					     "<main_menu>", accel_group);
 	gtk_object_set_data_full(GTK_OBJECT(window), "main_menu_if", item_factory,
 				 (GtkDestroyNotify) gtk_object_unref);
-	gtk_item_factory_create_items (item_factory, nmenu_items, 
+	gtk_item_factory_create_items (item_factory, nmenu_items,
 				       mband_menu_items, window);
 
-  /* Attach the new accelerator group to the window. */
+	/* Attach the new accelerator group to the window. */
 	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
 
-    /* Finally, return the actual menu bar created by the item factory. */ 
+	/* Finally, return the actual menu bar created by the item factory. */
 	ret = gtk_item_factory_get_widget (item_factory, "<main_menu>");
 	return ret;
 }
@@ -1106,7 +1270,7 @@ void mband_open_cb(gpointer data, guint action, GtkWidget *menu_item)
 					 dialog, (GtkDestroyNotify)(gtk_widget_unref));
 		gtk_object_set_data(GTK_OBJECT(dialog), "in_window", window);
 		vb = gtk_object_get_data(GTK_OBJECT(dialog), "mband_vbox");
-		gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", 
+		gtk_signal_connect(GTK_OBJECT(dialog), "delete_event",
 				   GTK_SIGNAL_FUNC(mband_window_delete), window);
 		menubar = get_main_menu_bar(dialog);
 		gtk_box_pack_start(GTK_BOX(vb), menubar, FALSE, TRUE, 0);
