@@ -9,8 +9,9 @@ void named_spin_set(GtkWidget *dialog, char *name, double val);
 int get_named_checkb_val(GtkWidget *dialog, char *name);
 char * named_entry_text(GtkWidget *dialog, char *name);
 void named_entry_set(GtkWidget *dialog, char *name, char *text);
-void set_named_callback(void *dialog, char *name, char *callback, void *func);
-void set_named_callback_data(void *dialog, char *name, char *callback, 
+void named_cbentry_set(GtkWidget *dialog, char *name, char *text);
+long set_named_callback(void *dialog, char *name, char *callback, void *func);
+long set_named_callback_data(void *dialog, char *name, char *callback, 
 			     void *func, gpointer data);
 void set_named_checkb_val(GtkWidget *dialog, char *name, int val);
 int check_seq_number(char *file, int *sqn);
