@@ -46,7 +46,6 @@
 #include "params.h"
 #include "interface.h"
 #include "wcs.h"
-#include "camera.h"
 #include "cameragui.h"
 #include "filegui.h"
 
@@ -203,6 +202,15 @@ static void mband_dataset_add_sob(struct mband_dataset *mbds, struct cat_star *c
 		  sob->imag, sob->imagerr);
 }
 
+void d3_print_decimal_string(char *c)
+{
+	while(*c) {
+		d3_printf("%d ", (*c) & 0xff);
+		c++;
+	}
+	d3_printf("\n");
+}
+
 int mband_dataset_search_add_band(struct mband_dataset *mbds, char *band)
 {
 	int i;
@@ -213,6 +221,8 @@ int mband_dataset_search_add_band(struct mband_dataset *mbds, char *band)
 
 	if (mbds->nbands >= MAX_MBANDS)
 		return -1;
+	d3_printf("%s\n", band);
+	d3_print_decimal_string(band);
 	mbds->trans[mbds->nbands].bname = strdup(band);
 	mbds->trans[mbds->nbands].c1 = -1;
 	mbds->trans[mbds->nbands].c2 = -1;
@@ -675,7 +685,7 @@ void mbds_fit_all(GList *ofrs, int (* progress)(char *msg, void *data), void *da
 		}
 	}
 	for (bl = bsl; bl != NULL; bl = g_list_next(bl)) {
-		band = (int) bl->data;
+		band = (long) bl->data;
 		d3_printf("band: %d\n", band);
 		mbds_fit_band(ofrs, band, progress, data);
 	}
@@ -691,7 +701,9 @@ void mband_dataset_set_bands_by_string(struct mband_dataset *mbds, char *bspec)
 	char buf[64];
 	double v;
 	char *endp;
+	int l;
 
+//	d1_printf("bandspec string '%s'\n", bspec);
 
 	b = -1; c1 = -1; c2 = -1; tok = TOK_EOL;
 	next_token(NULL, NULL, NULL);
@@ -704,7 +716,9 @@ void mband_dataset_set_bands_by_string(struct mband_dataset *mbds, char *bspec)
 			b = -1; c1 = -1; c2 = -1;
 			if (tok != TOK_WORD) 
 				break;
-			strncpy(buf, start, end - start > 63 ? 63 : end - start);
+			l = end - start > 63 ? 63 : end - start;
+			strncpy(buf, start, l);
+			buf[l] = 0;
 			b = mband_dataset_search_add_band(mbds, buf);
 			state = 1;
 			break;
@@ -721,14 +735,18 @@ void mband_dataset_set_bands_by_string(struct mband_dataset *mbds, char *bspec)
 			break;
 		case 2:		/* wait for first part of color index */
 			if (tok == TOK_WORD) {
-				strncpy(buf, start, end - start > 63 ? 63 : end - start);
+				l = end - start > 63 ? 63 : end - start;
+				strncpy(buf, start, l);
+				buf[l] = 0;
 				c1 = mband_dataset_search_add_band(mbds, buf);
 				state = 3;
 			}
 			break;
 		case 3:		/* wait for second part of color index or close paren */
 			if (tok == TOK_WORD) {
-				strncpy(buf, start, end - start > 63 ? 63 : end - start);
+				l = end - start > 63 ? 63 : end - start;
+				strncpy(buf, start, l);
+				buf[l] = 0;
 				c2 = mband_dataset_search_add_band(mbds, buf);
 				state = 4;
 			}
@@ -1450,7 +1468,7 @@ int fit_all_sky_zp(struct mband_dataset *mbds, GList *ofrs)
 	}
 	tlist = g_list_sort(tlist, (GCompareFunc)ofr_time_compare);
 	for (bl = bsl; bl != NULL; bl = g_list_next(bl)) {
-		band = (int) bl->data;
+		band = (long) bl->data;
 		fofrs = NULL;
 //		d3_printf("band: %d\n", band);
 		/* do the fit for each band */
@@ -1471,7 +1489,7 @@ int fit_all_sky_zp(struct mband_dataset *mbds, GList *ofrs)
 		}
 	}
 	for (bl = bsl; bl != NULL; bl = g_list_next(bl)) {
-		band = (int) bl->data;
+		band = (long) bl->data;
 		amlist = NULL;
 //		d3_printf("band: %d\n", band);
 		/* do the fit for each band */
