@@ -34,7 +34,8 @@
 #include <gtk/gtk.h>
 
 #include "gcx.h"
-#include "camera.h"
+#include "camera_indi.h"
+#include "tele_indi.h"
 #include "catalogs.h"
 #include "gui.h"
 #include "obsdata.h"
@@ -45,9 +46,8 @@
 #include "misc.h"
 #include "cameragui.h"
 #include "interface.h"
-#include "fwheel.h"
+#include "fwheel_indi.h"
 #include "multiband.h"
-#include "scope.h"
 
 static void obslist_background(gpointer window, int action);
 
@@ -64,6 +64,7 @@ typedef enum {
 	OBS_START,
 	OBS_DO_COMMAND,
 	OBS_NEXT_COMMAND,
+	OBS_CMD_RUNNING,
 	OBS_CMD_ERROR,
 	OBS_SKIP_OBJECT,
 } ObsState;
@@ -205,6 +206,20 @@ static char *get_cmd_line(GtkList *list, int index)
 	return cmd;
 }
 
+/* change the state-machine state, and schedule that it be called */
+static void obs_list_set_state(GtkWidget *dialog, long state)
+{
+	g_object_set_data(G_OBJECT (dialog), "obs_list_state", (void *)state);
+	g_idle_add((GSourceFunc)obs_list_sm, dialog);
+}
+
+/* last command has finished, so fire an event to start the next one */
+static int obs_list_cmd_done(GtkWidget *dialog)
+{
+	obs_list_set_state(dialog, OBS_NEXT_COMMAND);
+	return FALSE;
+}
+
 /* actual obs commands - which are really parts of the state machine 
  * they all get called from the OBS_DO_COMMAND state, and return the state
  * the machine should jump to. The commands generally manipulate the cam_control
@@ -212,49 +227,39 @@ static char *get_cmd_line(GtkList *list, int index)
 
 static int do_get_cmd (char *args, GtkWidget *dialog)
 {
-	struct ccd *ccd;
-	struct ccd_frame *fr;
-	int ret;
+	GtkWidget *main_window = gtk_object_get_data(GTK_OBJECT(dialog), "image_window");
+	struct camera_t *camera = camera_find(main_window, CAMERA_MAIN);
+	if(! camera) {
+		err_printf("no camera connected\n");
+		return OBS_CMD_ERROR;
+	}
 
 	set_named_checkb_val(dialog, "img_get_multiple_button", 0);
 	set_named_checkb_val(dialog, "img_dark_checkb", 0);
-	ccd = gtk_object_get_data(GTK_OBJECT(dialog), "open_camera");
-	if (ccd != NULL) {
-		set_exp_from_img_dialog(ccd, GTK_WIDGET(dialog), NULL);
-		ccd_set_exposure(ccd, &ccd->exp);
-		ccd_read_exposure(ccd, &ccd->exp);
-		cam_to_img(GTK_WIDGET(dialog));
-		fr = new_frame(ccd->exp.w, ccd->exp.h);
-		if (fr == NULL) {
-			err_printf("Cannot create %dx%d frame\n", 
-				   ccd->exp.w, ccd->exp.h);
-			return OBS_CMD_ERROR;
-		}
-		gtk_object_set_data_full(GTK_OBJECT(dialog), "current_frame", 
-					 fr, (GtkDestroyNotify)(release_frame));
-		ret = ccd_start_exposure (ccd, 0);
-	} else {
-		ret = -1;
-		err_printf("No CCD\n");
-	}
-	if (ret < 0)
+	if (capture_image(dialog)) {
+		err_printf("Failed to capture frame\n");
 		return OBS_CMD_ERROR;
-	else
-		return OBS_NEXT_COMMAND;
+	}
+
+	INDI_set_callback(INDI_COMMON (camera), CAMERA_CALLBACK_EXPOSE, obs_list_cmd_done, dialog);
+	return OBS_CMD_RUNNING;
 }
 
 static int do_filter_cmd (char *args, GtkWidget *dialog)
 {
-	struct fwheel *fw;
+	struct fwheel_t *fwheel;
+	GtkWidget *window;
 	GtkWidget *combo;
 	int i;
 	char **filters;
 
-	fw = gtk_object_get_data(GTK_OBJECT(dialog), "open_fwheel");
-	if (fw == NULL) {
-		err_printf("Filter wheel error or no wheel connected\n");
+	window = gtk_object_get_data(GTK_OBJECT(dialog), "image_window");
+	fwheel = fwheel_find(window);
+	if (! fwheel) {
+		err_printf("No filter wheel detected\n");
 		return OBS_CMD_ERROR;
 	}
+	filters = fwheel_get_filters(fwheel);
 	i = 0;
 	while (args[i]) {
 		if (isspace(args[i])) {
@@ -263,13 +268,14 @@ static int do_filter_cmd (char *args, GtkWidget *dialog)
 		}
 		i++;
 	} 
-	filters = fwheel_get_filter_names(fw);
 	i = 0;
 	while (*filters != NULL) {
 		if (!strcasecmp(*filters, args)) {
 			combo = gtk_object_get_data(GTK_OBJECT(dialog), "obs_filter_combo");
 			gtk_list_select_item(GTK_LIST(GTK_COMBO(combo) -> list), i);
-			return OBS_NEXT_COMMAND;
+			INDI_set_callback(INDI_COMMON (fwheel), FWHEEL_CALLBACK_DONE,
+			                  obs_list_cmd_done, dialog);
+			return OBS_CMD_RUNNING;
 		}
 		filters ++;
 		i++;
@@ -295,22 +301,21 @@ static int do_exp_cmd (char *args, GtkWidget *dialog)
 
 static int do_dark_cmd (char *args, GtkWidget *dialog)
 {
-	struct ccd *ccd;
-	int ret;
+	GtkWidget *main_window = gtk_object_get_data(GTK_OBJECT(dialog), "image_window");
+	struct camera_t *camera = camera_find(main_window, CAMERA_MAIN);
+	if(! camera) {
+		err_printf("no camera connected\n");
+		return OBS_CMD_ERROR;
+	}
 
 	set_named_checkb_val(dialog, "img_get_multiple_button", 0);
 	set_named_checkb_val(dialog, "img_dark_checkb", 1);
-	ccd = gtk_object_get_data(GTK_OBJECT(dialog), "open_camera");
-	if (ccd != NULL) {
-		ret = ccd_start_exposure (ccd, 1);
-	} else {
-		ret = -1;
-		err_printf("No CCD\n");
-	}
-	if (ret < 0)
+	if (capture_image(dialog)) {
+		err_printf("Failed to capture frame\n");
 		return OBS_CMD_ERROR;
-	else
-		return OBS_NEXT_COMMAND;
+	}
+	INDI_set_callback(INDI_COMMON (camera), CAMERA_CALLBACK_EXPOSE, obs_list_cmd_done, dialog);
+	return OBS_CMD_RUNNING;
 }
 
 /* check that the object in obs in within the limits set in the dialog
@@ -402,7 +407,7 @@ static int do_goto_cmd (char *args, GtkWidget *dialog)
 		}
 	}
 	if ( !goto_dialog_obs(dialog))
-		return OBS_NEXT_COMMAND;
+		return OBS_CMD_RUNNING;
 	else
 		return OBS_CMD_ERROR;
 }
@@ -410,28 +415,40 @@ static int do_goto_cmd (char *args, GtkWidget *dialog)
 static int do_match_cmd (char *args, GtkWidget *dialog)
 {
 	void *imw;
+	struct tele_t *tele;
 	int ret;
 	imw = gtk_object_get_data(GTK_OBJECT(dialog), "image_window");
 	g_return_val_if_fail(imw != NULL, OBS_CMD_ERROR);
+	tele = tele_find(imw);
+	if(! tele) {
+		err_printf("no telescope connected\n");
+		return OBS_CMD_ERROR;
+	}
+
 	ret = match_field_in_window(imw);
 	if (ret < 0) {
 		d3_printf("Cannot match\n");
 		return OBS_CMD_ERROR;
 	}
-	if ( !center_matched_field(dialog))
-		return OBS_NEXT_COMMAND;
-	else
+	if ( center_matched_field(dialog))
 		return OBS_CMD_ERROR;
+	INDI_set_callback(INDI_COMMON (tele), TELE_CALLBACK_STOP, obs_list_cmd_done, dialog);
+	return OBS_CMD_RUNNING;
 }
 
 static int do_mget_cmd (char *args, GtkWidget *dialog)
 {
-	struct ccd *ccd;
-	struct ccd_frame *fr;
-	int ret;
 	char *text, *start, *end;
 	int token, n;
 
+	GtkWidget *main_window = gtk_object_get_data(GTK_OBJECT(dialog), "image_window");
+	struct camera_t *camera = camera_find(main_window, CAMERA_MAIN);
+	if(! camera) {
+		err_printf("no camera connected\n");
+		return OBS_CMD_ERROR;
+	}
+
+
 	text = args;
 	next_token(NULL, NULL, NULL);
 	token = next_token(&text, &start, &end);
@@ -442,29 +459,12 @@ static int do_mget_cmd (char *args, GtkWidget *dialog)
 	}
 	set_named_checkb_val(dialog, "img_get_multiple_button", 1);
 	set_named_checkb_val(dialog, "img_dark_checkb", 0);
-	ccd = gtk_object_get_data(GTK_OBJECT(dialog), "open_camera");
-	if (ccd != NULL) {
-		set_exp_from_img_dialog(ccd, GTK_WIDGET(dialog), NULL);
-		ccd_set_exposure(ccd, &ccd->exp);
-		ccd_read_exposure(ccd, &ccd->exp);
-		cam_to_img(GTK_WIDGET(dialog));
-		fr = new_frame(ccd->exp.w, ccd->exp.h);
-		if (fr == NULL) {
-			err_printf("do_get_cmd: cannot create %dx%d frame\n", 
-				   ccd->exp.w, ccd->exp.h);
-			return OBS_CMD_ERROR;
-		}
-		gtk_object_set_data_full(GTK_OBJECT(dialog), "current_frame", 
-					 fr, (GtkDestroyNotify)(release_frame));
-		ret = ccd_start_exposure (ccd, 0);
-	} else {
-		ret = -1;
-		err_printf("No CCD\n");
-	}
-	if (ret < 0)
+	if (capture_image(dialog)) {
+		err_printf("Failed to capture frame\n");
 		return OBS_CMD_ERROR;
-	else
-		return OBS_NEXT_COMMAND;
+	}
+	INDI_set_callback(INDI_COMMON (camera), CAMERA_CALLBACK_EXPOSE, obs_list_cmd_done, dialog);
+	return OBS_CMD_RUNNING;
 }
 
 /* 1/cos(dec), clamped at 5 */
@@ -477,6 +477,18 @@ static double dec_factor(double dec)
 
 }
 
+/* Process rest of ckpoint command after movement is done */
+static int obs_list_ckpoint_move_done(GtkWidget *dialog)
+{
+	if (!lx_sync_to_obs(dialog)) {
+		obs_list_set_state(dialog, do_get_cmd(NULL, dialog));
+	} else {
+		obs_list_set_state(dialog, OBS_NEXT_COMMAND);
+	}
+	return FALSE;
+}
+
+
 static int do_ckpoint_cmd (char *args, GtkWidget *dialog)
 {
 	int ret;
@@ -484,10 +496,16 @@ static int do_ckpoint_cmd (char *args, GtkWidget *dialog)
 	struct wcs *wcs;
 	struct obs_data *obs;
 	double cerr;
-
+	struct tele_t *tele;
 
 	imw = gtk_object_get_data(GTK_OBJECT(dialog), "image_window");
 	g_return_val_if_fail(imw != NULL, OBS_CMD_ERROR);
+	tele = tele_find(imw);
+	if(! tele) {
+		err_printf("no telescope connected\n");
+		return OBS_CMD_ERROR;
+	}
+
 	ret = match_field_in_window_quiet(imw);
 	if (ret < 0) {
 		err_printf("Cannot match\n");
@@ -507,17 +525,12 @@ static int do_ckpoint_cmd (char *args, GtkWidget *dialog)
 	cerr = sqrt(sqr(wcs->xref - obs->ra) / dec_factor (obs->dec)
 		    + sqr(wcs->yref - obs->dec));
 	d3_printf("centering error is %.3f\n", cerr);
-	if (cerr > P_DBL(MAX_POINTING_ERR)) {
-		if (center_matched_field(dialog))
-			return OBS_CMD_ERROR;
-	} else {
+	if (cerr <= P_DBL(MAX_POINTING_ERR))
 		return OBS_NEXT_COMMAND;
-	}
-	if (!lx_sync_to_obs(dialog)) {
-		return do_get_cmd(NULL, dialog);
-	} else {
-		return OBS_NEXT_COMMAND;
-	}
+	if (center_matched_field(dialog))
+		return OBS_CMD_ERROR;
+	INDI_set_callback(INDI_COMMON (tele), TELE_CALLBACK_STOP, obs_list_ckpoint_move_done, dialog);
+	return OBS_CMD_RUNNING;
 }
 
 static int do_phot_cmd (char *args, GtkWidget *dialog)
@@ -595,22 +608,19 @@ static int do_qmatch_cmd (char *args, GtkWidget *dialog)
 static int lx_sync_to_obs(gpointer dialog)
 {
 	struct obs_data *obs;
-	struct scope *scope;
+	struct tele_t *tele;
 	int ret;
+	GtkWidget *main_window = g_object_get_data(G_OBJECT (dialog), "image_window");
 
 	obs = gtk_object_get_data(GTK_OBJECT(dialog), "obs_data");
 	if (obs == NULL) {
 		err_printf("No obs data for syncing\n");
 		return -1;
 	}
-	scope = scope_open(P_STR(FILE_SCOPE_SERIAL));
-	if (scope == NULL) {
-		error_beep();
-		status_message(dialog, last_err());
+	tele = tele_find(main_window);
+	if (! tele)
 		return -1;
-	}
-	ret = scope_sync_coords(scope, obs->ra, obs->dec, obs->equinox);
-	scope_close(scope);
+	ret = tele_set_coords(tele, TELE_COORDS_SYNC, obs->ra, obs->dec, obs->equinox);
 	return ret;
 }
 
@@ -634,33 +644,36 @@ static void center_selected(gpointer user_data, GtkList *list, int index)
 	}
 }
 
-/* called from the cam_dialog state machine whenever the camera is idle (all 
- * camera ops have ended; tries to go through the obs list */
+void obs_list_start_cb(GtkWidget *widget, gpointer data)
+{
+	GtkWidget *dialog = (GtkWidget *)data;
+	GtkWidget *list;
+
+	list = gtk_object_get_data(GTK_OBJECT(dialog), "list1");
+	g_return_if_fail(list != NULL);
+	if (get_named_checkb_val(dialog, "obs_list_run_button")) {
+		obs_list_set_state(dialog, OBS_DO_COMMAND);
+	}
+	else if (get_named_checkb_val(dialog, "obs_list_step_button")) {
+		set_named_checkb_val(dialog, "obs_list_step_button", 0);
+		obs_list_set_state(dialog, OBS_DO_COMMAND);
+	}
+}
+/* called via the idle-handler whenever the obs-list state changes */
 void obs_list_sm(GtkWidget *dialog)
 {
-	static ObsState state = OBS_START;
 	int index, all;
 	GtkWidget *list;
 	char *cmd, *cmdo, *cmdh, *cmdho;
 	int cl, clo;
+	long state;
 
+	state = (long)g_object_get_data(G_OBJECT (dialog), "obs_list_state");
 	list = gtk_object_get_data(GTK_OBJECT(dialog), "list1");
 	g_return_if_fail(list != NULL);
 
-	if (state != OBS_START)
-		d4_printf("obs_list_sm state: %d\n", state);
+	d4_printf("obs_list_sm state: %d\n", state);
 	switch(state) {
-	case OBS_START:
-		if (get_named_checkb_val(dialog, "obs_list_run_button")) {
-			state = OBS_DO_COMMAND;
-			break;
-		}
-		if (get_named_checkb_val(dialog, "obs_list_step_button")) {
-			set_named_checkb_val(dialog, "obs_list_step_button", 0);
-			state = OBS_DO_COMMAND;
-			break;
-		}
-		break;
 	case OBS_DO_COMMAND:
 		index = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(dialog), "index"));
 		obslist_background(dialog, OBSLIST_BACKGROUND_RUNNING);
@@ -669,6 +682,7 @@ void obs_list_sm(GtkWidget *dialog)
 //		state = OBS_CMD_WAIT;
 		state = do_command(cmd, dialog);
 		free(cmd);
+		obs_list_set_state(dialog, state);
 		break;
 	case OBS_CMD_ERROR:
 		error_beep();
@@ -676,13 +690,15 @@ void obs_list_sm(GtkWidget *dialog)
 		status_message(dialog, last_err());
 		if (get_named_checkb_val(dialog, "obs_list_err_stop_checkb")) {
 			set_named_checkb_val(dialog, "obs_list_run_button", 0);
-			state = OBS_START;
+			//There isn't ever a reason to set the state to START, since this
+			//state machine is event driven
+			//state = OBS_START;
 //			if (gtk_object_get_data(GTK_OBJECT(dialog), "batch_mode")) {
 //				err_printf("Error in obs file processing, exiting\n");
 //				gtk_exit(2);
 //			}
 		} else {
-			state = OBS_NEXT_COMMAND;
+			obs_list_set_state(dialog, OBS_NEXT_COMMAND);
 		}
 		break;
 	case OBS_SKIP_OBJECT:
@@ -697,7 +713,7 @@ void obs_list_sm(GtkWidget *dialog)
 			cmd = get_cmd_line(GTK_LIST(list), index);
 			cl = cmd_head(cmd, &cmdh, NULL);
 			if (cl == clo && !strncasecmp(cmdh, cmdho, cl)) {
-				state = OBS_NEXT_COMMAND;
+				obs_list_set_state(dialog, OBS_NEXT_COMMAND);
 				free(cmd);
 				break;
 			}
@@ -711,7 +727,9 @@ void obs_list_sm(GtkWidget *dialog)
 
 		if (index >= all) {
 			set_named_checkb_val(dialog, "obs_list_run_button", 0);
-			state = OBS_START;
+			//There isn't ever a reason to set the state to START, since this
+			//state machine is event driven
+			//state = OBS_START;
 			if (gtk_object_get_data(GTK_OBJECT(dialog), "batch_mode")) {
 				err_printf("obs file processing finished successfully\n");
 				gtk_exit(0);
@@ -725,7 +743,9 @@ void obs_list_sm(GtkWidget *dialog)
 		if (index + 1 >= all) {
 			set_named_checkb_val(dialog, "obs_list_run_button", 0);
 			obslist_background(dialog, OBSLIST_BACKGROUND_DONE);
-			state = OBS_START;
+			//There isn't ever a reason to set the state to START, since this
+			//state machine is event driven
+			//state = OBS_START;
 			if (gtk_object_get_data(GTK_OBJECT(dialog), "batch_mode")) {
 				err_printf("obs file processing finished successfully\n");
 				gtk_exit(0);
@@ -735,11 +755,11 @@ void obs_list_sm(GtkWidget *dialog)
 		gtk_list_select_item(GTK_LIST(list), index+1);
 		center_selected(dialog, GTK_LIST(list), index+1);
 		if (get_named_checkb_val(dialog, "obs_list_run_button")) {
-			state = OBS_DO_COMMAND;
-		} else {
-			state = OBS_START;
+			obs_list_set_state(dialog, OBS_DO_COMMAND);
 		}
 		break;
+	default:
+		break;
 	}
 }
 
@@ -760,6 +780,8 @@ void obs_list_callbacks(GtkWidget *dialog)
 	set_named_callback(dialog, "obs_list_file_button", "clicked", browse_cb);
 	combo = gtk_object_get_data(GTK_OBJECT(dialog), "obs_list_fname_combo");
 	gtk_combo_disable_activate(GTK_COMBO(combo));
+	set_named_callback(dialog, "obs_list_run_button", "clicked", obs_list_start_cb);
+	set_named_callback(dialog, "obs_list_step_button", "clicked", obs_list_start_cb);
 
 }
 
