@@ -1,23 +1,23 @@
 /*******************************************************************************
   Copyright(c) 2000 - 2003 Radu Corlan. All rights reserved.
-  
-  This program is free software; you can redistribute it and/or modify it 
-  under the terms of the GNU General Public License as published by the Free 
-  Software Foundation; either version 2 of the License, or (at your option) 
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the Free
+  Software Foundation; either version 2 of the License, or (at your option)
   any later version.
-  
-  This program is distributed in the hope that it will be useful, but WITHOUT 
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
-  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
+
+  This program is distributed in the hope that it will be useful, but WITHOUT
+  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.
-  
+
   You should have received a copy of the GNU General Public License along with
-  this program; if not, write to the Free Software Foundation, Inc., 59 
+  this program; if not, write to the Free Software Foundation, Inc., 59
   Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
   The full GNU General Public License is included in this distribution in the
   file called LICENSE.
-  
+
   Contact Information: radu@corlan.net
 *******************************************************************************/
 #define _GNU_SOURCE
@@ -49,6 +49,12 @@ char * tele_types[] = PAR_CHOICE_TELE_TYPES;
 
 char * stack_methods[] = PAR_CHOICE_STACK_METHODS;
 
+char * demosaic_methods[] = PAR_CHOICE_DEMOSAIC_METHODS;
+
+char * whitebal_methods[] = PAR_CHOICE_WHITEBAL_METHODS;
+
+char * default_cfa[] = PAR_CHOICE_DEFAULT_CFA;
+
 char * guide_algorithms[] = PAR_CHOICE_GUIDE_ALGORITHMS;
 
 char * synth_profiles[] = PAR_CHOICE_SYNTH_PROFILES;
@@ -177,7 +183,7 @@ void make_value_string(GcxPar p, char *c, int len)
 }
 
 /* try to parse the string to a double according to the format
- * return 0 if successfull. 
+ * return 0 if successfull.
  * We need to add stuff for formats other than the 'natural'
  */
 int try_parse_double(GcxPar p, char *text)
@@ -194,7 +200,7 @@ int try_parse_double(GcxPar p, char *text)
 }
 
 /* try to parse the string to an int according to the format
- * return 0 if successfull. 
+ * return 0 if successfull.
  * We need to add stuff for formats other than the 'natural'
  */
 int try_parse_int(GcxPar p, char *text)
@@ -238,8 +244,8 @@ int try_parse_int(GcxPar p, char *text)
 	}
 }
 
-/* change the parameter's value string using a copy of the supplied 
- * text */ 
+/* change the parameter's value string using a copy of the supplied
+ * text */
 void change_par_string(GcxPar p, char *text)
 {
 	if (P_STR(p) != NULL) {
@@ -251,8 +257,8 @@ void change_par_string(GcxPar p, char *text)
 		strcpy(P_STR(p), text);
 }
 
-/* change the parameter's value string using a copy of the supplied 
- * text */ 
+/* change the parameter's value string using a copy of the supplied
+ * text */
 void change_par_default_string(GcxPar p, char *text)
 {
 	if (PAR(p)->defval.s != NULL) {
@@ -303,7 +309,7 @@ void par_item_restore_default(GcxPar p)
 			} else {
 				memcpy(&(PAR(p)->val), &(PAR(p)->defval), sizeof(union pval));
 			}
-			PAR(p)->flags &= ~PAR_USER; 
+			PAR(p)->flags &= ~PAR_USER;
 			PAR(p)->flags &= ~PAR_TO_SAVE;
 	}
 }
@@ -311,7 +317,7 @@ void par_item_restore_default(GcxPar p)
 /* return the logic or of all children of a par subtree */
 int or_child_type(GcxPar p, int type)
 {
-	if (PAR_TYPE(p) != PAR_TREE || PAR(p)->child == PAR_NULL) 
+	if (PAR_TYPE(p) != PAR_TREE || PAR(p)->child == PAR_NULL)
 		return (type | PAR(p)->flags);
 	else {
 		type |= PAR(p)->flags;
@@ -351,7 +357,7 @@ static void fprint_par_leaf(FILE *fp, GcxPar p, char *path)
 	char buf[256];
 //	fprint_indent(fp, level);
 	fprintf(fp, "# %s\n", PAR(p)->comment);
-	if (PAR_TYPE(p) == PAR_INTEGER 
+	if (PAR_TYPE(p) == PAR_INTEGER
 	    && PAR_FORMAT(p) == FMT_BOOL) {
 		if ((PAR_FLAGS(p) & (PAR_FROM_RC | PAR_USER)) == 0) {
 			fprintf(fp, "# ");
@@ -361,8 +367,8 @@ static void fprint_par_leaf(FILE *fp, GcxPar p, char *path)
 		} else {
 			fprintf(fp, "%s.%s 0\n\n", path, PAR(p)->name);
 		}
-	} else if (PAR_TYPE(p) == PAR_INTEGER 
-	    && PAR_FORMAT(p) == FMT_OPTION 
+	} else if (PAR_TYPE(p) == PAR_INTEGER
+	    && PAR_FORMAT(p) == FMT_OPTION
 	    && PAR(p)->choices != NULL) {
 //		fprint_indent(fp, level);
 		c = PAR(p)->choices;
@@ -433,8 +439,8 @@ static void fprint_item(FILE *fp, GcxPar p, char *path)
 
 
 
-/* save params in a file 
- * if p is PAR_NULL, save all the params, otherwise just the 
+/* save params in a file
+ * if p is PAR_NULL, save all the params, otherwise just the
  * given param and it's children
  * returns a negative error code, or 0 for success
  */
@@ -465,13 +471,13 @@ static int is_name_char(char c) {return (isalnum(c)) || (c=='_');}
 static int is_number_char(char c) {return (isdigit(c) || (c) == '.' ||
 					   (c) == 'e' || (c) == 'E'); }
 static int is_number_start(char c) {return (isdigit(c) || c == '-'); }
-/* simple lexical scanner for param file reading 
- * returns the token type 
- * updates the start and end pointers, which 
- * give the first char of the token value, 
+/* simple lexical scanner for param file reading
+ * returns the token type
+ * updates the start and end pointers, which
+ * give the first char of the token value,
  * and the char immediately following the value;
  * Also updates the text pointer to
- * the next char the reader will look at. 
+ * the next char the reader will look at.
  * The scanner never goes back, so the text behind pos
  * can be altered at will.
  * when called with text=NULL, it resets the lex state
@@ -489,10 +495,10 @@ int next_token(char **text, char **start, char **end)
 
 //	d3_printf("text is: %s\n", *text);
 	while (**text != 0) {
-		c = **text; 
+		c = **text;
 		(*text) ++;
 //		d3_printf("st: %d t: %s\n", state, *text);
-			
+
 		switch(state) {
 		case T_START:
 			if (isspace(c))
@@ -588,7 +594,7 @@ static int parse_token_val(GcxPar p, char *start, int flags)
 */
 		if (flags & PAR_FROM_RC) {
 			PAR_FLAGS(p) &= ~PAR_USER;
-		} 
+		}
 		PAR_FLAGS(p) |= flags;
 
 		return 0;
@@ -601,7 +607,7 @@ static GcxPar node_lookup(GcxPar p, char *name, int len)
 {
 	while (p != PAR_NULL) {
 		if (name_matches(PAR(p)->name, name, len))
-//		if (PAR(p)->name && !strncmp(name, PAR(p)->name, len)) 
+//		if (PAR(p)->name && !strncmp(name, PAR(p)->name, len))
 			return p;
 		p = PAR(p)->next;
 	}
@@ -644,7 +650,7 @@ int fscan_params(FILE *fp, GcxPar root)
 	while (ret >= 0) {
 //		if (p)
 //			d3_printf("parse state is %d p=%d (%s)\n", state, p, PAR(p)->name);
-//		else 
+//		else
 //			d3_printf("parse state is %d p=%d\n", state, p);
 		switch(state) {
 		case P_START:
@@ -657,7 +663,7 @@ int fscan_params(FILE *fp, GcxPar root)
 			break;
 		case P_NLINE:
 			ret = getline (&line, &len, fp);
-			if (ret < 0) 
+			if (ret < 0)
 				break;
 			text = line;
 			token = next_token(&text, &start, &end);
@@ -668,7 +674,7 @@ int fscan_params(FILE *fp, GcxPar root)
 			case TOK_WORD: /* start of a relative path */
 				p = node_lookup(pp, start, end-start);
 //				d3_printf("tok_word %s\np=%d\n", start, p);
-				if (p == PAR_NULL) { 
+				if (p == PAR_NULL) {
 					*(end)=0;
 					err_printf("fscan_params: unknown name %s under node %d\n",
 						   start, PAR(pp)->parent);
@@ -706,7 +712,7 @@ int fscan_params(FILE *fp, GcxPar root)
 				break;
 			}
 			p = node_lookup(p, start, end-start);
-			if (p == PAR_NULL) { 
+			if (p == PAR_NULL) {
 				*(end)=0;
 				err_printf("fscan_params: unknown name %s \n",
 					   start);
@@ -730,7 +736,7 @@ int fscan_params(FILE *fp, GcxPar root)
 			}
 			if (token == TOK_WORD) {
 				p = node_lookup(pp, start, end-start);
-				if (p == PAR_NULL) { 
+				if (p == PAR_NULL) {
 					*(end)=0;
 					err_printf("fscan_params: unknown name %s under node %d\n",
 						   start, PAR(pp)->parent);
@@ -749,7 +755,7 @@ int fscan_params(FILE *fp, GcxPar root)
 				break;
 			}
 			break;
-		case P_LEAF: 
+		case P_LEAF:
 			token = next_token(&text, &start, &end);
 			if (token == TOK_DOT)
 				break;
@@ -799,7 +805,7 @@ int set_par_by_name(char *line)
 			case TOK_WORD: /* start of a relative path */
 				p = node_lookup(pp, start, end-start);
 //				d3_printf("tok_word %s\np=%d\n", start, p);
-				if (p == PAR_NULL) { 
+				if (p == PAR_NULL) {
 					*(end)=0;
 					err_printf("set_par_by_name: unknown name %s "
 						   "under node %d\n",
@@ -834,7 +840,7 @@ int set_par_by_name(char *line)
 				return -1;
 			}
 			p = node_lookup(p, start, end-start);
-			if (p == PAR_NULL) { 
+			if (p == PAR_NULL) {
 				*(end)=0;
 				err_printf("fscan_params: unknown name %s \n",
 					   start);
@@ -847,7 +853,7 @@ int set_par_by_name(char *line)
 				}
 			}
 			break;
-		case P_LEAF: 
+		case P_LEAF:
 			token = next_token(&text, &start, &end);
 			if (token == TOK_DOT)
 				break;
@@ -1008,14 +1014,14 @@ void doc_printf_par(FILE *of, GcxPar p, int level)
 	while(pp != PAR_NULL) {
 		if (PAR(pp)->child == PAR_NULL) {
 			doc_printf_par(of, pp, level+1);
-		}			
+		}
 		pp = PAR(pp)->next;
 	}
 	pp = PAR(p)->child;
 	while(pp != PAR_NULL) {
 		if (PAR(pp)->child != PAR_NULL) {
 			doc_printf_par(of, pp, level+1);
-		}			
+		}
 		pp = PAR(pp)->next;
 	}
 	if (PAR(p)->parent == PAR_NULL) { /* tail recursion for top-level pars */
