@@ -56,7 +56,7 @@ int open_plot(FILE **fp, char *initial)
 
 	if (!P_INT(FILE_PLOT_TO_FILE)) {
 		plfp = popen(P_STR(FILE_GNUPLOT), "w");
-		if (plfp != NULL && (int)plfp != -1) {
+		if (plfp != NULL && (long)plfp != -1) {
 			if (fp)
 				*fp = plfp;
 			return 1;
@@ -102,6 +102,7 @@ void plot_preamble(FILE *dfp)
 {
 	g_return_if_fail(dfp != NULL);
 	fprintf(dfp, "set key below\n");
+	fprintf(dfp, "set term x11\n");
 }
 
 /* create a plot of ofr residuals versus magnitude (as a gnuplot file) */
@@ -111,7 +112,7 @@ int ofrs_plot_residual_vs_mag(FILE *dfp, GList *ofrs, int weighted)
 	struct star_obs *sob;
 	struct o_frame *ofr = NULL;
 	int n = 0, i = 0;
-	int band;
+	long band;
 	double v, u;
 
 	g_return_val_if_fail(dfp != NULL, -1);
@@ -144,7 +145,7 @@ int ofrs_plot_residual_vs_mag(FILE *dfp, GList *ofrs, int weighted)
 	fprintf(dfp, "\n");
 
 	for (bl = bsl; bl != NULL; bl = g_list_next(bl)) {
-		band = (int) bl->data;
+		band = (long) bl->data;
 		osl = ofrs;
 		while (osl != NULL) {
 			ofr = O_FRAME(osl->data);
@@ -187,7 +188,7 @@ int ofrs_plot_zp_vs_time(FILE *dfp, GList *ofrs)
 	struct o_frame *ofr = NULL;
 	double mjdi = 0.0;
 	int n = 0, i = 0;
-	int band;
+	long band;
 	GList *asfl = NULL, *bnames = NULL;
 
 
@@ -240,7 +241,7 @@ int ofrs_plot_zp_vs_time(FILE *dfp, GList *ofrs)
 	fprintf(dfp, "\n");
 
 	for (bl = bsl; bl != NULL; bl = g_list_next(bl)) {
-		band = (int) bl->data;
+		band = (long) bl->data;
 		osl = ofrs;
 		while (osl != NULL) {
 			ofr = O_FRAME(osl->data);
@@ -259,7 +260,7 @@ int ofrs_plot_zp_vs_time(FILE *dfp, GList *ofrs)
 	}
 	if (asfl != NULL) 
 		for (bl = bsl; bl != NULL; bl = g_list_next(bl)) {
-			band = (int) bl->data;
+			band = (long) bl->data;
 			osl = asfl;
 			while (osl != NULL) {
 				ofr = O_FRAME(osl->data);
@@ -287,7 +288,7 @@ int ofrs_plot_zp_vs_am(FILE *dfp, GList *ofrs)
 	GList *osl, *bl, *bsl = NULL;
 	struct o_frame *ofr = NULL;
 	int n = 0, i = 0;
-	int band;
+	long band;
 	GList *asfl = NULL, *bnames = NULL;
 
 	g_return_val_if_fail(dfp != NULL, -1);
@@ -328,7 +329,7 @@ int ofrs_plot_zp_vs_am(FILE *dfp, GList *ofrs)
 	fprintf(dfp, "\n");
 
 	for (bl = bsl; bl != NULL; bl = g_list_next(bl)) {
-		band = (int) bl->data;
+		band = (long) bl->data;
 		osl = ofrs;
 		while (osl != NULL) {
 			ofr = O_FRAME(osl->data);
@@ -347,7 +348,7 @@ int ofrs_plot_zp_vs_am(FILE *dfp, GList *ofrs)
 	}
 	if (asfl != NULL) 
 		for (bl = bsl; bl != NULL; bl = g_list_next(bl)) {
-			band = (int) bl->data;
+			band = (long) bl->data;
 			osl = asfl;
 			while (osl != NULL) {
 				ofr = O_FRAME(osl->data);
