@@ -122,10 +122,10 @@ struct psf *psf_new(int w, int h)
 	psf = calloc(1, sizeof(struct psf));
 	g_assert(psf != NULL);
 
-	d = malloc(w * (h + 1) * sizeof(float *));
+	d = malloc(w * h * sizeof(float) + w * sizeof(float *));
 	g_assert(d != NULL);
 
-	f = (float *)d + w;
+	f = (float *)(d + w);
 	for (i = 0; i < w; i++) {
 		d[i] = f;
 		f += h;
@@ -200,7 +200,8 @@ void aperture_stats(struct ccd_frame *fr, struct psf *psf, double x, double y,
 	int nvals;
 	int ix, iy;
 	int rx, ry;
-	float *d;
+	int h_offset, w_offset;
+	float d;
 	rs->min = HUGE;
 	rs->max = -HUGE;
 	
@@ -240,26 +241,25 @@ void aperture_stats(struct ccd_frame *fr, struct psf *psf, double x, double y,
 	vals = malloc(psf->h * psf->w * sizeof(vals));
 	g_assert(vals != NULL);
 	
-	d = fr->dat;
-	d += rx - (psf->cx - xs) + fr->w * (ry - (psf->cy - ys));
+	w_offset = rx - psf->cx;
+	h_offset = ry - psf->cy;
 	for (iy = ys; iy < ye; iy++) {
 		for (ix = xs; ix < xe; ix++) {
+			d = get_pixel_luminence(fr, w_offset + ix, h_offset + iy);
 			if (psf->d[ix][iy] >= 0.5) {
-				vals[nvals++] = *d;
-				if (*d > rs->max) {
-					rs->max = *d;
+				vals[nvals++] = d;
+				if (d > rs->max) {
+					rs->max = d;
 					rs->max_x = rx + ix - psf->cx;
 					rs->max_y = ry + iy - psf->cy;
 				}
-				if (*d < rs->min)
-					rs->min = *d;
+				if (d < rs->min)
+					rs->min = d;
 			}
-			rs->sum += *d * psf->d[ix][iy];
-			rs->sumsq += sqr(*d) * psf->d[ix][iy];
+			rs->sum += d * psf->d[ix][iy];
+			rs->sumsq += sqr(d) * psf->d[ix][iy];
 			rs->all += psf->d[ix][iy];
-			d++;
 		}
-		d += fr->w - (xe - xs);
 	}
 	rs->median = fmedian(vals, nvals);
 	if (rs->all != 0.0) {
@@ -278,7 +278,7 @@ void aperture_multiply(struct ccd_frame *fr, struct psf *psf, double x, double y
 	int xs, xe, ys, ye;
 	int ix, iy;
 	int rx, ry;
-	float *d;
+	int w_offset, h_offset;
 	
 	xs = ys = 0;
 	xe = psf->w;
@@ -298,14 +298,12 @@ void aperture_multiply(struct ccd_frame *fr, struct psf *psf, double x, double y
 
 	d4_printf("xs:%d xe:%d ys:%d ys:%d\n", xs, xe, ys, ye);
 
-	d = fr->dat;
-	d += rx - (psf->cx - xs) + fr->w * (ry - (psf->cy - ys));
+	w_offset = rx - psf->cx;
+	h_offset = ry - psf->cy;
 	for (iy = ys; iy < ye; iy++) {
 		for (ix = xs; ix < xe; ix++) {
-			psf->d[ix][iy] *= *d;
-			d++;
+			psf->d[ix][iy] *= get_pixel_luminence(fr, w_offset + ix, h_offset + iy);
 		}
-		d += fr->w - (xe - xs);
 	}
 }
 
@@ -315,7 +313,8 @@ static void extract_patch(struct ccd_frame *fr, struct psf *psf, double x, doubl
 	int xs, xe, ys, ye;
 	int ix, iy;
 	int rx, ry;
-	float *d;
+	float d;
+	int w_offset, h_offset;
 	
 	xs = ys = 0;
 	xe = psf->w;
@@ -337,14 +336,13 @@ static void extract_patch(struct ccd_frame *fr, struct psf *psf, double x, doubl
 
 	d4_printf("xs:%d xe:%d ys:%d ys:%d\n", xs, xe, ys, ye);
 
-	d = fr->dat;
-	d += rx - (psf->cx - xs) + fr->w * (ry - (psf->cy - ys));
+	w_offset = rx - psf->cx;
+	h_offset = ry - psf->cy;
 	for (iy = ys; iy < ye; iy++) {
 		for (ix = xs; ix < xe; ix++) {
-			psf->d[ix][iy] = *d;
+			psf->d[ix][iy] = get_pixel_luminence(fr, w_offset + ix, h_offset + iy);
 			d++;
 		}
-		d += fr->w - (xe - xs);
 	}
 }
 
@@ -400,7 +398,7 @@ int dodge_high(struct ccd_frame *fr, struct psf *psf, double x, double y,
 {
 	int xs, xe, ys, ye;
 	int ix, iy, n = 0;
-	float *d;
+	int w_offset, h_offset;
 	int rx, ry;
 	
 	xs = ys = 0;
@@ -420,19 +418,17 @@ int dodge_high(struct ccd_frame *fr, struct psf *psf, double x, double y,
 	if (ry + psf->h - psf->cy > fr->h)
 		ye = fr->h + psf->cy - ry;
 
-	d = fr->dat;
-	d += rx - (psf->cx - xs) + fr->w * (ry - (psf->cy - ys));
+	w_offset = rx - psf->cx;
+	h_offset = ry - psf->cy;
 	for (iy = ys; iy < ye; iy++) {
 		for (ix = xs; ix < xe; ix++) {
-			if (*d > ht) {
+			if (get_pixel_luminence(fr, w_offset + ix, h_offset + iy) > ht) {
 				if (psf->d[ix][iy] > 0)
 					n++;
 				psf->d[ix][iy] = 0;
 				pixel_grow(psf, ix, iy, grow);
 			}
-			d++;
 		}
-		d += fr->w - (xe - xs);
 	}
 	return n;
 }
@@ -491,7 +487,7 @@ int radial_profile(struct ccd_frame *fr, double x, double y, double r,
 	struct ap_params apdef;
 	struct star s;
 	int ret, ns = 0;
-	float *d, *de;
+	float d;
 	int ri, i;
 	int ix, iy, xs, ys, xe, ye;
 	double max = 1.0;
@@ -517,30 +513,21 @@ int radial_profile(struct ccd_frame *fr, double x, double y, double r,
 	ys = floor(y - r);
 	xe = xs + ri;
 	ye = ys + ri;
-	d = fr->dat;
-	d += fr->w * ys + xs;
-	de = fr->dat;
-	de += fr->w * fr->h - 1;
 	g_return_val_if_fail(ri < fr->w, 0);
-	for (iy = 0; iy < ri; iy++) {
-		for (ix = 0; ix < ri; ix++) {
+	for (iy = 0; iy < ri && ys + iy < fr->w; iy++) {
+		for (ix = 0; ix < ri && xs + ix < fr->h; ix++) {
 			double dist;
 			dist = sqrt(sqr(ix + xs - x) + sqr(iy + ys - y));
 			if (ns < n && dist <= r) {
-//				d4_printf("dist %8.1f v:%.1f\n", dist, *d);
+				d = get_pixel_luminence(fr, xs + ix, ys + iy);
+//				d4_printf("dist %8.1f v:%.1f\n", dist, d);
 				rpp[ns].r = dist;
-				rpp[ns].v = *d;
-				if (dist < r/2 && max < *d)
-					max = *d;
+				rpp[ns].v = d;
+				if (dist < r/2 && max < d)
+					max = d;
 				ns++;
 			}
-			d++;
-			if (d >= de)
-				break;
 		}
-		d += fr->w - ri;
-		if (d >= de)
-			break;
 	}
 	for (i = 0; i < ns; i++) {
 		if ((max - s.aph.sky) / max > 0.001)
