@@ -87,6 +87,7 @@ static struct cat_star * parse_cat_line_ucac2_main(char *line)
 	double u, v, w, x;
 	double uc, m, j, k;
 	char *endp;
+	int ret = 0;
 	
 	nc = detabify(line, cols, 32);
 
@@ -126,14 +127,16 @@ static struct cat_star * parse_cat_line_ucac2_main(char *line)
 		j = strtod(line+cols[11], &endp);
 		k = strtod(line+cols[12], &endp);
 		if (m > 0.0 && j > 0.0 && k > 0.0) {
-			asprintf(&cats->comments, "p=u 2mass=%.0f", m);
+			if (-1 == asprintf(&cats->comments, "p=u 2mass=%.0f", m))
+				cats->comments = NULL;
 			if (uc == 0)
-				asprintf(&cats->smags, "j=%.3f k=%.3f", j, k);
+				ret = asprintf(&cats->smags, "j=%.3f k=%.3f", j, k);
 			else 
-				asprintf(&cats->smags, "uc=%.3f j=%.3f k=%.3f", uc, j, k);
+				ret = asprintf(&cats->smags, "uc=%.3f j=%.3f k=%.3f", uc, j, k);
 		}
 	}
-
+	if (ret == -1)
+		cats->smags = NULL;
 	w = strtod(line+cols[8], &endp);
 	if (line+cols[8] == endp)
 		return cats;
@@ -165,7 +168,7 @@ static struct cat_star * parse_cat_line_ucac2_bss(char *line)
 	double u, v, w, x;
 	double uc, m, j, bv, h, t1, t2;
 	char *endp;
-	char buf[128];
+	char buf[256];
 	int n;
 
 //	d3_printf("%s", line);
@@ -268,7 +271,7 @@ static struct cat_star * parse_cat_line_gsc2(char *line)
 	double u, v;
 	double fr, fre, bj, bje, class;
 	char *endp;
-	char buf[128];
+	char buf[256];
 	int n;
 	
 	nc = detabify(line, cols, 32);
@@ -304,7 +307,8 @@ static struct cat_star * parse_cat_line_gsc2(char *line)
 		cats->mag = bj;
 
 	cats->perr = BIG_ERR;
-	asprintf(&cats->comments, "p=g gsc2_class=%.0f", class);
+	if (-1 == asprintf(&cats->comments, "p=g gsc2_class=%.0f", class))
+		cats->comments = NULL;
 	n = 0;
 	if (fr > 0) {
 		if (fre > 0)
@@ -332,6 +336,7 @@ static struct cat_star * parse_cat_line_gsc_act(char *line)
 	double u, v;
 	double p, pe, ep, class;
 	char *endp;
+	int ret = 0;
 	
 	nc = detabify(line, cols, 32);
 
@@ -366,13 +371,16 @@ static struct cat_star * parse_cat_line_gsc_act(char *line)
 
 	cats->mag = p;
 
-	asprintf(&cats->comments, "p=G gsc_class=%.0f", class);
+	if (-1 == asprintf(&cats->comments, "p=G gsc_class=%.0f", class))
+		cats->comments = NULL;
 	if (p > 0) {
 		if (pe > 0)
-			asprintf(&cats->smags, "p=%.3f/%.3f ", p, pe);
+			ret = asprintf(&cats->smags, "p=%.3f/%.3f ", p, pe);
 		else
-			asprintf(&cats->smags, "p=%.3f ", p);
+			ret = asprintf(&cats->smags, "p=%.3f ", p);
 	}
+	if (ret == -1)
+		cats->smags = NULL;
 
 	cats->flags |= CAT_ASTROMET;
 	ep = strtod(line+cols[8], &endp);
@@ -591,6 +599,11 @@ GList *query_catalog(char *catalog, double ra, double dec, double w, double h,
 
 	d3_printf("vizquery: %s", cmd);
 
+	/* cmatei: FIXME
+
+	   vq != NULL if vizquery cannot be run, because the shell
+	   itself DOES run. We then go in a rather infinite loop
+	   below, due to suboptimal error handling :-) */
 	vq = popen(cmd, "r");
 	if (vq == NULL) {
 		err_printf("cannot run vizquery (%s)\n", strerror(errno));
@@ -625,7 +638,7 @@ GList *query_catalog(char *catalog, double ra, double dec, double w, double h,
 		if (ret <= 0)
 			continue;
 		n++;
-//		d4_printf("%s", line);
+		d4_printf("%s", line);
 		switch(qstat) {
 		case 0:		/* look for a table name */
 			if (!strncasecmp(line, "#Title", 6)) {
@@ -764,11 +777,12 @@ static int logw_print(char *msg, void *data)
 
 	text = gtk_object_get_data(GTK_OBJECT(logw), "query_log_text");
 	g_return_val_if_fail(text != NULL, 0);
-	gtk_text_set_line_wrap(GTK_TEXT (text), 1);
-	gtk_text_insert (GTK_TEXT (text), text->style->font, &text->style->black, 
-			 NULL, msg, -1);
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW (text), GTK_WRAP_CHAR);
+	
+	gtk_text_buffer_insert_at_cursor(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text)),
+					 msg, -1);
 
-	while(gtk_events_pending())
+	while (gtk_events_pending())
 		gtk_main_iteration();
 
 	stopb = gtk_object_get_data(GTK_OBJECT(logw), "query_stop_toggle");
