@@ -47,7 +47,6 @@
 #include "params.h"
 #include "interface.h"
 #include "wcs.h"
-#include "camera.h"
 #include "cameragui.h"
 #include "filegui.h"
 
@@ -494,7 +493,8 @@ int read_gsc2(struct cat_star *csl[], FILE *fp, int n, double *cra, double *cdec
 			update_band_by_name(&cats->smags, "n(gsc2)", nmag, nmagerr);
 		}
 		cats->flags = CAT_STAR_TYPE_SREF | CAT_ASTROMET;
-		asprintf(&cats->comments, "p=g ");
+		if (-1 == asprintf(&cats->comments, "p=g "))
+			cats->comments = NULL;
 		cst[sn]=cats;
 		sn ++;
 	}
@@ -566,11 +566,13 @@ int read_landolt_table(struct cat_star *csl[], FILE *fp, int n, double *cra, dou
 		cats->dec = dec;
 		cats->mag = vmag;
 		cats->equinox = 2000.0;
-		asprintf(&cats->smags, "v=%.3f/0.005 b=%.3f/0.005 "
-			 "u=%.3f/0.005 r=%.3f/0.005 i=%.3f/0.005",
-			 vmag, bmag, umag, rmag, imag);
+		if (-1 == asprintf(&cats->smags, "v=%.3f/0.005 b=%.3f/0.005 "
+		                   "u=%.3f/0.005 r=%.3f/0.005 i=%.3f/0.005",
+		                   vmag, bmag, umag, rmag, imag))
+			cats->smags = NULL;
 		cats->flags = CAT_STAR_TYPE_APSTD | CAT_ASTROMET;
-		asprintf(&cats->comments, "%s", nam);
+		if (-1 == asprintf(&cats->comments, "%s", nam))
+			cats->comments = NULL;
 		cst[sn]=cats;
 		sn ++;
 	}
@@ -661,14 +663,16 @@ int read_varlist_table(struct cat_star *csl[], FILE *fp, int n, double *cra, dou
 		cats->equinox = 2000.0;
 		cats->flags = CAT_STAR_TYPE_CAT;
 		if (field[4] != 0)
-			asprintf(&cats->comments, "p=%s i=%s %s", 
+			ret = asprintf(&cats->comments, "p=%s i=%s %s", 
 				 line+field[2], line+field[3], line+field[4]);
 		else if (field[3] != 0)
-			asprintf(&cats->comments, "p=%s i=%s", 
+			ret = asprintf(&cats->comments, "p=%s i=%s", 
 				 line+field[2], line+field[3]);
 		else if (field[2] != 0)
-			asprintf(&cats->comments, "p=%s", 
+			ret = asprintf(&cats->comments, "p=%s", 
 				 line+field[2]);
+		if (ret == -1)
+			cats->comments = NULL;
 		cst[sn]=cats;
 		sn ++;
 	}
@@ -889,8 +893,9 @@ int read_sumner_table(struct cat_star *csl[], FILE *fp, int n, double *cra, doub
 			continue;
 		cats = cat_star_new();
 		snprintf(cats->name, CAT_STAR_NAME_SZ, "%s_%.0f", id, c1);
-		asprintf(&cats->smags, "v=%.3f/%.3f b=%.3f/%.3f",
-				       c11, c12, c11+c13, sqrt(sqr(c12)+ sqr(c14)));
+		if (-1 == asprintf(&cats->smags, "v=%.3f/%.3f b=%.3f/%.3f",
+		                   c11, c12, c11+c13, sqrt(sqr(c12)+ sqr(c14))))
+			cats->smags = NULL;
 		cats->ra = 15 * c2 + 15 * c3 / 60 + 15 * c4 / 3600;
 		if (c5 < 0) 
 			cats->dec = -(c6 / 60 + c7 / 3600 - c5);
@@ -924,8 +929,9 @@ int read_sumner_table(struct cat_star *csl[], FILE *fp, int n, double *cra, doub
 			continue;
 		cats = cat_star_new();
 		snprintf(cats->name, CAT_STAR_NAME_SZ, "%s_%d", id, sn);
-		asprintf(&cats->smags, "v=%.3f/%.3f b=%.3f/%.3f",
-				       c11, c12, c11+c13, sqrt(sqr(c12)+ sqr(c14)));
+		if (-1 == asprintf(&cats->smags, "v=%.3f/%.3f b=%.3f/%.3f",
+		                   c11, c12, c11+c13, sqrt(sqr(c12)+ sqr(c14))))
+			cats->smags = NULL;
 		cats->ra = 15 * c2 + 15 * c3 / 60 + 15 * c4 / 3600;
 		if (c5 < 0) 
 			cats->dec = -(c6 / 60 + c7 / 3600 - c5);
@@ -982,6 +988,7 @@ static int read_gcvs_table(struct cat_star *csl[], FILE *fp, int n, double *cra,
 	char type[24];
 	char id[64];
 	int amp;
+	int ret;
 
 	if (csl == NULL)
 		return -1;
@@ -1071,18 +1078,20 @@ static int read_gcvs_table(struct cat_star *csl[], FILE *fp, int n, double *cra,
 		cats->equinox = 2000.0;
 		cats->flags = CAT_STAR_TYPE_CAT | CAT_VARIABLE;
 		if (spec[0] != 0 && !amp) {
-			asprintf(&cats->comments, "p=V t=%s s=%s m(%s)=%.2f/%.2f",
+			ret = asprintf(&cats->comments, "p=V t=%s s=%s m(%s)=%.2f/%.2f",
 				 type, spec, phot, mmax, mmin);
 		} else if (!amp) {
-			asprintf(&cats->comments, "p=V t=%s m(%s)=%.2f/%.2f",
+			ret = asprintf(&cats->comments, "p=V t=%s m(%s)=%.2f/%.2f",
 				 type, phot, mmax, mmin);
 		} else if (spec[0] != 0 && amp) {
-			asprintf(&cats->comments, "p=V t=%s s=%s m(%s)=%.2f(%.3f)",
+			ret = asprintf(&cats->comments, "p=V t=%s s=%s m(%s)=%.2f(%.3f)",
 				 type, spec, phot, mmax, mmin);
 		} else {
-			asprintf(&cats->comments, "p=V t=%s m(%s)=%.2f(%.3f)",
+			ret = asprintf(&cats->comments, "p=V t=%s m(%s)=%.2f(%.3f)",
 				 type, phot, mmax, mmin);
 		}
+		if (ret == -1)
+			cats->comments = NULL;
 		
 		cst[sn]=cats;
 		sn ++;
