@@ -1,29 +1,29 @@
 /*******************************************************************************
   Copyright(c) 2000 - 2003 Radu Corlan. All rights reserved.
-  
-  This program is free software; you can redistribute it and/or modify it 
-  under the terms of the GNU General Public License as published by the Free 
-  Software Foundation; either version 2 of the License, or (at your option) 
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the Free
+  Software Foundation; either version 2 of the License, or (at your option)
   any later version.
-  
-  This program is distributed in the hope that it will be useful, but WITHOUT 
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
-  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for 
+
+  This program is distributed in the hope that it will be useful, but WITHOUT
+  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
   more details.
-  
+
   You should have received a copy of the GNU General Public License along with
-  this program; if not, write to the Free Software Foundation, Inc., 59 
+  this program; if not, write to the Free Software Foundation, Inc., 59
   Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-  
+
   The full GNU General Public License is included in this distribution in the
   file called LICENSE.
-  
+
   Contact Information: radu@corlan.net
 *******************************************************************************/
 
 /*  reduce.c: routines for ccd reduction and image list handling */
-/*  $Revision: 1.22 $ */
-/*  $Date: 2006/07/17 13:30:53 $ */
+/*  $Revision: 1.31 $ */
+/*  $Date: 2009/08/02 20:46:19 $ */
 
 #define _GNU_SOURCE
 
@@ -52,6 +52,7 @@
 #include "multiband.h"
 #include "recipy.h"
 #include "filegui.h"
+#include "demosaic.h"
 
 int progress_print(char *msg, void *data)
 {
@@ -59,7 +60,7 @@ int progress_print(char *msg, void *data)
 	return 0;
 }
 
-/* pixel combination methods 
+/* pixel combination methods
  * they all take an array of pointers to float, and
  * the "sigmas" and "iter" parameters, and return a
  * single float value. Some methods ignore sigmas and/or iter.
@@ -173,17 +174,17 @@ static int is_zip_name(char *fn)
 	len = strlen(fn);
 	if (len < 4)
 		return 0;
-	if (strcasecmp(fn + len - 3, ".gz") == 0) 
+	if (strcasecmp(fn + len - 3, ".gz") == 0)
 		return 1;
-	if (strcasecmp(fn + len - 2, ".z") == 0) 
+	if (strcasecmp(fn + len - 2, ".z") == 0)
 		return 1;
-	if (strcasecmp(fn + len - 4, ".zip") == 0) 
+	if (strcasecmp(fn + len - 4, ".zip") == 0)
 		return 1;
 	return 0;
 }
 
 /* overwrite the the original file */
-static int save_image_file_inplace(struct image_file *imf, 
+static int save_image_file_inplace(struct image_file *imf,
 				   int (* progress)(char *msg, void *data), void *data)
 {
 	char fn[16384];
@@ -206,7 +207,7 @@ static int save_image_file_inplace(struct image_file *imf,
 			if (fn[i] == '.') {
 				fn[i] = 0;
 				break;
-			}		
+			}
 		return write_gz_fits_frame(imf->fr, fn, P_STR(FILE_COMPRESS));
 	} else {
 		return write_fits_frame(imf->fr, fn);
@@ -241,7 +242,7 @@ static int save_image_file_to_dir(struct image_file *imf, char *outf,
 			if (fn[i] == '.') {
 				fn[i] = 0;
 				break;
-			}		
+			}
 		return write_gz_fits_frame(imf->fr, fn, P_STR(FILE_COMPRESS));
 	} else {
 		return write_fits_frame(imf->fr, fn);
@@ -250,7 +251,7 @@ static int save_image_file_to_dir(struct image_file *imf, char *outf,
 
 /* outf is a "stud" to a file name; the function adds sequence numbers to it; files
  * are not overwritten, the sequence is just incremented until a new slot is
- * found; sequeces are searched from seq up; seq is updated 
+ * found; sequeces are searched from seq up; seq is updated
  * if seq is NULL, no sequence number is appended, and outf becomes the filename
  * calls progress with a short progress message from time to time. If progress returns
  * a non-zero value, abort remaining operations and return an error (non-zero) */
@@ -269,7 +270,7 @@ static int save_image_file_to_stud(struct image_file *imf, char *outf, int *seq,
 	if (imf->flags & IMG_SKIP)
 		return 0;
 
-	if (seq != NULL) { 
+	if (seq != NULL) {
 		strncpy(fn, outf, 16383);
 		fn[16383] = 0;
 		check_seq_number(fn, seq);
@@ -288,7 +289,7 @@ static int save_image_file_to_stud(struct image_file *imf, char *outf, int *seq,
 			if (fn[i] == '.') {
 				fn[i] = 0;
 				break;
-			}		
+			}
 		return write_gz_fits_frame(imf->fr, fn, P_STR(FILE_COMPRESS));
 	} else {
 		return write_fits_frame(imf->fr, fn);
@@ -297,7 +298,7 @@ static int save_image_file_to_stud(struct image_file *imf, char *outf, int *seq,
 
 
 /* save the (processed) frames from the image_file_list, using the filename logic
- * described in batch_reduce; 
+ * described in batch_reduce;
  * call progress with a short progress message from time to time. If progress returns
  * a non-zero value, abort remaining operations and return an error (non-zero) */
 
@@ -327,12 +328,12 @@ int save_image_file(struct image_file *imf, char *outf, int inplace, int *seq,
 	return 0;
 }
 
-/* call point from main; reduce the frames acording to ccdr. 
- * Print progress messages at level 1. If the inplace flag in ccdr->ops is true, 
+/* call point from main; reduce the frames acording to ccdr.
+ * Print progress messages at level 1. If the inplace flag in ccdr->ops is true,
  * the source files are overwritten, else new files will be created according
- * to outf. if outf is a dir name, files will be saved there. If not, 
+ * to outf. if outf is a dir name, files will be saved there. If not,
  * it's used as the beginning of a filename.
- * when stacking is requested, intermediate processing steps 
+ * when stacking is requested, intermediate processing steps
  * are never saved */
 
 int batch_reduce_frames(struct image_file_list *imfl, struct ccd_reduce *ccdr,
@@ -361,8 +362,8 @@ int batch_reduce_frames(struct image_file_list *imfl, struct ccd_reduce *ccdr,
 			if (ccdr->ops & IMG_OP_INPLACE) {
 				save_image_file(imf, outf, 1, NULL, progress_print, NULL);
 			} else {
-				save_image_file(imf, outf, 0, 
-						(nframes == 1 ? NULL : &seq), 
+				save_image_file(imf, outf, 0,
+						(nframes == 1 ? NULL : &seq),
 						progress_print, NULL);
 			}
 			imf->flags &= ~IMG_SKIP;
@@ -385,8 +386,8 @@ int batch_reduce_frames(struct image_file_list *imfl, struct ccd_reduce *ccdr,
 }
 
 
-/* call point from main; reduce the frames acording to ccdr. 
- * Print progress messages at level 1. 
+/* call point from main; reduce the frames acording to ccdr.
+ * Print progress messages at level 1.
  * returns the result of the stack if a stacking op was specified, or the
  * first frame in the list
  * files are saved if the inplace flag is set */
@@ -414,7 +415,7 @@ struct ccd_frame *reduce_frames_load(struct image_file_list *imfl, struct ccd_re
 				continue;
 			if (ccdr->ops & IMG_OP_INPLACE) {
 				save_image_file(imf, NULL, 1, NULL, progress_print, NULL);
-			} 
+			}
 			if (fr == NULL) {
 				fr = imf->fr;
 				get_frame(fr);
@@ -438,7 +439,7 @@ struct ccd_frame *reduce_frames_load(struct image_file_list *imfl, struct ccd_re
 
 
 /* reduce the frame according to ccdr, up to the stacking step.
- * return 0 for succes or a positive error code 
+ * return 0 for succes or a positive error code
  * call progress with a short progress message from time to time. If progress returns
  * a non-zero value, abort remaining operations and return an error (non-zero) */
 
@@ -492,13 +493,30 @@ int reduce_frame(struct image_file *imf, struct ccd_reduce *ccdr,
 		if (load_image_file(ccdr->flat))
 			return 2;
 	}
+	if (ccdr->ops & IMG_OP_BADPIX) {
+		if (ccdr->bad_pix_map == NULL) {
+			err_printf("no bad pixel file\n");
+			return 1;
+		}
+		if (!ccdr->bad_pix_map->pix) {
+			snprintf(msg, 255, "bad pixels: %s\n", ccdr->bad_pix_map->filename);
+			if (progress) {
+				if ((*progress)(msg, data))
+					return -1;
+			}
+		}
+
+		if (load_bad_pix(ccdr->bad_pix_map))
+			return 2;
+	}
+
 	if (ccdr->ops & IMG_OP_ALIGN) {
 		if (ccdr->alignref == NULL) {
 			err_printf("no alignment reference file\n");
 			return 1;
 		}
 		if (!(ccdr->ops & CCDR_ALIGN_STARS)) {
-			snprintf(msg, 255, "alignment reference frame: %s\n", 
+			snprintf(msg, 255, "alignment reference frame: %s\n",
 				 ccdr->alignref->filename);
 			if (progress) {
 				if ((*progress)(msg, data))
@@ -519,6 +537,7 @@ int reduce_frame(struct image_file *imf, struct ccd_reduce *ccdr,
 		imf->flags |= IMG_SKIP;
 		return -1;
 	}
+
 	if (progress) {
 		if ((*progress)(" loaded", data))
 			return -1;
@@ -529,7 +548,7 @@ int reduce_frame(struct image_file *imf, struct ccd_reduce *ccdr,
 
 
 /* reduce the files in list according to ccdr, up to the stacking step.
- * return 0 for succes or a positive error code 
+ * return 0 for succes or a positive error code
  * call progress with a short progress message from time to time. If progress returns
  * a non-zero value, abort remaining operations and return an error (non-zero) */
 
@@ -596,6 +615,7 @@ int reduce_frames(struct image_file_list *imfl, struct ccd_reduce *ccdr,
 			imf->flags |= IMG_SKIP;
 			continue;
 		}
+
 		if (progress) {
 			if ((*progress)(" loaded", data))
 				return -1;
@@ -609,7 +629,7 @@ int reduce_frames(struct image_file_list *imfl, struct ccd_reduce *ccdr,
  * call progress with a short progress message from time to time. If progress returns
  * a non-zero value, abort remaining operations and return an error (non-zero) */
 
-int ccd_reduce_imf(struct image_file *imf, struct ccd_reduce *ccdr, 
+int ccd_reduce_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 		   int (* progress)(char *msg, void *data), void *data)
 {
 	char lb[81];
@@ -623,7 +643,12 @@ int ccd_reduce_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 				return -1;
 		}
 		if (!(imf->flags & IMG_OP_BIAS)) {
-			sub_frames(imf->fr, ccdr->bias->fr);
+			if (sub_frames(imf->fr, ccdr->bias->fr)) {
+				if (progress) {
+					(*progress)(" (FAILED)", data);
+				}
+				return -1;
+			}
 			fits_add_history(imf->fr, "'BIAS FRAME SUBTRACTED'");
 		} else {
 			if (progress) {
@@ -641,8 +666,13 @@ int ccd_reduce_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 				return -1;
 		}
 		if (!(imf->flags & IMG_OP_DARK)) {
-			sub_frames(imf->fr, ccdr->dark->fr);
-			fits_add_history(imf->fr, "'BIAS FRAME SUBTRACTED'");
+			if (sub_frames(imf->fr, ccdr->dark->fr)) {
+				if (progress) {
+					(*progress)(" (FAILED)", data);
+				}
+				return -1;
+			}
+			fits_add_history(imf->fr, "'DARK FRAME SUBTRACTED'");
 		} else {
 			if (progress) {
 				if ((*progress)(" (already_done)", data))
@@ -659,7 +689,12 @@ int ccd_reduce_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 				return -1;
 		}
 		if (!(imf->flags & IMG_OP_FLAT)) {
-			flat_frame(imf->fr, ccdr->flat->fr);
+			if (flat_frame(imf->fr, ccdr->flat->fr)) {
+				if (progress) {
+					(*progress)(" (FAILED)", data);
+				}
+				return -1;
+			}
 			fits_add_history(imf->fr, "'FLAT FIELDED'");
 		} else {
 			if (progress) {
@@ -670,6 +705,30 @@ int ccd_reduce_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 		imf->flags |= IMG_OP_FLAT | IMG_DIRTY;
 	}
 
+	if (ccdr->ops & IMG_OP_BADPIX) {
+		g_return_val_if_fail(ccdr->bad_pix_map != NULL, -1);
+		if (progress) {
+			if ((*progress)(" badpix", data))
+				return -1;
+		}
+		if (!(imf->flags & IMG_OP_BADPIX)) {
+			if (fix_bad_pixels(imf->fr, ccdr->bad_pix_map)) {
+				if (progress) {
+					(*progress)(" (FAILED)", data);
+				}
+				return -1;
+			}
+			fits_add_history(imf->fr, "'PIXEL DEFECTS REMOVED'");
+		} else {
+			if (progress) {
+				if ((*progress)(" (already_done)", data))
+					return -1;
+			}
+		}
+		imf->flags |= IMG_OP_BADPIX | IMG_DIRTY;
+
+	}
+
 	if (ccdr->ops & (IMG_OP_MUL | IMG_OP_ADD)) {
 		double m = 1.0, a = 0.0;
 		if (progress) {
@@ -714,8 +773,8 @@ int ccd_reduce_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 					return -1;
 			}
 		} else {
-			if ((ccdr->ops & IMG_OP_BG_ALIGN_MUL) && 
-			    imf->fr->stats.median < P_DBL(MIN_BG_SIGMAS) * 
+			if ((ccdr->ops & IMG_OP_BG_ALIGN_MUL) &&
+			    imf->fr->stats.median < P_DBL(MIN_BG_SIGMAS) *
 				imf->fr->stats.csigma) {
 				imf->flags |= IMG_SKIP;
 				if (progress) {
@@ -726,13 +785,13 @@ int ccd_reduce_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 					ccdr->bg = imf->fr->stats.median;
 					ccdr->ops |= CCDR_BG_VAL_SET;
 				} else if (ccdr->ops & IMG_OP_BG_ALIGN_MUL) {
-					scale_shift_frame(imf->fr, 
+					scale_shift_frame(imf->fr,
 							  ccdr->bg / imf->fr->stats.median, 0);
 				} else if (ccdr->ops & IMG_OP_BG_ALIGN_ADD) {
-					scale_shift_frame(imf->fr, 
+					scale_shift_frame(imf->fr,
 							  1.0, ccdr->bg - imf->fr->stats.median);
 				}
-				imf->flags |= (ccdr->ops & (IMG_OP_BG_ALIGN_MUL 
+				imf->flags |= (ccdr->ops & (IMG_OP_BG_ALIGN_MUL
 							    | IMG_OP_BG_ALIGN_ADD)) | IMG_DIRTY;
 			}
 		}
@@ -740,7 +799,26 @@ int ccd_reduce_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 
 	noise_to_fits_header(imf->fr, &(imf->fr->exp));
 
+	if (ccdr->ops & IMG_OP_DEMOSAIC) {
+		if (progress) {
+			if ((*progress)(" demosaic", data))
+				return -1;
+		}
+
+		if (!(imf->flags & IMG_OP_DEMOSAIC)) {
+			bayer_interpolate(imf->fr);
+			fits_add_history(imf->fr, "'DEMOSAIC'");
+			imf->flags |= IMG_OP_DEMOSAIC | IMG_DIRTY;
+		} else {
+			if (progress) {
+				if ((*progress)(" (already done)", data))
+					return -1;
+			}
+		}
+	}
+
 	if (ccdr->ops & (IMG_OP_BLUR)) {
+		remove_bayer_info(imf->fr);
 		if (progress) {
 			if ((*progress)(" blur", data))
 				return -1;
@@ -764,6 +842,8 @@ int ccd_reduce_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 
 	if (ccdr->ops & IMG_OP_ALIGN) {
 //		g_return_val_if_fail(ccdr->align_stars != NULL, -1);
+		// after alignment, it is no longer possible to use the raw frame
+		remove_bayer_info(imf->fr);
 		if (progress) {
 			if ((*progress)(" align", data))
 				return -1;
@@ -808,24 +888,28 @@ int ccd_reduce_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 	return 0;
 }
 
-/* the real work of avg-stacking frames 
+/* the real work of avg-stacking frames
  * return -1 for errors */
 static int do_stack_avg(struct image_file_list *imfl, struct ccd_frame *ofr,
 			int (* progress)(char *msg, void *data), void *data)
 {
 	int w, h, i, y, x, n;
 	float *dp[COMB_MAX];
+	struct ccd_frame *frames[COMB_MAX];
 	float *odp;
 	int rs[COMB_MAX];
 	GList *gl;
 	struct image_file *imf;
 	char lb[81];
 
+	int plane_iter = 0;
+
 	w = ofr->w;
 	h = ofr->h;
 
 	gl = imfl->imlist;
 	i=0;
+
 	while (gl != NULL) {
 		imf = gl->data;
 		gl = g_list_next(gl);
@@ -835,7 +919,7 @@ static int do_stack_avg(struct image_file_list *imfl, struct ccd_frame *ofr,
 			err_printf("bad frame size\n");
 			continue;
 		}
-		dp[i] = imf->fr->dat;
+		frames[i] = imf->fr;
 		rs[i] = imf->fr->w - w;
 		i++;
 		if (i >= COMB_MAX)
@@ -848,33 +932,41 @@ static int do_stack_avg(struct image_file_list *imfl, struct ccd_frame *ofr,
 	}
 	snprintf(lb, 80, "'AVERAGE STACK %d FRAMES'", i);
 	fits_add_history(ofr, lb);
-	odp = ofr->dat;
-	for (y = 0; y < h; y++) {
-		for (x = 0; x < w; x++) {
-			*odp = pix_average(dp, n, 0, 1);
-			for (i = 0; i < n; i++) 
-				dp[i]++;
-			odp++;
+
+	while ((plane_iter = color_plane_iter(frames[0], plane_iter))) {
+		for (i = 0; i <= n; i++) {
+			dp[i] = get_color_plane(frames[i], plane_iter);
 		}
-		for (i = 0; i < n; i++) {
-			dp[i] += rs[i];
+		odp = get_color_plane(ofr, plane_iter);
+		for (y = 0; y < h; y++) {
+			for (x = 0; x < w; x++) {
+				*odp = pix_average(dp, n, 0, 1);
+				for (i = 0; i < n; i++)
+					dp[i]++;
+				odp++;
+			}
+			for (i = 0; i < n; i++) {
+				dp[i] += rs[i];
+			}
 		}
 	}
 	return 0;
 }
 
-/* the real work of median-stacking frames 
+/* the real work of median-stacking frames
  * return -1 for errors */
 static int do_stack_median(struct image_file_list *imfl, struct ccd_frame *ofr,
 			   int (* progress)(char *msg, void *data), void *data)
 {
 	int w, h, i, y, x, n;
 	float *dp[COMB_MAX];
+	struct ccd_frame *frames[COMB_MAX];
 	float *odp;
 	int rs[COMB_MAX];
 	GList *gl;
 	struct image_file *imf;
 	char lb[80];
+	int plane_iter = 0;
 
 	w = ofr->w;
 	h = ofr->h;
@@ -890,7 +982,7 @@ static int do_stack_median(struct image_file_list *imfl, struct ccd_frame *ofr,
 			err_printf("bad frame size\n");
 			continue;
 		}
-		dp[i] = imf->fr->dat;
+		frames[i] = imf->fr;
 		rs[i] = imf->fr->w - w;
 		i++;
 		if (i >= COMB_MAX)
@@ -903,33 +995,40 @@ static int do_stack_median(struct image_file_list *imfl, struct ccd_frame *ofr,
 	}
 	snprintf(lb, 80, "'MEDIAN STACK of %d FRAMES'", i);
 	fits_add_history(ofr, lb);
-	odp = ofr->dat;
-	for (y = 0; y < h; y++) {
-		for (x = 0; x < w; x++) {
-			*odp = pix_median(dp, n, 0, 1);
-			for (i = 0; i < n; i++) 
-				dp[i]++;
-			odp++;
+	while ((plane_iter = color_plane_iter(frames[0], plane_iter))) {
+		for (i = 0; i <= n; i++) {
+			dp[i] = get_color_plane(frames[i], plane_iter);
 		}
-		for (i = 0; i < n; i++) {
-			dp[i] += rs[i];
+		odp = get_color_plane(ofr, plane_iter);
+		for (y = 0; y < h; y++) {
+			for (x = 0; x < w; x++) {
+				*odp = pix_median(dp, n, 0, 1);
+				for (i = 0; i < n; i++)
+					dp[i]++;
+				odp++;
+			}
+			for (i = 0; i < n; i++) {
+				dp[i] += rs[i];
+			}
 		}
 	}
 	return 0;
 }
 
-/* the real work of k-s-stacking frames 
+/* the real work of k-s-stacking frames
  * return -1 for errors */
 static int do_stack_ks(struct image_file_list *imfl, struct ccd_frame *ofr,
 			int (* progress)(char *msg, void *data), void *data)
 {
 	int w, h, i, y, x, n, t;
 	float *dp[COMB_MAX];
+	struct ccd_frame *frames[COMB_MAX];
 	float *odp;
 	int rs[COMB_MAX];
 	GList *gl;
 	struct image_file *imf;
 	char lb[81];
+	int plane_iter = 0;
 
 	w = ofr->w;
 	h = ofr->h;
@@ -945,7 +1044,7 @@ static int do_stack_ks(struct image_file_list *imfl, struct ccd_frame *ofr,
 			err_printf("bad frame size\n");
 			continue;
 		}
-		dp[i] = imf->fr->dat;
+		frames[i] = imf->fr;
 		rs[i] = imf->fr->w - w;
 		i++;
 		if (i >= COMB_MAX) {
@@ -961,7 +1060,7 @@ static int do_stack_ks(struct image_file_list *imfl, struct ccd_frame *ofr,
 	if (progress) {
 		int ret;
 		char msg[64];
-		snprintf(msg, 63, "kappa-sigma s=%.1f iter=%d (%d frames) ", 
+		snprintf(msg, 63, "kappa-sigma s=%.1f iter=%d (%d frames) ",
 			 P_DBL(CCDRED_SIGMAS), P_INT(CCDRED_ITER), i);
 		ret = (* progress)(msg, data);
 		if (ret) {
@@ -975,25 +1074,30 @@ static int do_stack_ks(struct image_file_list *imfl, struct ccd_frame *ofr,
 		 P_DBL(CCDRED_SIGMAS), i);
 	fits_add_history(ofr, lb);
 
-	odp = ofr->dat;
-	for (y = 0; y < h; y++) {
-		if (t-- == 0 && progress) {
-			int ret;
-			ret = (* progress)(".", data);
-			if (ret) {
-				d1_printf("aborted\n");
-					return -1;
+	while ((plane_iter = color_plane_iter(frames[0], plane_iter))) {
+		for (i = 0; i <= n; i++) {
+			dp[i] = get_color_plane(frames[i], plane_iter);
+		}
+		odp = get_color_plane(ofr, plane_iter);
+		for (y = 0; y < h; y++) {
+			if (t-- == 0 && progress) {
+				int ret;
+				ret = (* progress)(".", data);
+				if (ret) {
+					d1_printf("aborted\n");
+						return -1;
+				}
+				t = h / 16;
+			}
+			for (x = 0; x < w; x++) {
+				*odp = pix_ks(dp, n, P_DBL(CCDRED_SIGMAS), P_INT(CCDRED_ITER));
+				for (i = 0; i < n; i++)
+					dp[i]++;
+				odp++;
+			}
+			for (i = 0; i < n; i++) {
+				dp[i] += rs[i];
 			}
-			t = h / 16;
-		}
-		for (x = 0; x < w; x++) {
-			*odp = pix_ks(dp, n, P_DBL(CCDRED_SIGMAS), P_INT(CCDRED_ITER));
-			for (i = 0; i < n; i++) 
-				dp[i]++;
-			odp++;
-		}
-		for (i = 0; i < n; i++) {
-			dp[i] += rs[i];
 		}
 	}
 	if (progress) {
@@ -1002,18 +1106,20 @@ static int do_stack_ks(struct image_file_list *imfl, struct ccd_frame *ofr,
 	return 0;
 }
 
-/* the real work of mean-median-stacking frames 
+/* the real work of mean-median-stacking frames
  * return -1 for errors */
 static int do_stack_mm(struct image_file_list *imfl, struct ccd_frame *ofr,
 			int (* progress)(char *msg, void *data), void *data)
 {
 	int w, h, i, y, x, n, t;
 	float *dp[COMB_MAX];
+	struct ccd_frame *frames[COMB_MAX];
 	float *odp;
 	int rs[COMB_MAX];
 	GList *gl;
 	struct image_file *imf;
 	char lb[81];
+	int plane_iter = 0;
 
 	w = ofr->w;
 	h = ofr->h;
@@ -1029,7 +1135,7 @@ static int do_stack_mm(struct image_file_list *imfl, struct ccd_frame *ofr,
 			err_printf("bad frame size\n");
 			continue;
 		}
-		dp[i] = imf->fr->dat;
+		frames[i] = imf->fr;
 		rs[i] = imf->fr->w - w;
 		i++;
 		if (i >= COMB_MAX) {
@@ -1056,26 +1162,31 @@ static int do_stack_mm(struct image_file_list *imfl, struct ccd_frame *ofr,
 		 P_DBL(CCDRED_SIGMAS), i);
 	fits_add_history(ofr, lb);
 
-	odp = ofr->dat;
-	t = h / 16;
-	for (y = 0; y < h; y++) {
-		if (t-- == 0 && progress) {
-			int ret;
-			ret = (* progress)(".", data);
-			if (ret) {
-				d1_printf("aborted\n");
-					return -1;
+	while ((plane_iter = color_plane_iter(frames[0], plane_iter))) {
+		for (i = 0; i <= n; i++) {
+			dp[i] = get_color_plane(frames[i], plane_iter);
+		}
+		odp = get_color_plane(ofr, plane_iter);
+		t = h / 16;
+		for (y = 0; y < h; y++) {
+			if (t-- == 0 && progress) {
+				int ret;
+				ret = (* progress)(".", data);
+				if (ret) {
+					d1_printf("aborted\n");
+						return -1;
+				}
+				t = h / 16;
+			}
+			for (x = 0; x < w; x++) {
+				*odp = pix_mmedian(dp, n, P_DBL(CCDRED_SIGMAS), P_INT(CCDRED_ITER));
+				for (i = 0; i < n; i++)
+					dp[i]++;
+				odp++;
+			}
+			for (i = 0; i < n; i++) {
+				dp[i] += rs[i];
 			}
-			t = h / 16;
-		}
-		for (x = 0; x < w; x++) {
-			*odp = pix_mmedian(dp, n, P_DBL(CCDRED_SIGMAS), P_INT(CCDRED_ITER));
-			for (i = 0; i < n; i++) 
-				dp[i]++;
-			odp++;
-		}
-		for (i = 0; i < n; i++) {
-			dp[i] += rs[i];
 		}
 	}
 	if (progress) {
@@ -1120,7 +1231,7 @@ static int do_stack_time(struct image_file_list *imfl, struct ccd_frame *ofr,
 			continue;
 		}
 		if (fits_get_double(imf->fr, P_STR(FN_EXPTIME), &v) > 0) {
-			d1_printf("stack time: using exptime = %.3f from %s\n", 
+			d1_printf("stack time: using exptime = %.3f from %s\n",
 				  v, P_STR(FN_EXPTIME));
 		} else {
 			v = 0.0;
@@ -1133,7 +1244,7 @@ static int do_stack_time(struct image_file_list *imfl, struct ccd_frame *ofr,
 		expsum += v;
 		exptime += (jd + v /2/24/3600) * v;
 	}
-	if (expsum == 0) 
+	if (expsum == 0)
 		return 0;
 	if (progress) {
 		snprintf(lb, 79, "eqivalent exp: %.1fs, center at jd:%.7f\n",
@@ -1142,7 +1253,7 @@ static int do_stack_time(struct image_file_list *imfl, struct ccd_frame *ofr,
 	}
 	sprintf(lb, "%20.3f / EXPOSURE TIME IN SECONDS", expsum);
 	fits_add_keyword(ofr, P_STR(FN_EXPTIME), lb);
-	sprintf(lb, "%20.8f / JULIAN DATE OF EXPOSURE START (UTC)", 
+	sprintf(lb, "%20.8f / JULIAN DATE OF EXPOSURE START (UTC)",
 		exptime/expsum - expsum/2/24/3600);
 	fits_add_keyword(ofr, P_STR(FN_JDATE), lb);
 	date_time_from_jdate(exptime/expsum - expsum/2/24/3600, date, 63);
@@ -1215,28 +1326,28 @@ struct ccd_frame * stack_frames(struct image_file_list *imfl, struct ccd_reduce
 		if (do_stack_avg(imfl, ofr, progress, data)) {
 			release_frame(ofr);
 			return NULL;
-		} 
+		}
 		break;
 	case PAR_STACK_METHOD_KAPPA_SIGMA:
 		eff = 0.9;
 		if (do_stack_ks(imfl, ofr, progress, data)) {
 			release_frame(ofr);
 			return NULL;
-		} 
+		}
 		break;
 	case PAR_STACK_METHOD_MEDIAN:
 		eff = 0.65;
 		if (do_stack_median(imfl, ofr, progress, data)) {
 			release_frame(ofr);
 			return NULL;
-		} 
+		}
 		break;
 	case PAR_STACK_METHOD_MEAN_MEDIAN:
 		eff = 0.85;
 		if (do_stack_mm(imfl, ofr, progress, data)) {
 			release_frame(ofr);
 			return NULL;
-		} 
+		}
 		break;
 	default:
 		err_printf("unknown/unsupported stacking method %d\n",
@@ -1258,10 +1369,8 @@ struct ccd_frame * stack_frames(struct image_file_list *imfl, struct ccd_reduce
 
 
 
-/* load the frame specififed in the image file struct into memory and 
- * compute it's stats; return a negative error code (and update the 
-
-error string)
+/* load the frame specififed in the image file struct into memory and
+ * compute it's stats; return a negative error code (and update the error string)
  * if something goes wrong. Return 0 for success. If the file is already loaded, it's not
  * read again */
 int load_image_file(struct image_file *imf)
@@ -1280,9 +1389,9 @@ int load_image_file(struct image_file *imf)
 			return -1;
 		}
 		return 0;
-	} 
-	imf->fr = read_gz_fits_file(imf->filename, P_STR(FILE_UNCOMPRESS),
-				    P_INT(FILE_UNSIGNED_FITS));
+	}
+	imf->fr = read_image_file(imf->filename, P_STR(FILE_UNCOMPRESS), P_INT(FILE_UNSIGNED_FITS),
+			default_cfa[P_INT(FILE_DEFAULT_CFA)]);
 	if (imf->fr == NULL) {
 		err_printf("load_image_file: cannot load %s\n", imf->filename);
 		return -1;
@@ -1290,7 +1399,8 @@ int load_image_file(struct image_file *imf)
 	rescan_fits_exp(imf->fr, &(imf->fr->exp));
 	if (!imf->fr->stats.statsok)
 		frame_stats(imf->fr);
-	imf->flags = IMG_LOADED;
+	imf->flags |= IMG_LOADED;
+
 	return 0;
 }
 
@@ -1318,16 +1428,16 @@ void unload_clean_frames(struct image_file_list *imfl)
 static double star_size_flux(double flux, double ref_flux, double fwhm)
 {
 	double size;
-	size = 1.0 * P_INT(DO_MAX_STAR_SZ) + 2.5 * P_DBL(DO_PIXELS_PER_MAG) 
+	size = 1.0 * P_INT(DO_MAX_STAR_SZ) + 2.5 * P_DBL(DO_PIXELS_PER_MAG)
 		* log10(flux / ref_flux);
-	clamp_double(&size, 1.0 * P_INT(DO_MIN_STAR_SZ), 
+	clamp_double(&size, 1.0 * P_INT(DO_MIN_STAR_SZ),
 		     1.0 * P_INT(DO_MAX_STAR_SZ));
 	return size;
 }
 
 /* detect the stars in frame and return them in a list of gui-stars
  * of "simple" type */
-static GSList *detect_frame_stars(struct ccd_frame *fr) 
+static GSList *detect_frame_stars(struct ccd_frame *fr)
 {
 	struct sources *src;
 	struct gui_star *gs;
@@ -1389,7 +1499,7 @@ void free_alignment_stars(struct ccd_reduce *ccdr)
 	ccdr->ops &= ~CCDR_ALIGN_STARS;
 }
 
-/* search the alignment ref frame for alignment stars and build them in the 
+/* search the alignment ref frame for alignment stars and build them in the
    align_stars list; return the number of stars found */
 int load_alignment_stars(struct ccd_reduce *ccdr)
 {
@@ -1418,7 +1528,7 @@ int load_alignment_stars(struct ccd_reduce *ccdr)
 		}
 		ccdr->ops |= CCDR_ALIGN_STARS;
 	}
-	
+
 	if (ccdr->alignref->fr) {
 		d3_printf("unloading align\n");
 		release_frame(ccdr->alignref->fr);
@@ -1429,7 +1539,7 @@ int load_alignment_stars(struct ccd_reduce *ccdr)
 }
 
 
-int align_imf(struct image_file *imf, struct ccd_reduce *ccdr, 
+int align_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 		   int (* progress)(char *msg, void *data), void *data)
 {
 	char msg[256];
@@ -1487,7 +1597,7 @@ int align_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 	pairs_cs_diff(pairs, &dx, &dy, &ds, &dt, 1, 1);
 
 	if (progress) {
-		snprintf(msg, 255, " [%.1f, %.1f, %.3f, %.2f]", 
+		snprintf(msg, 255, " [%.1f, %.1f, %.3f, %.2f]",
 			 dx, dy, ds, dt);
 		(* progress)(msg, data);
 	}
@@ -1513,7 +1623,7 @@ fexit:
 
 
 
-int aphot_imf(struct image_file *imf, struct ccd_reduce *ccdr, 
+int aphot_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 	      int (* progress)(char *msg, void *data), void *data)
 {
 	char msg[256];
@@ -1549,14 +1659,14 @@ int aphot_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 		match_field_in_window_quiet(window);
 		wcs = gtk_object_get_data(GTK_OBJECT(window), "wcs_of_window");
 	}
-	
+
 	if ((wcs == NULL) || ((wcs->wcsset & WCS_VALID) == 0)) {
 		if (progress) {
 			snprintf(msg, 255, " bad wcs");
 			(* progress)(msg, data);
 		}
 		return -1;
-	} 
+	}
 	gsl = gtk_object_get_data(GTK_OBJECT(window), "gui_star_list");
 	if (gsl == NULL) {
 		if (progress) {
@@ -1564,10 +1674,10 @@ int aphot_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 			(* progress)(msg, data);
 		}
 		return -1;
-	} 
+	}
 
 	stf = run_phot(window, wcs, gsl, fr);
-	if (stf == NULL) 
+	if (stf == NULL)
 		return -1;
 
 
@@ -1580,7 +1690,7 @@ int aphot_imf(struct image_file *imf, struct ccd_reduce *ccdr,
 		d3_printf("mbds has %d frames\n", g_list_length(mbds->ofrs));
 		ofr_fit_zpoint(O_FRAME(mbds->ofrs->data), P_DBL(AP_ALPHA), P_DBL(AP_BETA), 1);
 		ofr_transform_stars(O_FRAME(mbds->ofrs->data), mbds, 0, 0);
-		if (3 * O_FRAME(mbds->ofrs->data)->outliers > 
+		if (3 * O_FRAME(mbds->ofrs->data)->outliers >
 		    O_FRAME(mbds->ofrs->data)->vstars) {
 			info_printf(
 				"\nWarning: Frame has a large number of outliers (more than 1/3\n"
@@ -1716,5 +1826,3 @@ void image_file_list_release(struct image_file_list *imfl)
 	g_list_free(imfl->imlist);
 	free(imfl);
 }
-
-
