@@ -191,6 +191,8 @@ static GtkWidget *make_image_processing(gpointer window)
 			    GTK_SIGNAL_FUNC (imf_red_browse_cb));
 	set_named_callback (GTK_OBJECT (dialog), "flat_browse", "clicked",
 			    GTK_SIGNAL_FUNC (imf_red_browse_cb));
+	set_named_callback (GTK_OBJECT (dialog), "badpix_browse", "clicked",
+			    GTK_SIGNAL_FUNC (imf_red_browse_cb));	
 	set_named_callback (GTK_OBJECT (dialog), "align_browse", "clicked",
 			    GTK_SIGNAL_FUNC (imf_red_browse_cb));
 	set_named_callback (GTK_OBJECT (dialog), "output_file_browse", "clicked",
@@ -215,7 +217,8 @@ static GtkWidget *make_image_processing(gpointer window)
 			    GTK_SIGNAL_FUNC (imf_red_activate_cb));
 	menubar = get_main_menu_bar(dialog);
 	gtk_object_set_data(GTK_OBJECT(dialog), "menubar", menubar);
-	gtk_menu_bar_set_shadow_type(GTK_MENU_BAR(menubar), GTK_SHADOW_NONE);
+	//gtk_menu_bar_set_shadow_type(GTK_MENU_BAR(menubar), GTK_SHADOW_NONE);
+	
 	top_hb = gtk_object_get_data(GTK_OBJECT(dialog), "top_hbox");
 	gtk_box_pack_start(GTK_BOX(top_hb), menubar, TRUE, TRUE, 0);
 	gtk_box_reorder_child(GTK_BOX(top_hb), menubar, 0);
@@ -226,10 +229,16 @@ static GtkWidget *make_image_processing(gpointer window)
 	return dialog;
 }
 
+static void demosaic_method_activate(GtkWidget *wid, gpointer data)
+{
+	P_INT(CCDRED_DEMOSAIC_METHOD) = (long)data;
+	par_touch(CCDRED_DEMOSAIC_METHOD);
+}
+ 
 
 static void stack_method_activate(GtkWidget *wid, gpointer data)
 {
-	P_INT(CCDRED_STACK_METHOD) = (int)data;
+	P_INT(CCDRED_STACK_METHOD) = (long)data;
 	par_touch(CCDRED_STACK_METHOD);
 }
  
@@ -240,7 +249,7 @@ static void set_processing_dialog_ccdr(GtkWidget *dialog, struct ccd_reduce *ccd
 	GtkWidget *menu, *omenu;
 	GtkWidget *menuitem;
 	char **c;
-	int i;
+	long i;
 
 	omenu = gtk_object_get_data(GTK_OBJECT(dialog), "stack_method_optmenu");
 	g_return_if_fail(omenu != NULL);
@@ -255,7 +264,7 @@ static void set_processing_dialog_ccdr(GtkWidget *dialog, struct ccd_reduce *ccd
 		gtk_menu_append (GTK_MENU (menu), menuitem);
 		gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
 			    GTK_SIGNAL_FUNC (stack_method_activate), (gpointer)i);
-		d3_printf("add %s to stak method menu\n", *c);
+		d3_printf("add %s to stack method menu\n", *c);
 		c++;
 		i++;
 	}
@@ -268,6 +277,30 @@ static void set_processing_dialog_ccdr(GtkWidget *dialog, struct ccd_reduce *ccd
 	named_spin_set(dialog, "stack_sigmas_spin", P_DBL(CCDRED_SIGMAS));
 	named_spin_set(dialog, "stack_iter_spin", 1.0 * P_INT(CCDRED_ITER));
 
+	omenu = gtk_object_get_data(GTK_OBJECT(dialog), "demosaic_method_optmenu");
+	g_return_if_fail(omenu != NULL);
+	menu = gtk_menu_new();
+	g_return_if_fail(menu != NULL);
+	
+	c = demosaic_methods;
+	i = 0;
+	while (*c != NULL) {
+		menuitem = gtk_menu_item_new_with_label (*c);
+		gtk_widget_show (menuitem);
+		gtk_menu_append (GTK_MENU (menu), menuitem);
+		gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
+			    GTK_SIGNAL_FUNC (demosaic_method_activate), (gpointer)i);
+		d3_printf("add %s to demosaic method menu\n", *c);
+		c++;
+		i++;
+	}
+	gtk_widget_show(menu);
+	gtk_widget_show(omenu);
+	gtk_option_menu_remove_menu(GTK_OPTION_MENU (omenu));
+	gtk_option_menu_set_menu(GTK_OPTION_MENU (omenu), menu);
+	gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), 
+				     P_INT(CCDRED_DEMOSAIC_METHOD));
+
 	if (ccdr == NULL)
 		return;
 	if ((ccdr->bias) && (ccdr->ops & IMG_OP_BIAS)) {
@@ -282,6 +315,10 @@ static void set_processing_dialog_ccdr(GtkWidget *dialog, struct ccd_reduce *ccd
 		named_entry_set(dialog, "flat_entry", ccdr->flat->filename);
 		set_named_checkb_val(dialog, "flat_checkb", 1);
 	}
+	if ((ccdr->bad_pix_map) && (ccdr->ops & IMG_OP_BADPIX)) {
+		named_entry_set(dialog, "badpix_entry", ccdr->bad_pix_map->filename);
+		set_named_checkb_val(dialog, "badpix_checkb", 1);
+	}
 	if ((ccdr->alignref) && (ccdr->ops & IMG_OP_ALIGN)) {
 		named_entry_set(dialog, "align_entry", ccdr->alignref->filename);
 		set_named_checkb_val(dialog, "align_checkb", 1);
@@ -298,6 +335,9 @@ static void set_processing_dialog_ccdr(GtkWidget *dialog, struct ccd_reduce *ccd
 		named_spin_set(dialog, "mul_spin", ccdr->addv);
 		set_named_checkb_val(dialog, "mul_checkb", 1);
 	}
+	if ((ccdr->ops & IMG_OP_DEMOSAIC)) {
+		set_named_checkb_val(dialog, "demosaic_checkb", 1);
+	}
 	if ((ccdr->ops & IMG_OP_STACK)) {
 		if ((ccdr->ops & IMG_OP_BG_ALIGN_MUL)) {
 			set_named_checkb_val(dialog, "bg_match_mul_rb", 1);
@@ -616,7 +656,7 @@ static void imf_selall_cb(gpointer dialog, guint action, GtkWidget *menu_item)
 	gtk_list_select_all(list);
 }
 
-static void imf_add_files(GList *files, gpointer dialog)
+static void imf_add_files(GSList *files, gpointer dialog)
 {
 	GtkList *list;
 	GtkWidget *item;
@@ -638,7 +678,7 @@ static void imf_add_files(GList *files, gpointer dialog)
 	while (files != NULL) {
 		imf = image_file_new();
 		text = files->data;
-		files = g_list_next(files);
+		files = g_slist_next(files);
 
 		imf->filename = strdup(text);
 		item=gtk_list_item_new_with_label(imf->filename);
@@ -761,6 +801,9 @@ static void imf_update_status_label(GtkWidget *label, struct image_file *imf)
 	if (imf->flags & IMG_OP_FLAT)
 		i += snprintf(msg+i, 127-i, " FLAT");
 	clamp_int(&i, 0, 127);
+	if (imf->flags & IMG_OP_BADPIX)
+		i += snprintf(msg+i, 127-i, " BADPIX");
+	clamp_int(&i, 0, 127);
 	if (imf->flags & IMG_OP_MUL)
 		i += snprintf(msg+i, 127-i, " MUL");
 	clamp_int(&i, 0, 127);
@@ -769,6 +812,10 @@ static void imf_update_status_label(GtkWidget *label, struct image_file *imf)
 	clamp_int(&i, 0, 127);
 	if (imf->flags & IMG_OP_ALIGN)
 		i += snprintf(msg+i, 127-i, " ALIGN");
+	clamp_int(&i, 0, 127);
+	if (imf->flags & IMG_OP_DEMOSAIC)
+		i += snprintf(msg+i, 127-i, " DEMOSAIC");
+	clamp_int(&i, 0, 127);	
 	if (imf->flags & IMG_OP_PHOT)
 		i += snprintf(msg+i, 127-i, " PHOT");
 	gtk_label_set_text(GTK_LABEL(label), msg);
@@ -865,6 +912,11 @@ static void imf_red_browse_cb(GtkWidget *wid, gpointer dialog)
 		g_return_if_fail(entry != NULL);
 		file_select_to_entry(dialog, entry, "Select flat frame", "*", 1);
 	}
+	if (wid == gtk_object_get_data(GTK_OBJECT(dialog), "badpix_browse")) {
+		entry = gtk_object_get_data(GTK_OBJECT(dialog), "badpix_entry");
+		g_return_if_fail(entry != NULL);
+		file_select_to_entry(dialog, entry, "Select bad pixel file", "*", 1);
+	}	
 	if (wid == gtk_object_get_data(GTK_OBJECT(dialog), "align_browse")) {
 		entry = gtk_object_get_data(GTK_OBJECT(dialog), "align_entry");
 		g_return_if_fail(entry != NULL);
@@ -892,9 +944,10 @@ static int progress_pr(char *msg, void *dialog)
 	text = gtk_object_get_data(GTK_OBJECT(dialog), "processing_log_text");
 	g_return_val_if_fail(text != NULL, 0);
 
-	gtk_text_set_line_wrap(GTK_TEXT (text), 0);
-	gtk_text_insert (GTK_TEXT (text), text->style->font, &text->style->black, NULL,
-				 msg, -1);
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW (text), GTK_WRAP_CHAR);
+	gtk_text_buffer_insert_at_cursor(
+		gtk_text_view_get_buffer (GTK_TEXT_VIEW(text)),
+		msg, -1);
 
 	while(gtk_events_pending())
 		gtk_main_iteration();
@@ -913,9 +966,10 @@ static int log_msg(char *msg, void *dialog)
 	text = gtk_object_get_data(GTK_OBJECT(dialog), "processing_log_text");
 	g_return_val_if_fail(text != NULL, 0);
 
-	gtk_text_set_line_wrap(GTK_TEXT (text), 0);
-	gtk_text_insert (GTK_TEXT (text), text->style->font, &text->style->black, NULL,
-				 msg, -1);
+	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_CHAR);
+	gtk_text_buffer_insert_at_cursor(
+		gtk_text_view_get_buffer (GTK_TEXT_VIEW(text)),
+		msg, -1);
 
 	return 0;
 }
@@ -940,10 +994,10 @@ static void dialog_to_ccdr(GtkWidget *dialog, struct ccd_reduce *ccdr)
 		g_free(text);
 		ccdr->ops |= IMG_OP_BIAS;
 	} else {
-		ccdr->ops &= ~IMG_OP_BIAS;
 		if ((ccdr->ops & IMG_OP_BIAS) && ccdr->bias)
 			image_file_release(ccdr->bias);
 		ccdr->bias = NULL;
+		ccdr->ops &= ~IMG_OP_BIAS;
 	}
 
 	if (get_named_checkb_val(dialog, "dark_checkb")) {
@@ -960,10 +1014,10 @@ static void dialog_to_ccdr(GtkWidget *dialog, struct ccd_reduce *ccdr)
 		g_free(text);
 		ccdr->ops |= IMG_OP_DARK;
 	} else {
-		ccdr->ops &= ~IMG_OP_DARK;
 		if ((ccdr->ops & IMG_OP_DARK) && ccdr->dark)
 			image_file_release(ccdr->dark);
 		ccdr->dark = NULL;
+		ccdr->ops &= ~IMG_OP_DARK;		
 	}
 
 	if (get_named_checkb_val(dialog, "flat_checkb")) {
@@ -980,10 +1034,31 @@ static void dialog_to_ccdr(GtkWidget *dialog, struct ccd_reduce *ccdr)
 		g_free(text);
 		ccdr->ops |= IMG_OP_FLAT;
 	} else {
-		ccdr->ops &= ~IMG_OP_FLAT;
 		if ((ccdr->ops & IMG_OP_FLAT) && ccdr->flat)
 			image_file_release(ccdr->flat);
 		ccdr->flat = NULL;
+		ccdr->ops &= ~IMG_OP_FLAT;		
+	}
+
+	if (get_named_checkb_val(dialog, "badpix_checkb")) {
+		text = named_entry_text(dialog, "badpix_entry");
+		if ((ccdr->ops & IMG_OP_BADPIX) && ccdr->bad_pix_map &&
+		    strcmp(text, ccdr->bad_pix_map->filename)) {
+			free_bad_pix(ccdr->bad_pix_map);
+			ccdr->bad_pix_map = calloc(1, sizeof(struct bad_pix_map));
+			ccdr->bad_pix_map->filename = strdup(text);
+		} else if (!(ccdr->ops & IMG_OP_BADPIX)) {
+			ccdr->bad_pix_map = calloc(1, sizeof(struct bad_pix_map));
+			ccdr->bad_pix_map->filename = strdup(text);
+		} 
+		g_free(text);
+		ccdr->ops |= IMG_OP_BADPIX;
+	} else {
+		if ((ccdr->ops & IMG_OP_BADPIX) && ccdr->bad_pix_map) {
+			free_bad_pix(ccdr->bad_pix_map);
+		}
+		ccdr->bad_pix_map = NULL;
+		ccdr->ops &= ~IMG_OP_BADPIX;
 	}
 
 
@@ -1066,6 +1141,12 @@ static void dialog_to_ccdr(GtkWidget *dialog, struct ccd_reduce *ccdr)
 		ccdr->ops &= ~(IMG_OP_PHOT | IMG_OP_PHOT_REUSE_WCS);
 	}
 
+	if (get_named_checkb_val(dialog, "demosaic_checkb")) {
+		ccdr->ops |= IMG_OP_DEMOSAIC;
+	} else {
+		ccdr->ops &= ~IMG_OP_DEMOSAIC;
+	}
+
 	if (get_named_checkb_val(dialog, "stack_checkb")) {
 		ccdr->ops |= IMG_OP_STACK;
 		if (get_named_checkb_val(dialog, "bg_match_add_rb")) {
@@ -1458,6 +1539,9 @@ static void imf_red_activate_cb(GtkWidget *wid, gpointer dialog)
 	if (wid == gtk_object_get_data(GTK_OBJECT(dialog), "align_entry")) {
 		set_named_checkb_val(dialog, "align_checkb", 1);
 	}
+	if (wid == gtk_object_get_data(GTK_OBJECT(dialog), "recipe_entry")) {
+		set_named_checkb_val(dialog, "phot_en_checkb", 1);
+	}
 
 }
 
