@@ -69,6 +69,7 @@
 #include "sourcesdraw.h"
 #include "gui.h"
 #include "wcs.h"
+#include "demosaic.h"
 /*
  * create a geometry map; returns NULL for error
  */
@@ -203,6 +204,7 @@ struct image_channel *new_image_channel(void)
 	}
 	channel->channel_changed = 1;
 	channel->fr = NULL;
+	channel->color = 0;
 	return channel;
 alloc_err:
 	err_printf("new_image_channel: alloc error\n");
@@ -253,6 +255,258 @@ static int area_in_cache(GdkRectangle *area, struct map_cache *cache)
 	return 1;
 }
 
+/* render a float frame to the cache at zoom >= 1*/
+static void cache_render_rgb_float_zi(struct map_cache *cache, struct image_channel *channel, 
+			  int zoom, int fx, int fy, int fw, int fh)
+{
+	struct ccd_frame *fr = channel->fr;
+	int fjump = fr->w - fw;
+	int cjump;
+	int x, y, z;
+	float *fdat_r, *fdat_g, *fdat_b;
+	int lndx;
+	float gain_r = LUT_SIZE / (channel->hcut - channel->lcut);
+	float flr_r = channel->lcut;
+	float gain_g = LUT_SIZE / (channel->hcut - channel->lcut);
+	float flr_g = channel->lcut;
+	float gain_b = LUT_SIZE / (channel->hcut - channel->lcut);
+	float flr_b = channel->lcut;
+
+	unsigned char *cdat = cache->dat;
+	unsigned char pix_r, pix_g, pix_b;
+
+	cache->x = fx * zoom;
+	cache->y = fy * zoom;
+
+	cache->w = fw * zoom;
+	cache->h = fh * zoom;
+
+	cjump = cache->w * 3 - fw * zoom * 3;
+
+	fdat_r = (float *)fr->rdat + fx + fy * fr->w;
+	fdat_g = (float *)fr->gdat + fx + fy * fr->w;
+	fdat_b = (float *)fr->bdat + fx + fy * fr->w;
+
+	if (channel->lut_mode == LUT_MODE_DIRECT) {
+		for (y = 0; y < fh; y++) {
+			for (x = 0; x < fw; x++) {
+				lndx = (*fdat_r);
+				if (lndx < 0)
+					lndx = 0;
+				if (lndx > LUT_SIZE - 1)
+					lndx = LUT_SIZE - 1;
+				pix_r = channel->lut[lndx] >> 8;
+				fdat_r ++;
+
+				lndx = (*fdat_g);
+				if (lndx < 0)
+					lndx = 0;
+				if (lndx > LUT_SIZE - 1)
+					lndx = LUT_SIZE - 1;
+				pix_g = channel->lut[lndx] >> 8;
+				fdat_g ++;
+
+				lndx = (*fdat_b);
+				if (lndx < 0)
+					lndx = 0;
+				if (lndx > LUT_SIZE - 1)
+					lndx = LUT_SIZE - 1;
+				pix_b = channel->lut[lndx] >> 8;
+				fdat_b ++;
+
+				for (z = 0; z < zoom; z++) {
+					*cdat++ = pix_r;
+					*cdat++ = pix_g;
+					*cdat++ = pix_b;
+				}
+			}
+			fdat_r += fjump;
+			fdat_g += fjump;
+			fdat_b += fjump;
+			cdat += cjump;
+			for (z = 0; z < zoom - 1; z++) {
+				memcpy(cdat, cdat - cache->w*3, cache->w*3);
+				cdat += cache->w*3;
+			}
+		}
+	} else {
+		for (y = 0; y < fh; y++) {
+			for (x = 0; x < fw; x++) {
+				lndx = (gain_r * (*fdat_r - flr_r));
+				if (lndx < 0)
+					lndx = 0;
+				if (lndx > LUT_SIZE - 1)
+					lndx = LUT_SIZE - 1;
+				pix_r = channel->lut[lndx] >> 8;
+				fdat_r ++;
+
+				lndx = (gain_g * (*fdat_g - flr_g));
+				if (lndx < 0)
+					lndx = 0;
+				if (lndx > LUT_SIZE - 1)
+					lndx = LUT_SIZE - 1;
+				pix_g = channel->lut[lndx] >> 8;
+				fdat_g ++;
+
+				lndx = (gain_b * (*fdat_b - flr_b));
+				if (lndx < 0)
+					lndx = 0;
+				if (lndx > LUT_SIZE - 1)
+					lndx = LUT_SIZE - 1;
+				pix_b = channel->lut[lndx] >> 8;
+				fdat_b ++;
+
+				for (z = 0; z < zoom; z++) {
+					*cdat++ = pix_r;
+					*cdat++ = pix_g;
+					*cdat++ = pix_b;
+				}
+			}
+			fdat_r += fjump;
+			fdat_g += fjump;
+			fdat_b += fjump;
+			cdat += cjump;
+			for (z = 0; z < zoom - 1; z++) {
+				memcpy(cdat, cdat - cache->w*3, cache->w*3);
+				cdat += cache->w*3;
+			}
+		}
+	}
+	cache->cache_valid = 1;
+//	d3_printf("end of render\n");
+}
+
+static void cache_render_rgb_float_zo(struct map_cache *cache, struct image_channel *channel, 
+			  int zoom, int fx, int fy, int fw, int fh)
+{
+	struct ccd_frame *fr = channel->fr;
+	int fjump = fr->w - fw;
+	int x, y;
+	float *fdat_r, *fdat_g, *fdat_b;
+	float *fd0_r, *fd0_g, *fd0_b;
+	int lndx;
+	float gain_r = LUT_SIZE / (channel->hcut - channel->lcut);
+	float flr_r = channel->lcut;
+	float gain_g = LUT_SIZE / (channel->hcut - channel->lcut);
+	float flr_g = channel->lcut;
+	float gain_b = LUT_SIZE / (channel->hcut - channel->lcut);
+	float flr_b = channel->lcut;
+
+	unsigned char *cdat = cache->dat;
+	unsigned char pix;
+
+	cache->x = fx / zoom;
+	cache->y = fy / zoom;
+
+	cache->w = fw / zoom;
+	cache->h = fh / zoom;
+
+	float avgf = 1.0 / (zoom * zoom);
+	float pixf_r, pixf_g, pixf_b;
+
+	int fj2, xx, yy;
+
+	fdat_r = (float *) fr->rdat + fx + fy * fr->w;
+	fdat_g = (float *) fr->gdat + fx + fy * fr->w;
+	fdat_b = (float *) fr->bdat + fx + fy * fr->w;
+
+	fjump = fr->w - fw + (zoom - 1) * fr->w;
+	fj2 = fr->w - zoom; /* jump from last pixel in zoom row to first one in the next row */
+
+	if (channel->lut_mode == LUT_MODE_DIRECT) {
+		for (y = 0; y < cache->h; y++) {
+			for (x = 0; x < cache->w; x++) {
+				pixf_r = pixf_g = pixf_b = 0.0;
+
+				fd0_r = fdat_r;
+				fd0_g = fdat_g;
+				fd0_b = fdat_b;
+				
+				for (yy = 0; yy < zoom; yy++) {
+					for (xx = 0; xx < zoom; xx++) {
+						pixf_r += *fdat_r++;
+						pixf_g += *fdat_g++;
+						pixf_b += *fdat_b++;
+					}
+
+					fdat_r += fj2;
+					fdat_g += fj2;
+					fdat_b += fj2;
+				}
+				
+				fdat_r = fd0_r + zoom;
+				fdat_g = fd0_g + zoom;
+				fdat_b = fd0_b + zoom;
+				
+				lndx = floor(pixf_r * avgf);
+				clamp_int(&lndx, 0, LUT_SIZE - 1);
+				pix = channel->lut[lndx] >> 8;
+				*cdat++ = pix;
+
+				lndx = floor(pixf_g * avgf);
+				clamp_int(&lndx, 0, LUT_SIZE - 1);
+				pix = channel->lut[lndx] >> 8;
+				*cdat++ = pix;
+
+				lndx = floor(pixf_b * avgf);
+				clamp_int(&lndx, 0, LUT_SIZE - 1);
+				pix = channel->lut[lndx] >> 8;
+				*cdat++ = pix;
+			}
+			
+			fdat_r += fjump;
+			fdat_g += fjump;
+			fdat_b += fjump;			
+		}
+	} else {
+		for (y = 0; y < cache->h; y++) {
+			for (x = 0; x < cache->w; x++) {
+				pixf_r = pixf_g = pixf_b = 0.0;
+
+				fd0_r = fdat_r;
+				fd0_g = fdat_g;
+				fd0_b = fdat_b;
+				
+				for (yy = 0; yy < zoom; yy++) {
+					for (xx = 0; xx < zoom; xx++) {
+						pixf_r += *fdat_r++;
+						pixf_g += *fdat_g++;
+						pixf_b += *fdat_b++;
+					}
+
+					fdat_r += fj2;
+					fdat_g += fj2;
+					fdat_b += fj2;
+				}
+				
+				fdat_r = fd0_r + zoom;
+				fdat_g = fd0_g + zoom;
+				fdat_b = fd0_b + zoom;
+
+				lndx = floor(gain_r * (pixf_r * avgf - flr_r));
+				clamp_int(&lndx, 0, LUT_SIZE - 1);
+				pix = channel->lut[lndx] >> 8;
+				*cdat++ = pix;
+
+				lndx = floor(gain_g * (pixf_g * avgf - flr_g));
+				clamp_int(&lndx, 0, LUT_SIZE - 1);
+				pix = channel->lut[lndx] >> 8;
+				*cdat++ = pix;
+
+				lndx = floor(gain_b * (pixf_b * avgf - flr_b));
+				clamp_int(&lndx, 0, LUT_SIZE - 1);
+				pix = channel->lut[lndx] >> 8;
+				*cdat++ = pix;
+				
+			}
+			fdat_r += fjump;
+			fdat_g += fjump;
+			fdat_b += fjump;			
+		}
+	}
+	cache->cache_valid = 1;
+}			
+
 /* render a float frame to the cache at zoom >= 1*/
 static void cache_render_float_zi(struct map_cache *cache, struct image_channel *channel, 
 			  int zoom, int fx, int fy, int fw, int fh)
@@ -269,6 +523,11 @@ static void cache_render_float_zi(struct map_cache *cache, struct image_channel
 	unsigned char *cdat = cache->dat;
 	unsigned char pix;
 
+	if (channel->color) {
+		cache_render_rgb_float_zi(cache, channel, zoom, fx, fy, fw, fh);
+		return;
+	}
+
 	cache->x = fx * zoom;
 	cache->y = fy * zoom;
 
@@ -335,6 +594,11 @@ static void cache_render_float_zo(struct map_cache *cache, struct image_channel
 	float gain = LUT_SIZE / (channel->hcut - channel->lcut);
 	float flr = channel->lcut;
 
+	if (channel->color) {
+		cache_render_rgb_float_zo(cache, channel, zoom, fx, fy, fw, fh);
+		return;
+	}
+	
 	unsigned char *cdat = cache->dat;
 	unsigned char pix;
 
@@ -472,7 +736,7 @@ static void cache_render_byte_zi(struct map_cache *cache, struct image_channel *
 static void cache_render_byte_zo(struct map_cache *cache, struct image_channel *channel, 
 			  int zoom, int fx, int fy, int fw, int fh)
 {
-	err_printf("float zo:not yet\n");
+	err_printf("byte zo:not yet\n");
 }			
 
 /*
@@ -490,7 +754,7 @@ static unsigned cached_area_size(GdkRectangle *area, struct map_cache *cache)
 /*
  * update the cache so that it contains a representation of the given area
  */
-static void update_gray_cache(struct map_cache *cache, 
+static void update_cache(struct map_cache *cache, 
 		       struct map_geometry *geom, struct image_channel *channel,
 		       GdkRectangle *area)
 {
@@ -498,6 +762,7 @@ static void update_gray_cache(struct map_cache *cache,
 	int fx, fy, fw, fh;
 	int zoom_in = 1;
 	int zoom_out = 1;
+
 	if (geom->zoom > 1.0 && geom->zoom <= 16.0) {
 		zoom_in = floor(geom->zoom + 0.5);
 	} else if (geom->zoom < 1.0 && geom->zoom >= (1.0 / 16.0)) {
@@ -514,7 +779,10 @@ static void update_gray_cache(struct map_cache *cache,
 	}
 	if (cache->dat == NULL) { /* we need to alloc (new) data area */
 		void *dat;
-		cache->type = MAP_CACHE_GRAY;
+		if (channel->color)
+			cache->type = MAP_CACHE_RGB;
+		else
+			cache->type = MAP_CACHE_GRAY;
 		cache->size = cached_area_size(area, cache);
 		dat = malloc(cache->size);
 		if (dat == NULL) {
@@ -593,6 +861,24 @@ void paint_from_gray_cache(GtkWidget *widget, struct map_cache *cache, GdkRectan
 			     GDK_RGB_DITHER_MAX, dat, cache->w);
 }
 
+void paint_from_rgb_cache(GtkWidget *widget, struct map_cache *cache, GdkRectangle *area)
+{
+	unsigned char *dat;
+	if (!area_in_cache(area, cache)) {
+		err_printf("paint_from_cache: oops - area not in cache\n");
+		d3_printf("area is %d by %d starting at %d,%d\n", 
+			  area->width, area->height, area->x, area->y);
+		d3_printf("cache is %d by %d starting at %d,%d\n", 
+			  cache->w, cache->h, cache->x, cache->y);
+//		return;
+	}
+	dat = cache->dat + (area->x - cache->x 
+		+ (area->y - cache->y) * cache->w) * 3;
+	gdk_draw_rgb_image (widget->window, widget->style->fg_gc[GTK_STATE_NORMAL],
+			     area->x, area->y, area->width, area->height,
+			     GDK_RGB_DITHER_MAX, dat, cache->w*3);
+}
+
 
 /*
  * an expose event to our image window 
@@ -626,7 +912,8 @@ gboolean image_expose_cb(GtkWidget *widget, GdkEventExpose *event, gpointer wind
 		return TRUE;
 	geom = ret;
 
-	if (cache->type != MAP_CACHE_GRAY) {
+	if ((!i_channel->color && cache->type != MAP_CACHE_GRAY) ||
+		(i_channel->color && cache->type != MAP_CACHE_RGB)) {
 //		d3_printf("expose: other cache type\n");
 		if (cache->dat)
 			free(cache->dat);
@@ -643,13 +930,16 @@ gboolean image_expose_cb(GtkWidget *widget, GdkEventExpose *event, gpointer wind
 		cache->cache_valid = 0;
 	} 
 	if (!cache->cache_valid) {
-		update_gray_cache(cache, geom, i_channel, &(event->area));
+		update_cache(cache, geom, i_channel, &(event->area));
 	}
 	if (cache->cache_valid) {
 //		d3_printf("cache valid, area is %d by %d starting at %d, %d\n",
 //			  cache->w, cache->h, cache->x, cache->y);
 //		d3_printf("expose: from cache\n");
-		paint_from_gray_cache(widget, cache, &(event->area));
+		if (cache->type == MAP_CACHE_GRAY)
+			paint_from_gray_cache(widget, cache, &(event->area));
+		else 
+			paint_from_rgb_cache(widget, cache, &(event->area));
 	}
 	draw_sources_hook(widget, window, &(event->area));
 	return TRUE;
@@ -678,7 +968,7 @@ void image_box_to_cache(struct map_cache *cache, struct image_channel *channel,
 	area.y = y * zoom_in / zoom_out;
 	area.width = w * zoom_in;
 	area.height = h * zoom_in;
-	update_gray_cache(cache, &geom, channel, &area);
+	update_cache(cache, &geom, channel, &area);
 
 
 }
@@ -705,11 +995,20 @@ int frame_to_channel(struct ccd_frame *fr, GtkWidget *window, char *chname)
 	wcs = gtk_object_get_data(GTK_OBJECT(window), "wcs_of_window");
 /* now update them and create if necessary */
 
+	if ((fr->magic & FRAME_VALID_RGB) == 0)
+		bayer_interpolate(fr);
+
 	if (cache == NULL) {
 		int w=100, h=100, x, y, d;
 		if (window->window)
 			gdk_window_get_geometry(window->window, &x, &y, &w, &h, &d);
-		cache = new_map_cache((w + MAX_ZOOM) * (h + MAX_ZOOM), MAP_CACHE_GRAY);
+
+		if (fr->magic & FRAME_VALID_RGB)
+			cache = new_map_cache((w + MAX_ZOOM) * (h + MAX_ZOOM), 
+					      MAP_CACHE_RGB);
+		else 
+			cache = new_map_cache((w + MAX_ZOOM) * (h + MAX_ZOOM), 
+					      MAP_CACHE_GRAY);
 		gtk_object_set_data_full(GTK_OBJECT(window), "map_cache", 
 					 cache, (GtkDestroyNotify)release_map_cache);
 	}
@@ -727,6 +1026,8 @@ int frame_to_channel(struct ccd_frame *fr, GtkWidget *window, char *chname)
 					 channel, (GtkDestroyNotify)release_image_channel);
 		get_frame(fr);
 		channel->fr = fr;
+		if (fr->magic & FRAME_VALID_RGB)
+			channel->color = 1;
 		if (fr->stats.statsok && fr->pix_format != PIX_BYTE) {
 			set_default_channel_cuts(channel);
 //			set_channel_from_stats(channel);
@@ -735,6 +1036,8 @@ int frame_to_channel(struct ccd_frame *fr, GtkWidget *window, char *chname)
 		release_frame(channel->fr);
 		get_frame(fr);
 		channel->fr = fr;
+		if (fr->magic & FRAME_VALID_RGB)
+			channel->color = 1;
 		if (!fr->stats.statsok)
 			frame_stats(fr);
 		if (fr->stats.statsok && fr->pix_format != PIX_BYTE) {
@@ -791,7 +1094,7 @@ int frame_to_channel(struct ccd_frame *fr, GtkWidget *window, char *chname)
 }
 
 /* write the (float) image as a pnm file */
-static void float_chan_to_pnm(struct image_channel *channel, FILE *pnmf)
+static void float_chan_to_pnm(struct image_channel *channel, FILE *pnmf, int is_16bit)
 {
 	struct ccd_frame *fr = channel->fr;
 	int i;
@@ -813,7 +1116,8 @@ static void float_chan_to_pnm(struct image_channel *channel, FILE *pnmf)
 				lndx = LUT_SIZE - 1;
 			pix = channel->lut[lndx];
 			putc(pix >> 8, pnmf);
-//			putc(pix & 0xff, pnmf);
+			if (is_16bit)
+				putc(pix & 0xff, pnmf);
 			fdat ++;
 		}
 	} else {
@@ -826,13 +1130,14 @@ static void float_chan_to_pnm(struct image_channel *channel, FILE *pnmf)
 			pix = channel->lut[lndx];
 			fdat ++;
 			putc(pix >> 8, pnmf);
-//			putc(pix & 0xff, pnmf);
+			if (is_16bit)
+				putc(pix & 0xff, pnmf);
 		}
 	}
 }
 
 /* write the (byte) image as a pnm file */
-static void byte_chan_to_pnm(struct image_channel *channel, FILE *pnmf)
+static void byte_chan_to_pnm(struct image_channel *channel, FILE *pnmf, int is_16bit)
 {
 	struct ccd_frame *fr = channel->fr;
 	int i;
@@ -854,7 +1159,8 @@ static void byte_chan_to_pnm(struct image_channel *channel, FILE *pnmf)
 				lndx = LUT_SIZE - 1;
 			pix = channel->lut[lndx];
 			putc(pix >> 8, pnmf);
-//			putc(pix & 0xff, pnmf);
+			if (is_16bit)
+				putc(pix & 0xff, pnmf);
 			fdat ++;
 		}
 	} else {
@@ -867,7 +1173,8 @@ static void byte_chan_to_pnm(struct image_channel *channel, FILE *pnmf)
 			pix = channel->lut[lndx];
 			fdat ++;
 			putc(pix >> 8, pnmf);
-//			putc(pix & 0xff, pnmf);
+			if (is_16bit)
+				putc(pix & 0xff, pnmf);
 		}
 	}
 }
@@ -879,7 +1186,7 @@ static void byte_chan_to_pnm(struct image_channel *channel, FILE *pnmf)
  * if fname is null, the file is output on stdout
  * return 0 for success */
 
-int channel_to_pnm_file(struct image_channel *channel, GtkWidget *window, char *fn)
+int channel_to_pnm_file(struct image_channel *channel, GtkWidget *window, char *fn, int is_16bit)
 {
 	int ret=0;
 	FILE *pnmf;
@@ -900,14 +1207,14 @@ int channel_to_pnm_file(struct image_channel *channel, GtkWidget *window, char *
 		pnmf = stdout;
 	}
 
+	fprintf(pnmf, "P5 %d %d %d\n", channel->fr->w, channel->fr->h, is_16bit ? 65535 : 255);
+
 	switch (channel->fr->pix_format) {
 	case PIX_FLOAT :
-		fprintf(pnmf, "P5 %d %d 255\n", channel->fr->w, channel->fr->h);
-		float_chan_to_pnm(channel, pnmf);
+		float_chan_to_pnm(channel, pnmf, is_16bit);
 		break;
 	case PIX_BYTE :
-		fprintf(pnmf, "P5 %d %d 255\n", channel->fr->w, channel->fr->h);
-		byte_chan_to_pnm(channel, pnmf);
+		byte_chan_to_pnm(channel, pnmf, is_16bit);
 		break;
 #ifdef LITTLE_ENDIAN
 	case PIX_16LE :
