@@ -283,7 +283,7 @@ static void draw_star_helper(struct gui_star *gs, GdkWindow *drawable, struct gu
 		    gs->label.label, gs->label.ox, gs->label.oy, font);
 	if ((gs->pair != NULL) && (gs->pair->flags & STAR_HAS_PAIR)) {
 		int pix, piy;
-		char dash_list[]={1, 1};
+		gint8 dash_list[]={1, 1};
 		GdkGC *gc_pline;
 		gc_pline = gdk_gc_new(drawable);
 		gdk_gc_copy(gc_pline, gc);
@@ -347,7 +347,7 @@ void draw_gui_star(struct gui_star *gs, GtkWidget *window)
 	if (gsl == NULL)
 		return;
 	gc = gdk_gc_new(darea->window);
-	draw_star_helper(gs, darea->window, gsl, gc, geom->zoom, darea->style->font);
+	draw_star_helper(gs, darea->window, gsl, gc, geom->zoom, gtk_style_get_font(darea->style));
 	gdk_gc_destroy(gc);
 }
 
@@ -377,7 +377,7 @@ void draw_star_list(GSList *stars, GtkWidget *window)
 	while(sl != NULL) {
 		gs = GUI_STAR(sl->data);
 		sl = g_slist_next(sl);
-		draw_star_helper(gs, darea->window, gsl, gc, geom->zoom, darea->style->font);
+		draw_star_helper(gs, darea->window, gsl, gc, geom->zoom, gtk_style_get_font(darea->style));
 	}
 	gdk_gc_destroy(gc);
 }
@@ -415,7 +415,7 @@ void draw_sources_hook(GtkWidget *darea, GtkWidget *window, GdkRectangle *area)
 		isz = gs->size * geom->zoom + gsl->max_size;
 		if (star_near_area(ix, iy, area, isz)) {
 			draw_star_helper(gs, darea->window, gsl, gc, 
-					 geom->zoom, darea->style->font);
+					 geom->zoom, gtk_style_get_font(darea->style));
 		}
 		if (gs->pair != NULL) {
 			ix = (gs->pair->x + 0.5) * geom->zoom;
@@ -423,7 +423,7 @@ void draw_sources_hook(GtkWidget *darea, GtkWidget *window, GdkRectangle *area)
 			isz = gs->pair->size * geom->zoom + gsl->max_size;
 			if (star_near_area(ix, iy, area, isz)) {
 				draw_star_helper(gs, darea->window, gsl, 
-						 gc, geom->zoom, darea->style->font);
+						 gc, geom->zoom, gtk_style_get_font(darea->style));
 			}
 		}
 	}
