@@ -44,7 +44,6 @@
 #include "params.h"
 #include "interface.h"
 #include "wcs.h"
-#include "camera.h"
 #include "cameragui.h"
 #include "filegui.h"
 #include "symbols.h"
@@ -135,7 +134,7 @@ static GList *read_star_list(GScanner *scan)
 /* create a scanner and tell it about the symbols we look for */
 GScanner* init_scanner()
 {
-	int i;
+	long i;
 	GScanner *scan;
 
 	scan = g_scanner_new(NULL);
