@@ -120,6 +120,7 @@ void gui_star_label_from_cats (struct gui_star *gs)
 {
 	double mag;
 	struct cat_star *cats;
+	int ret = 0;
 
 	cats = CAT_STAR(gs->s);
 	if (gs->s == NULL)
@@ -132,21 +133,23 @@ void gui_star_label_from_cats (struct gui_star *gs)
 
 	if (CATS_TYPE(cats) == CAT_STAR_TYPE_APSTAR &&
 	    P_INT(LABEL_APSTAR)) {
-		asprintf(&gs->label.label, "%s", cats->name);
+		ret = asprintf(&gs->label.label, "%s", cats->name);
 	} else if (CATS_TYPE(cats) == CAT_STAR_TYPE_APSTD &&
 		   P_INT(LABEL_APSTAR)) {
 		if (cats->smags 
 		    && !get_band_by_name(cats->smags, 
 					 P_STR(LABEL_APSTD_BAND), &mag, NULL)) {
-			asprintf(&gs->label.label, "%.0f", mag * 10);
+			ret = asprintf(&gs->label.label, "%.0f", mag * 10);
 		} else {
-			asprintf(&gs->label.label, "~%.0f", cats->mag * 10);
+			ret = asprintf(&gs->label.label, "~%.0f", cats->mag * 10);
 		}
 	} else if (CATS_TYPE(cats) == CAT_STAR_TYPE_SREF) {
 	} else if (CATS_TYPE(cats) == CAT_STAR_TYPE_CAT &&
 		   P_INT(LABEL_CAT)) {
-		asprintf(&gs->label.label, "%s", cats->name);
+		ret = asprintf(&gs->label.label, "%s", cats->name);
 	}
+	if (ret == -1)
+		gs->label.label = NULL;
 }
 
 
@@ -783,7 +786,7 @@ void gui_star_release(struct gui_star *gs)
 				cat_star_release(CAT_STAR(gs->s));
 				break;
 			default:
-				cat_star_release(CAT_STAR(gs->s));
+				release_star(STAR(gs->s));
 				break;
 			}
 		}
